@@ -8,17 +8,17 @@
  *	Based on ff-csv code.
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 
 #include <cell.h>
 #include <sheet.h>
 #include <value.h>
-#include <io-context.h>
+#include <goffice/app/io-context.h>
 #include <workbook-view.h>
 #include <workbook.h>
-#include <plugin-util.h>
-#include <module-plugin-defs.h>
+#include <gnm-plugin.h>
+#include <goffice/app/error-info.h>
 
 #include <gsf/gsf-input-textline.h>
 #include <gsf/gsf-output.h>
@@ -26,13 +26,13 @@
 #include <string.h>
 #include <stdlib.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
 #define N_INPUT_LINES_BETWEEN_UPDATES   50
 
-void dif_file_open (GnmFileOpener const *fo, IOContext *io_context,
+void dif_file_open (GOFileOpener const *fo, IOContext *io_context,
                     WorkbookView *wbv, GsfInput *input);
-void dif_file_save (GnmFileSaver const *fs, IOContext *io_context,
+void dif_file_save (GOFileSaver const *fs, IOContext *io_context,
                     WorkbookView const *wbv, GsfOutput *output);
 
 typedef struct {
@@ -60,7 +60,7 @@ dif_input_context_new (IOContext *io_context, Workbook *wb, GsfInput *input)
 
 	ctxt->line_no        = 1;
 	ctxt->line           = NULL;
-	ctxt->sheet          = workbook_sheet_add (wb, NULL, FALSE);
+	ctxt->sheet          = workbook_sheet_add (wb, -1);
 	ctxt->converter      = g_iconv_open ("UTF-8", "ISO-8859-1");
 
 	io_progress_message (io_context, _("Reading file..."));
@@ -251,14 +251,14 @@ dif_parse_sheet (DifInputContext *ctxt)
 }
 
 void
-dif_file_open (GnmFileOpener const *fo, IOContext *io_context,
+dif_file_open (GOFileOpener const *fo, IOContext *io_context,
                WorkbookView *wbv, GsfInput *input)
 {
-	Workbook *wb = wb_view_workbook (wbv);
+	Workbook *wb = wb_view_get_workbook (wbv);
 	DifInputContext *ctxt = dif_input_context_new (io_context, wb, input);
 
 	workbook_set_saveinfo (wb, FILE_FL_MANUAL_REMEMBER,
-		gnm_file_saver_for_id ("Gnumeric_dif:dif"));
+		go_file_saver_for_id ("Gnumeric_dif:dif"));
 	if (ctxt != NULL) {
 		dif_parse_sheet (ctxt);
 		if (gnumeric_io_error_occurred (io_context))
@@ -273,7 +273,7 @@ dif_file_open (GnmFileOpener const *fo, IOContext *io_context,
  * Write _current_ sheet of the workbook to a DIF format file
  */
 void
-dif_file_save (GnmFileSaver const *fs, IOContext *io_context,
+dif_file_save (GOFileSaver const *fs, IOContext *io_context,
                WorkbookView const *wbv, GsfOutput *output)
 {
 	Sheet *sheet;
