@@ -50,7 +50,7 @@
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-input.h>
 #include <gmodule.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
