@@ -7,7 +7,7 @@
  * Copyright (C) 2001 Adrian Custer, Berkeley
  * email: acuster@nature.berkeley.edu
  *
- * Copyright (C) 2001 Andreas J. Guelzow, Edmonton
+ * Copyright (C) 2001-2006 Andreas J. Guelzow, Edmonton
  * email: aguelzow@taliesin.ca
  *
  * This program is free software; you can redistribute it and/or modify
@@ -131,21 +131,65 @@ static latex_border_connectors_t const conn_styles[LATEX_MAX_BORDER]
 	  {{":b:",""}, { ":b:",""}, { ":",":"}}}}
 };
 
+/**
+ * latex_raw_str :
+ * @p :	    a pointer to a char, start of the string to be processed
+ * @output : output stream where the processed characters are written.
+ * @utf8:   is this a utf8 string?
+ *
+ * @return:
+ * If @p is in form of \L{foo}, return the char pointer pointing to '}' of \L{foo}
+ * else return @p untouched; 
+ * 
+ * Check if @p is in form of \L{foo}.
+ * If it is, the exact "foo" will be put into @output, without any esacaping.
+ *
+ */
+static char const *
+latex_raw_str(char const *p, GsfOutput *output, gboolean utf8)
+{
+	char const *p_begin, *p_orig = p;
+	int depth = 1;
+	if(strncasecmp(p, "\\L{", 3) == 0){
+		p += 3;
+		p_begin = p;
+		/* find the matching close bracket */
+		for(; *p; p = utf8 ? g_utf8_next_char(p) : p + 1){
+			switch(*p){ /* FIXME: how to put in unmatched brackets? */
+				case '{':
+					depth ++;
+					break;
+				case '}':
+					depth--;
+					if(depth == 0){
+						/* put the string beginning from p_begin to p to output */
+						gsf_output_write(output, p - p_begin, p_begin);
+						return p;
+					}
+			}
+		}
+	}
+	return p_orig;
+}
+
+
 /**
  * latex_fputs_utf :
  *
  * @p :      a pointer to a char, start of the string to be processed.
  * @output : output stream where the processed characters are written.
  *
- * This escapes any special LaTeX characters from the LaTeX engine. Re-ordered
- * from Rasca's code to have most common first.
+ * This escapes any special LaTeX characters from the LaTeX engine,
+ * except the ones enclosed in "\L{" and "}".
+ * Re-ordered from Rasca's code to have most common first.
  */
 static void
 latex_fputs_utf (char const *p, GsfOutput *output)
 {
+	char const *rlt;
 	for (; *p; p = g_utf8_next_char (p)) {
 		switch (g_utf8_get_char (p)) {
-			
+
 			/* These are the classic TeX symbols $ & % # _ { } (see Lamport, p.15) */
 		case '$': case '&': case '%': case '#':
 		case '_': case '{': case '}':
@@ -156,7 +200,11 @@ latex_fputs_utf (char const *p, GsfOutput *output)
 			gsf_output_printf (output, "\\%c{ }", *p);
 			break;
 		case '\\':
-			gsf_output_puts (output, "$\\backslash$");
+			rlt = latex_raw_str(p, output, TRUE);
+			if(rlt == p)
+			    gsf_output_puts (output, "$\\backslash$");
+			else
+			    p = rlt;
 			break;
 			/* Are these available only in LaTeX through mathmode? */
 		case '>': case '<':
@@ -177,13 +225,15 @@ latex_fputs_utf (char const *p, GsfOutput *output)
  * @p :     a pointer to a char, start of the string to be processed.
  * @output: output stream where the processed characters are written.
  *
- * This escapes any special LaTeX characters from the LaTeX engine.
+ * This escapes any special LaTeX characters from the LaTeX engine,
+ * except the ones enclosed in "\L{" and "}".
  * 
  * We assume that htis will be set in Mathematics mode.
  */
 static void
 latex_math_fputs_utf (char const *p, GsfOutput *output)
 {
+	char const *rlt;
 	for (; *p; p = g_utf8_next_char (p)) {
 		switch (g_utf8_get_char (p)) {
 
@@ -196,9 +246,12 @@ latex_math_fputs_utf (char const *p, GsfOutput *output)
 				gsf_output_printf (output, "\\%c{ }", *p);
 				break;
 			case '\\':
-				gsf_output_puts (output, "\\backslash");
+				rlt = latex_raw_str(p, output, TRUE);
+				if(rlt == p)
+				    gsf_output_puts (output, "$\\backslash$");
+				else
+				    p = rlt;
 				break;
-
 			default:
 				gsf_output_write (output, 
 						  (g_utf8_next_char (p)) - p, p);
@@ -260,14 +313,16 @@ latex_convert_latin_to_utf (char const *text)
  * @p :      a pointer to a char, start of the string to be processed.
  * @output : output stream where the processed characters are written.
  *
- * This escapes any special LaTeX characters from the LaTeX engine. Re-ordered
- * from Rasca's code to have most common first.
+ * This escapes any special LaTeX characters from the LaTeX engine,
+ * except the ones enclosed in "\L{" and "}".
+ * Re-ordered from Rasca's code to have most common first.
  */
 static void
 latex_fputs_latin (char const *text, GsfOutput *output)
 {
 	char * encoded_text = NULL;
-	char * p;
+	char const *p;
+	char const *rlt;
 
 	encoded_text = latex_convert_latin_to_utf (text);
 
@@ -284,7 +339,11 @@ latex_fputs_latin (char const *text, GsfOutput *output)
 			gsf_output_printf (output, "\\%c{ }", *p);
 			break;
 		case '\\':
-			gsf_output_puts (output, "$\\backslash$");
+			rlt = latex_raw_str(p, output, FALSE);
+			if(rlt == p)
+			    gsf_output_puts (output, "$\\backslash$");
+			else
+			    p = rlt;
 			break;
 			/* Are these available only in LaTeX through mathmode? */
 		case '>': case '<': case '':
@@ -305,7 +364,8 @@ latex_fputs_latin (char const *text, GsfOutput *output)
  * @p :     a pointer to a char, start of the string to be processed.
  * @output: output stream where the processed characters are written.
  *
- * This escapes any special LaTeX characters from the LaTeX engine.
+ * This escapes any special LaTeX characters from the LaTeX engine, 
+ * except the ones enclosed in "\L{" and "}".
  * 
  * We assume that htis will be set in Mathematics mode.
  */
@@ -313,7 +373,8 @@ static void
 latex_math_fputs_latin (char const *text, GsfOutput *output)
 {
 	char * encoded_text = NULL;
-	char * p;
+	char const *p;
+	char const *rlt;
 
 	encoded_text = latex_convert_latin_to_utf (text);
 
@@ -329,7 +390,11 @@ latex_math_fputs_latin (char const *text, GsfOutput *output)
 				gsf_output_printf (output, "\\%c{ }", *p);
 				break;
 			case '\\':
-				gsf_output_puts (output, "\\backslash");
+				rlt = latex_raw_str(p, output, FALSE);
+				if(rlt == p)
+				    gsf_output_puts (output, "$\\backslash$");
+				else
+				    p = rlt;
 				break;
 
 			default:
@@ -636,12 +701,13 @@ latex2e_print_vert_border (GsfOutput *output, StyleBorderType style)
 }
 
 /**
- * latex2e_write_blank_cell:
+ * latex2e_write_blank_multicolumn_cell:
  *
  * @output : output stream where the cell contents will be written.
- * @col :
- * @row :
- * @first_column :
+ * @star_col :
+ * @start_row :
+ * @num_merged_cols : an integer value of the number of columns to merge.
+ * @num_merged_rows : an integer value of the number of rows to merge.
  * @sheet :  the current sheet.
  *
  * This function creates all the LaTeX code for the cell of a table (i.e. all
@@ -651,24 +717,58 @@ latex2e_print_vert_border (GsfOutput *output, StyleBorderType style)
  *
  */
 static void
-latex2e_write_blank_cell (GsfOutput *output, gint col, gint row, gint index,
-			  StyleBorderType *borders, Sheet *sheet)
+latex2e_write_blank_multicolumn_cell (GsfOutput *output, int start_col, int start_row,
+				      int num_merged_cols, int num_merged_rows,
+				      gint index,
+				      StyleBorderType *borders, Sheet *sheet)
 {
-	GnmCellPos pos;
+	int merge_width = 0;
 	StyleBorderType left_border = STYLE_BORDER_NONE;
 	StyleBorderType right_border = STYLE_BORDER_NONE;
 
-	pos.col = col;
-	pos.row = row;
+	if (num_merged_cols > 1 || num_merged_rows > 1) {
+		ColRowInfo const * ci;
+		int i;
+
+		for (i = 0; i < num_merged_cols; i++) {
+			ci = sheet_col_get_info (sheet, start_col + i);
+			merge_width += ci->size_pixels;
+		}
+	}
 
 	if (index == 0) {
 		left_border = *borders;
 	}
-	right_border = borders[index + 1];
+	right_border = borders[index + num_merged_cols];
+
+	/* We only set up a multicolumn command if necessary */
+	if (num_merged_cols > 1) {
+		int i;
+
+		/* Open the multicolumn statement. */
+		gsf_output_printf (output, "\\multicolumn{%d}{", num_merged_cols);
+
+		if (left_border != STYLE_BORDER_NONE)
+			latex2e_print_vert_border (output, left_border);
+
+		if (num_merged_rows > 1) {
+			gsf_output_printf (output, "c");
+		} else {
+			gsf_output_printf (output, "p{");
+			for (i = 0; i < num_merged_cols; i++) {
+				gsf_output_printf (output, "\t\\gnumericCol%s+%%\n",
+					 col_name (start_col + i));
+			}
+			gsf_output_printf (output, "\t\\tabcolsep*2*%i}", num_merged_cols - 1);
+		}
+
+		if (right_border != STYLE_BORDER_NONE)
+			latex2e_print_vert_border (output, right_border);
+
+		/*Close the right delimiter, as above. Also open the text delimiter.*/
+		gsf_output_printf (output,"}%%\n\t{");
+	} else if (left_border != STYLE_BORDER_NONE || right_border != STYLE_BORDER_NONE) {
 
-	if (left_border == STYLE_BORDER_NONE && right_border == STYLE_BORDER_NONE)
-		gsf_output_printf (output, "\n");
-	else {
 		/* Open the multicolumn statement. */
 		gsf_output_printf (output, "\\multicolumn{1}{");
 
@@ -676,14 +776,40 @@ latex2e_write_blank_cell (GsfOutput *output, gint col, gint row, gint index,
 			latex2e_print_vert_border (output, left_border);
 
 		/* Drop in the left hand format delimiter. */
-		gsf_output_printf (output, "c");
+		gsf_output_printf (output, "p{\\gnumericCol%s}", col_name(start_col));
 
 		if (right_border != STYLE_BORDER_NONE)
 			latex2e_print_vert_border (output, right_border);
 
-		/*Close the right delimiter, as above. Also add the empty text delimiters.*/
-		gsf_output_printf (output,"}{}%%\n");
+		/*Close the right delimiter, as above. Also open the text delimiter.*/
+		gsf_output_printf (output,"}%%\n\t{");
+
 	}
+
+	if (num_merged_rows > 1) {
+		int i;
+		/* Open the multirow statement. */
+		gsf_output_printf (output, "\\multirow{%d}[%i]*{\\begin{tabular}{p{",
+			 num_merged_rows, num_merged_rows/2);
+		for (i = 0; i < num_merged_cols; i++) {
+			gsf_output_printf (output, "\t\\gnumericCol%s+%%\n", col_name (start_col + i));
+		}
+		if (num_merged_cols > 2)
+			gsf_output_printf (output, "\t\\tabcolsep*2*%i}}", num_merged_cols - 2);
+		else
+			gsf_output_printf (output, "\t0pt}}");
+		/* Close the multirowtext. */
+		gsf_output_printf (output, "\\end{tabular}}");
+	}
+
+	/* Close the multicolumn text bracket. */
+	if (num_merged_cols > 1 || left_border != STYLE_BORDER_NONE
+	    || right_border != STYLE_BORDER_NONE)
+		gsf_output_printf (output, "}");
+
+	/* And we are done. */
+	gsf_output_printf (output, "\n");
+
 }
 
 
@@ -692,6 +818,7 @@ latex2e_write_blank_cell (GsfOutput *output, gint col, gint row, gint index,
  *
  * @output : output stream where the cell contents will be written.
  * @cell :   the cell whose contents are to be written.
+ * @star_col :
  * @num_merged_cols : an integer value of the number of columns to merge.
  * @num_merged_rows : an integer value of the number of rows to merge.
  * @sheet :  the current sheet.
@@ -772,7 +899,7 @@ latex2e_write_multicolumn_cell (GsfOutput *output, GnmCell *cell, int start_col,
 			latex2e_print_vert_border (output, left_border);
 
 		/* Drop in the left hand format delimiter. */
-		gsf_output_printf (output, "p{\\gnumericCol%s}", col_name(cell->pos.col));
+		gsf_output_printf (output, "p{\\gnumericCol%s}", col_name(start_col));
 
 		if (right_border != STYLE_BORDER_NONE)
 			latex2e_print_vert_border (output, right_border);
@@ -1080,8 +1207,8 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 	gsf_output_puts (output, ""
 "\\setlength\\gnumericTableWidthComplete{\\gnumericTableWidth+%\n"
 "         \\tabcolsep*\\gumericNumCols*2+\\arrayrulewidth*\\gumericNumCols}\n"
-"\\ifthenelse{\\lengthtest{\\gnumericTableWidthComplete > \\textwidth}}%\n"
-"         {\\def\\gnumericScale{\\ratio{\\textwidth-%\n"
+"\\ifthenelse{\\lengthtest{\\gnumericTableWidthComplete > \\linewidth}}%\n"
+"         {\\def\\gnumericScale{\\ratio{\\linewidth-%\n"
 "                        \\tabcolsep*\\gumericNumCols*2-%\n"
 "                        \\arrayrulewidth*\\gumericNumCols}%\n"
 "{\\gnumericTableWidth}}}%\n"
@@ -1120,7 +1247,7 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 		ColRowInfo const * ri;
 		ri = sheet_row_get_info (current_sheet, row);
 		if (ri->needs_respan)
-			row_calc_spans ((ColRowInfo *) ri, current_sheet);
+			row_calc_spans ((ColRowInfo *) ri, row, current_sheet);
 
 		/* We need to check for horizontal borders at the top of this row */
 		length = num_cols;
@@ -1167,8 +1294,11 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 		g_free (clines);
 
 		for (col = total_range.start.col; col <= total_range.end.col; col++) {
-			CellSpanInfo const *the_span;
+		        GnmCellPos pos;
 
+			pos.col = col;
+			pos.row = row;
+			
 			/* Get the cell. */
 			cell = sheet_cell_get (current_sheet, col, row);
 
@@ -1178,33 +1308,17 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 			else
 				gsf_output_printf (output, "\t ");
 
-			/* Even an empty cell (especially an empty cell!) can be */
-			/* covered by a span!                                    */
-			the_span = row_span_get (ri, col);
-			if (the_span != NULL) {
-				latex2e_write_multicolumn_cell(output, (GnmCell *)the_span->cell,
-							       col,
-							       the_span->right -
-							       col + 1, 1,
+			/* Check a merge. */
+			merge_range = sheet_merge_is_corner (current_sheet, &pos);
+			if (merge_range == NULL) {
+			        if (cell_is_empty(cell))
+				        latex2e_write_blank_multicolumn_cell(output, col, row,
+									     1, 1,
 							       col - total_range.start.col,
 							       next_vert, current_sheet);
-				col += the_span->right - col;
-				continue;
-			}
-
-
-			/* A blank cell has only a few options*/
-			if (cell_is_empty(cell)) {
-				latex2e_write_blank_cell(output, col, row,
-							col - total_range.start.col,
-							next_vert, current_sheet);
-				continue;
-			}
-
-			/* Check a merge. */
-			merge_range = sheet_merge_is_corner (current_sheet, &cell->pos);
-			if (merge_range == NULL) {
-				latex2e_write_multicolumn_cell(output, cell, col, 1, 1,
+				else
+				        latex2e_write_multicolumn_cell(output, cell, col,
+								       1, 1,
 							       col - total_range.start.col,
 							       next_vert, current_sheet);
 				continue;
@@ -1214,10 +1328,17 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 			num_merged_cols = merge_range->end.col - merge_range->start.col + 1;
 			num_merged_rows = merge_range->end.row - merge_range->start.row + 1;
 
-			latex2e_write_multicolumn_cell(output, cell, col, num_merged_cols,
-						       num_merged_rows,
-						       col - total_range.start.col,
-						       next_vert, current_sheet);
+			if (cell_is_empty(cell))
+				latex2e_write_blank_multicolumn_cell(output, col, row, 
+								     num_merged_cols,
+								     num_merged_rows,
+								     col - total_range.start.col,
+								     next_vert, current_sheet);
+			else
+				latex2e_write_multicolumn_cell(output, cell, col, num_merged_cols,
+							       num_merged_rows,
+							       col - total_range.start.col,
+							       next_vert, current_sheet);
 			col += (num_merged_cols - 1);
 			continue;
 		}
@@ -1257,3 +1378,104 @@ latex_file_save (GOFileSaver const *fs, IOContext *io_context,
 	gsf_output_printf (output, "\\end{longtable}\n\n");
 	gsf_output_printf (output, "\\gnumericTableEnd\n");
 }
+
+
+/**
+ * latex2e_table_cell:
+ *
+ * @output : output stream where the cell contents will be written.
+ * @cell :   the cell whose contents are to be written.
+ * @sheet :  the current sheet.
+ *
+ * This function creates all the LaTeX code for the cell of a table (i.e. all
+ * the code that might fall between two ampersands (&)).
+ *
+ */
+static void
+latex2e_table_write_cell (GsfOutput *output, GnmCell *cell, Sheet *sheet)
+{
+	GnmStyle const *style = cell_get_style (cell);
+
+	if (gnm_style_get_contents_hidden (style))
+		return;
+
+	if (!cell_is_empty (cell)) {
+		char * rendered_string;
+
+		rendered_string = cell_get_rendered_text (cell);
+		latex_fputs (rendered_string, output);
+		g_free (rendered_string);
+	}
+}
+
+
+/**
+ * latex2e_table_write_file_header:
+ *
+ * @output: Output stream where the cell contents will be written.
+ *
+ * This ouputs the LaTeX header. Kept separate for esthetics.
+ */
+
+static void
+latex2e_table_write_file_header(GsfOutput *output)
+{
+	gsf_output_puts (output,
+"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
+"%%                                                                  %%\n"
+"%%  This is a LaTeX2e table fragment exported from Gnumeric.        %%\n"
+"%%                                                                  %%\n"
+"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
+        );
+}
+
+/**
+ * latex_table_file_save :  The LaTeX2e exporter plugin function.
+ *
+ * @FileSaver :        New structure for file plugins. I don't understand.
+ * @IOcontext :        currently not used but reserved for the future.
+ * @WorkbookView :     this provides the way to access the sheet being exported.
+ * @filename :         where we'll write.
+ *
+ * This writes the top sheet of a Gnumeric workbook as the content of a latex table environment.
+ * We try to avoid all formatting.
+ */
+void
+latex_table_file_save (GOFileSaver const *fs, IOContext *io_context,
+		 WorkbookView const *wb_view, GsfOutput *output)
+{
+	GnmCell *cell;
+	Sheet *current_sheet;
+	GnmRange total_range;
+	int row, col;
+
+	/* This is the preamble of the LaTeX2e file. */
+	latex2e_table_write_file_header(output);
+
+	/* Get the topmost sheet and its range from the plugin function argument. */
+	current_sheet = wb_view_cur_sheet(wb_view);
+	total_range = sheet_get_extent (current_sheet, TRUE);
+
+	/* Step through the sheet, writing cells as appropriate. */
+	for (row = total_range.start.row; row <= total_range.end.row; row++) {
+		ColRowInfo const * ri;
+		ri = sheet_row_get_info (current_sheet, row);
+		if (ri->needs_respan)
+			row_calc_spans ((ColRowInfo *) ri, row, current_sheet);
+
+		for (col = total_range.start.col; col <= total_range.end.col; col++) {
+			/* Get the cell. */
+			cell = sheet_cell_get (current_sheet, col, row);
+
+			/* Check if we are not the first cell in the row.*/
+			if (col != total_range.start.col)
+				gsf_output_printf (output, "\t&");
+
+			if (cell_is_empty (cell))
+				continue;
+
+			latex2e_table_write_cell(output, cell, current_sheet);
+		}
+		gsf_output_printf (output, "\\\\\n");
+	}
+}
