@@ -7,12 +7,11 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
-#include <plugin.h>
-#include <plugin-util.h>
-#include <module-plugin-defs.h>
-#include <io-context.h>
+#include <goffice/app/go-plugin.h>
+#include <gnm-plugin.h>
+#include <goffice/app/io-context.h>
 #include <sheet.h>
 #include <ranges.h>
 #include <value.h>
@@ -24,18 +23,19 @@
 #include <sheet-style.h>
 #include <style.h>
 #include <mstyle.h>
-#include <gutils.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/app/error-info.h>
 
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
 #include <string.h>
 #include <math.h>
 
-GNUMERIC_MODULE_PLUGIN_INFO_DECL;
+GNM_PLUGIN_MODULE_HEADER;
 
-gboolean pln_file_probe (GnmFileOpener const *fo, GsfInput *input,
+gboolean pln_file_probe (GOFileOpener const *fo, GsfInput *input,
 			 FileProbeLevel pl);
-void     pln_file_open (GnmFileOpener const *fo, IOContext *io_context,
+void     pln_file_open (GOFileOpener const *fo, IOContext *io_context,
 			WorkbookView *wb_view, GsfInput *input);
 
 static char const *formula1[] = {
@@ -62,7 +62,7 @@ static char const *formula1[] = {
 	"COLUMN("			/* 20 */
 };
 
-static const char* formula2[] =
+static char const * formula2[] =
 {
 	"?bad1?(",		/* 0 */
 	"POWER(",
@@ -173,7 +173,7 @@ pln_get_func_table2 (unsigned i)
 }
 
 gboolean
-pln_file_probe (GnmFileOpener const *fo, GsfInput *input,
+pln_file_probe (GOFileOpener const *fo, GsfInput *input,
 		FileProbeLevel pl)
 {
 	/*
@@ -211,20 +211,21 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 		GnmStyle *def = sheet_style_default (state->sheet);
 		if ((attr & 0x0700) == 0x0400) {
 			attr &= 0xf8ff;
-			switch (mstyle_get_align_h (def)) {
+			switch (gnm_style_get_align_h (def)) {
 			default :
 			case HALIGN_GENERAL:break;
 			case HALIGN_LEFT:	attr |= 0x0100; break;
 			case HALIGN_RIGHT:	attr |= 0x0200; break;
+			case HALIGN_DISTRIBUTED:
 			case HALIGN_CENTER_ACROSS_SELECTION :
 			case HALIGN_CENTER:	attr |= 0x0300; break;
 			}
 		}
 		if ((attr & 0x8000)) {
-			gboolean is_locked = mstyle_get_content_locked (def);
+			gboolean is_locked = gnm_style_get_contents_locked (def);
 			attr = (attr & 0x3fff) | (is_locked ? 0x4000 : 0);
 		}
-		mstyle_unref (def);
+		gnm_style_unref (def);
 	}
 
 	/* bit bash a key containing all relevant info */
@@ -234,23 +235,23 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 
 	res = g_hash_table_lookup (state->styles, GINT_TO_POINTER (key));
 	if (res == NULL) {
-		static StyleHAlignFlags const haligns[] = {
+		static GnmHAlign const haligns[] = {
 			HALIGN_GENERAL, HALIGN_LEFT, HALIGN_RIGHT, HALIGN_CENTER
 		};
-		res = mstyle_new_default ();
-		mstyle_set_font_italic (res, (attr & 0x0010) ? TRUE : FALSE);
-		mstyle_set_content_hidden (res, (attr & 0x0020) ? TRUE : FALSE);
-		mstyle_set_font_uline (res,
+		res = gnm_style_new_default ();
+		gnm_style_set_font_italic (res, (attr & 0x0010) ? TRUE : FALSE);
+		gnm_style_set_contents_hidden (res, (attr & 0x0020) ? TRUE : FALSE);
+		gnm_style_set_font_uline (res,
 			(attr & 0x1000) ?  UNDERLINE_DOUBLE :
 			((attr & 0x0040) ?  UNDERLINE_SINGLE : UNDERLINE_NONE));
-		mstyle_set_font_bold (res, (attr & 0x0080) ? TRUE : FALSE);
-		mstyle_set_align_h (res, haligns [(attr & 0x300) >> 8]);
+		gnm_style_set_font_bold (res, (attr & 0x0080) ? TRUE : FALSE);
+		gnm_style_set_align_h (res, haligns [(attr & 0x300) >> 8]);
 
 		g_hash_table_insert (state->styles, GINT_TO_POINTER (key), res);
 #warning generate formats
 	}
 
-	mstyle_ref (res);
+	gnm_style_ref (res);
 	return res;
 }
 
@@ -269,7 +270,7 @@ pln_get_number (guint8 const * ch)
 	}
 	if (exp & 128)
 		dvalue = -dvalue;
-	dvalue = ldexpgnum (dvalue, ((exp & 127) - 64) * 4);
+	dvalue = gnm_ldexp (dvalue, ((exp & 127) - 64) * 4);
 
 	return dvalue;
 }
@@ -337,7 +338,7 @@ pln_convert_expr (GnmParsePos const *pp, guint8 const *ch)
 			len = *ch;
 			str = pln_get_str (ch + 1, len);
 			g_string_append_c (expr, '\"');
-			gnm_strescape (expr, str);
+			go_strescape (expr, str);
 			g_string_append_c (expr, '\"');
 			ch += len + 1;
 			g_free (str);
@@ -489,9 +490,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 	int max_row = SHEET_MAX_ROWS;
 	int i, rcode, rlength;
 	guint8 const *data;
-	GnmCell    *cell;
 	GnmValue   *v;
-	GnmExpr const *expr;
 	GnmStyle  *style;
 	GnmParsePos pp;
 	GnmRange r;
@@ -557,6 +556,8 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 
 	/* process the CELL information */
 	while (NULL != (data = gsf_input_read (input, 20, NULL))) {
+		GnmExprTop const *texpr = NULL;
+		GnmCell *cell = NULL;
 		unsigned type = GSF_LE_GET_GUINT16 (data + 12);
 		unsigned length = GSF_LE_GET_GUINT16 (data + 18);
 
@@ -576,8 +577,6 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 				pp.eval.col, max_col);
 
 		v = NULL;
-		expr = NULL;
-		cell = NULL;
 		if ((type & 0x7) != 0) {
 			style = pln_get_style (state, data, TRUE);
 			if (style != NULL)
@@ -626,11 +625,11 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 				char *expr_txt = pln_convert_expr (&pp, data);
 
 				if (expr_txt != NULL) {
-					expr = gnm_expr_parse_str (expr_txt, &pp,
+					texpr = gnm_expr_parse_str (expr_txt, &pp,
 								   GNM_EXPR_PARSE_DEFAULT,
 								   gnm_expr_conventions_default,
 								   NULL);
-					if (expr == NULL) {
+					if (texpr == NULL) {
 						if (v != NULL)
 							value_release (v);
 						v = value_new_string_nocopy (expr_txt);
@@ -640,12 +639,12 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 			}
 		}
 
-		if (expr != NULL) {
+		if (texpr != NULL) {
 			if (v != NULL)
-				cell_set_expr_and_value (cell, expr, v, TRUE);
+				cell_set_expr_and_value (cell, texpr, v, TRUE);
 			else
-				cell_set_expr (cell, expr);
-			gnm_expr_unref (expr);
+				cell_set_expr (cell, texpr);
+			gnm_expr_top_unref (texpr);
 		} else if (v != NULL)
 			cell_set_value (cell, v);
 	}
@@ -654,7 +653,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 }
 
 void
-pln_file_open (GnmFileOpener const *fo, IOContext *io_context,
+pln_file_open (GOFileOpener const *fo, IOContext *io_context,
                WorkbookView *wb_view, GsfInput *input)
 {
 	Workbook *wb;
@@ -663,21 +662,21 @@ pln_file_open (GnmFileOpener const *fo, IOContext *io_context,
 	ErrorInfo *error;
 	PlanPerfectImport state;
 
-	wb    = wb_view_workbook (wb_view);
+	wb    = wb_view_get_workbook (wb_view);
 	name  = workbook_sheet_get_free_name (wb, "PlanPerfect", FALSE, TRUE);
 	sheet = sheet_new (wb, name);
 	g_free (name);
-	workbook_sheet_attach (wb, sheet, NULL);
+	workbook_sheet_attach (wb, sheet);
 	sheet_flag_recompute_spans (sheet);
 
 	state.sheet  = sheet;
 	state.styles = g_hash_table_new_full (
 		g_direct_hash, g_direct_equal,
-		NULL, (GDestroyNotify) mstyle_unref);
+		NULL, (GDestroyNotify) gnm_style_unref);
 	error = pln_parse_sheet (input, &state);
 	g_hash_table_destroy (state.styles);
 	if (error != NULL) {
-		workbook_sheet_detach (wb, sheet, FALSE);
+		workbook_sheet_delete (sheet);
 		gnumeric_io_error_info_set (io_context, error);
 	}
 }
