@@ -15,7 +15,9 @@ GObject     *gnm_app_get_app (void);
 void         gnm_app_workbook_list_add     (Workbook *wb);
 void         gnm_app_workbook_list_remove  (Workbook *wb);
 GList *      gnm_app_workbook_list 	   (void);
-Workbook    *gnm_app_workbook_get_by_name  (char const *name);
+Workbook    *gnm_app_workbook_get_by_uri   (char const *uri);
+Workbook    *gnm_app_workbook_get_by_name  (char const *name,
+					    char const *ref_uri);
 Workbook    *gnm_app_workbook_get_by_index (int i);
 gboolean     gnm_app_workbook_foreach	   (GnmWbIterFunc func, gpointer data);
 
@@ -39,6 +41,7 @@ double	     gnm_app_dpi_to_pixels	  (void);
 
 /* Clipboard */
 void		 gnm_app_clipboard_clear	  (gboolean drop_selection);
+void		 gnm_app_clipboard_invalidate_sheet (Sheet *sheet);
 void		 gnm_app_clipboard_cut_copy	  (WorkbookControl *wbc, gboolean is_cut,
 						   SheetView *sv, GnmRange const *area,
 						   gboolean animate_range);
