@@ -1,9 +1,9 @@
-/* vim: set sw=8: */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * cell.c: Cell content and simple management.
  *
  * Author:
- *    Jody Goldberg 2000, 2001 (jody@gnome.org)
+ *    Jody Goldberg 2000-2006 (jody@gnome.org)
  *    Miguel de Icaza 1998, 1999 (miguel@kernel.org)
  */
 #include <gnumeric-config.h>
@@ -19,31 +19,14 @@
 #include "value.h"
 #include "str.h"
 #include "style.h"
+#include "ranges.h"
 #include "gnm-format.h"
+#include "number-match.h"
 #include "sheet-object-cell-comment.h"
 #include "sheet-style.h"
 #include "parse-util.h"
 #include <goffice/utils/go-glib-extras.h>
 
-#define USE_CELL_POOL 1
-
-/**
- * cell_dirty : Mark the sheet containing the cell as being dirty.
- * @cell : the dirty cell.
- *
- * INTERNAL.
- */
-#warning this is just plain silly handle this at a higher level
-static inline void
-cell_dirty (GnmCell *cell)
-{
-	Sheet *sheet = cell->base.sheet;
-
-	/* Cells from the clipboard do not have a sheet attached */
-	if (sheet)
-		sheet_set_dirty (sheet, TRUE);
-}
-
 /**
  * cell_cleanout :
  *      Empty a cell's
@@ -62,7 +45,7 @@ cell_dirty (GnmCell *cell)
  *      	- unqueue a previously queued recalc.
  *      	- Mark sheet as dirty.
  */
-static void
+void
 cell_cleanout (GnmCell *cell)
 {
 	/* A cell can have either an expression or entered text */
@@ -70,8 +53,8 @@ cell_cleanout (GnmCell *cell)
 		/* Clipboard cells, e.g., are not attached to a sheet.  */
 		if (cell_expr_is_linked (cell))
 			dependent_unlink (CELL_TO_DEP (cell));
-		gnm_expr_unref (cell->base.expression);
-		cell->base.expression = NULL;
+		gnm_expr_top_unref (cell->base.texpr);
+		cell->base.texpr = NULL;
 	}
 
 	if (cell->value != NULL) {
@@ -86,110 +69,6 @@ cell_cleanout (GnmCell *cell)
 		cell->row_info->needs_respan = TRUE;
 }
 
-/* The pool from which all cells are allocated.  */
-static GOMemChunk *cell_pool;
-
-/**
- * cell_new:
- * @sheet: sheet in which to allocate cell.
- *
- * Creates a new cell.
- */
-GnmCell *
-cell_new (void)
-{
-	GnmCell *cell = USE_CELL_POOL ? go_mem_chunk_alloc0 (cell_pool) : g_new0 (GnmCell, 1);
-	cell->base.flags = DEPENDENT_CELL;
-	return cell;
-}
-
-
-/**
- * cell_copy:
- * @cell: existing cell to duplicate
- *
- * Makes a copy of a GnmCell.
- *
- * Returns a copy of the cell.
- */
-GnmCell *
-cell_copy (GnmCell const *cell)
-{
-	GnmCell *new_cell;
-
-	g_return_val_if_fail (cell != NULL, NULL);
-
-	new_cell = cell_new ();
-
-	/* bitmap copy first */
-	*new_cell = *cell;
-
-	/* The new cell is not linked into any of the major management structures */
-	new_cell->base.sheet = NULL;
-	new_cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC|CELL_IN_SHEET_LIST|DEPENDENT_IS_LINKED);
-
-	/* now copy properly the rest */
-	if (cell_has_expr (new_cell))
-		gnm_expr_ref (new_cell->base.expression);
-
-	new_cell->rendered_value = NULL;
-
-	new_cell->value = (new_cell->value)
-		? value_dup (new_cell->value)
-		: value_new_empty ();
-
-	return new_cell;
-}
-
-/**
- * cell_destroy: Frees all resources allocated to the cell's content and marks the
- *     GnmCell's container as dirty.
- *
- * @cell : The cell to destroy
- */
-void
-cell_destroy (GnmCell *cell)
-{
-	g_return_if_fail (cell != NULL);
-
-	cell_cleanout (cell);
-	if (USE_CELL_POOL) go_mem_chunk_free (cell_pool, cell); else g_free (cell);
-}
-
-/*
- * cell_relocate:
- * @cell   : The cell that is changing position
- * @rwinfo : An OPTIONAL pointer to allow for bounds checking and relocation
- *
- * This routine is used to move a cell to a different location:
- */
-void
-cell_relocate (GnmCell *cell, GnmExprRewriteInfo const *rwinfo)
-{
-	g_return_if_fail (cell != NULL);
-	g_return_if_fail (rwinfo != NULL);
-
-	/* 1. Tag the cell as dirty */
-	cell_dirty (cell);
-
-	/* 2. If the cell contains a expr, relocate it */
-	if (cell_has_expr (cell)) {
-		GnmExpr const *expr = gnm_expr_rewrite (cell->base.expression, rwinfo);
-
-#warning "make this a precondition"
-		if (cell_expr_is_linked (cell))
-			dependent_unlink (CELL_TO_DEP (cell));
-
-		/* bounds check, and adjust local references from the cell */
-		if (expr != NULL) {
-			gnm_expr_unref (cell->base.expression);
-			cell->base.expression = expr;
-		}
-
-		dependent_link (CELL_TO_DEP (cell));
-	}
-}
-
 /****************************************************************************/
 
 /*
@@ -209,25 +88,24 @@ cell_relocate (GnmCell *cell, GnmExprRewriteInfo const *rwinfo)
 void
 cell_set_text (GnmCell *cell, char const *text)
 {
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	GnmValue      *val;
-	GnmParsePos       pos;
+	GnmParsePos    pos;
 
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
-	g_return_if_fail (!cell_is_partial_array (cell));
+	g_return_if_fail (!cell_is_nonsingleton_array (cell));
 
 	parse_text_value_or_expr (parse_pos_init_cell (&pos, cell),
-		text, &val, &expr, gnm_style_get_format (cell_get_mstyle (cell)),
+		text, &val, &texpr, gnm_style_get_format (cell_get_style (cell)),
 		workbook_date_conv (cell->base.sheet->workbook));
 
 	if (val != NULL) {	/* String was a value */
 		cell_cleanout (cell);
 		cell->value = val;
-		cell_dirty (cell);
 	} else {		/* String was an expression */
-		cell_set_expr (cell, expr);
-		gnm_expr_unref (expr);
+		cell_set_expr (cell, texpr);
+		gnm_expr_top_unref (texpr);
 	}
 }
 
@@ -274,11 +152,10 @@ cell_set_value (GnmCell *cell, GnmValue *v)
 {
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (v != NULL);
-	g_return_if_fail (!cell_is_partial_array (cell));
+	g_return_if_fail (!cell_is_nonsingleton_array (cell));
 
 	cell_cleanout (cell);
 	cell->value = v;
-	cell_dirty (cell);
 }
 
 /*
@@ -294,19 +171,19 @@ cell_set_value (GnmCell *cell, GnmValue *v)
  * NOTE : This DOES check for array partitioning.
  */
 void
-cell_set_expr_and_value (GnmCell *cell, GnmExpr const *expr, GnmValue *v,
+cell_set_expr_and_value (GnmCell *cell, GnmExprTop const *texpr, GnmValue *v,
 			 gboolean link_expr)
 {
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (expr != NULL);
-	g_return_if_fail (!cell_is_partial_array (cell));
+	g_return_if_fail (texpr != NULL);
+	g_return_if_fail (!cell_is_nonsingleton_array (cell));
 
 	/* Repeat after me.  Ref before unref. */
-	gnm_expr_ref (expr);
+	gnm_expr_top_ref (texpr);
 	cell_cleanout (cell);
-	cell_dirty (cell);
 
-	cell->base.expression = expr;
+	cell->base.flags |= CELL_HAS_NEW_EXPR;
+	cell->base.texpr = texpr;
 	cell->value = v;
 	if (link_expr)
 		dependent_link (CELL_TO_DEP (cell));
@@ -325,14 +202,14 @@ cell_set_expr_and_value (GnmCell *cell, GnmExpr const *expr, GnmValue *v,
  * 	- link the expression into the master list.
  */
 static void
-cell_set_expr_internal (GnmCell *cell, GnmExpr const *expr)
+cell_set_expr_internal (GnmCell *cell, GnmExprTop const *texpr)
 {
-	gnm_expr_ref (expr);
+	gnm_expr_top_ref (texpr);
 
 	cell_cleanout (cell);
 
 	cell->base.flags |= CELL_HAS_NEW_EXPR;
-	cell->base.expression = expr;
+	cell->base.texpr = texpr;
 
 	/* Until the value is recomputed, we put in this value.
 	 *
@@ -340,7 +217,6 @@ cell_set_expr_internal (GnmCell *cell, GnmExpr const *expr)
 	 * cell_needs_recalc call in sheet_foreach_cell_in_range.
 	 */
 	cell->value = value_new_empty ();
-	cell_dirty (cell);
 }
 
 /*
@@ -355,12 +231,12 @@ cell_set_expr_internal (GnmCell *cell, GnmExpr const *expr)
  *           using this.
  */
 void
-cell_set_expr_unsafe (GnmCell *cell, GnmExpr const *expr)
+cell_set_expr_unsafe (GnmCell *cell, GnmExprTop const *texpr)
 {
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 
-	cell_set_expr_internal (cell, expr);
+	cell_set_expr_internal (cell, texpr);
 }
 
 /**
@@ -374,13 +250,13 @@ cell_set_expr_unsafe (GnmCell *cell, GnmExpr const *expr)
  *           Be very careful using this.
  */
 void
-cell_set_expr (GnmCell *cell, GnmExpr const *expr)
+cell_set_expr (GnmCell *cell, GnmExprTop const *texpr)
 {
-	g_return_if_fail (!cell_is_partial_array (cell));
+	g_return_if_fail (!cell_is_nonsingleton_array (cell));
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 
-	cell_set_expr_internal (cell, expr);
+	cell_set_expr_internal (cell, texpr);
 	dependent_link (CELL_TO_DEP (cell));
 }
 
@@ -399,8 +275,8 @@ cell_set_expr (GnmCell *cell, GnmExpr const *expr)
  * upper left corner is handled as a special case and care is taken to
  * put it at the head of the recalc queue if recalcs are requested.
  *
- * NOTE : Does not add a reference to the expression.  It takes over the callers
- *        reference.
+ * NOTE : Does not add a reference to the expression.  It takes over the
+ *        caller's reference.
  *
  * Does not regenerate spans, dimensions or autosize cols/rows.
  *
@@ -409,38 +285,43 @@ cell_set_expr (GnmCell *cell, GnmExpr const *expr)
 void
 cell_set_array_formula (Sheet *sheet,
 			int col_a, int row_a, int col_b, int row_b,
-			GnmExpr const *expr)
+			GnmExprTop const *texpr)
 {
 	int const num_rows = 1 + row_b - row_a;
 	int const num_cols = 1 + col_b - col_a;
 	int x, y;
-	GnmCell * const corner = sheet_cell_fetch (sheet, col_a, row_a);
-	GnmExpr const *wrapper;
+	GnmCell *corner;
+	GnmExprTop const *wrapper;
 
 	g_return_if_fail (num_cols > 0);
 	g_return_if_fail (num_rows > 0);
-	g_return_if_fail (expr != NULL);
-	g_return_if_fail (corner != NULL);
+	g_return_if_fail (texpr != NULL);
 	g_return_if_fail (col_a <= col_b);
 	g_return_if_fail (row_a <= row_b);
 
-	wrapper = gnm_expr_new_array (0, 0, num_cols, num_rows, expr);
-	cell_set_expr_internal (corner, wrapper);
-	gnm_expr_unref (wrapper);
+	corner = sheet_cell_fetch (sheet, col_a, row_a);
+	g_return_if_fail (corner != NULL);
 
-	for (x = 0; x < num_cols; ++x)
+	wrapper = gnm_expr_top_new_array_corner (num_cols, num_rows, gnm_expr_copy (texpr->expr));
+	gnm_expr_top_unref (texpr);
+	cell_set_expr_internal (corner, wrapper);
+	gnm_expr_top_unref (wrapper);
+
+	for (x = 0; x < num_cols; ++x) {
 		for (y = 0; y < num_rows; ++y) {
 			GnmCell *cell;
+			GnmExprTop const *te;
 
 			if (x == 0 && y == 0)
 				continue;
 
 			cell = sheet_cell_fetch (sheet, col_a + x, row_a + y);
-			wrapper = gnm_expr_new_array (x, y, num_cols, num_rows, NULL);
-			cell_set_expr_internal (cell, wrapper);
+			te = gnm_expr_top_new_array_elem (x, y);
+			cell_set_expr_internal (cell, te);
 			dependent_link (CELL_TO_DEP (cell));
-			gnm_expr_unref (wrapper);
+			gnm_expr_top_unref (te);
 		}
+	}
 
 	dependent_link (CELL_TO_DEP (corner));
 }
@@ -456,9 +337,7 @@ cell_set_array_formula (Sheet *sheet,
 gboolean
 cell_is_empty (GnmCell const * cell)
 {
-	return  cell == NULL ||
-		cell->value == NULL ||
-		cell->value->type == VALUE_EMPTY;
+	return cell == NULL || VALUE_IS_EMPTY (cell->value);
 }
 
 /**
@@ -470,9 +349,7 @@ cell_is_empty (GnmCell const * cell)
 gboolean
 cell_is_blank (GnmCell const * cell)
 {
-	return  cell == NULL ||
-		cell->value == NULL ||
-		cell->value->type == VALUE_EMPTY ||
+	return cell_is_empty (cell) ||
 		(VALUE_IS_STRING (cell->value) && *(cell->value->v_str.val->str) == '\0');
 }
 
@@ -482,7 +359,7 @@ cell_is_error (GnmCell const * cell)
 	g_return_val_if_fail (cell != NULL, NULL);
 	g_return_val_if_fail (cell->value != NULL, NULL);
 
-	if (cell->value->type == VALUE_ERROR)
+	if (VALUE_IS_ERROR (cell->value))
 		return cell->value;
 	return NULL;
 }
@@ -498,36 +375,76 @@ gboolean
 cell_is_zero (GnmCell const *cell)
 {
 	GnmValue const * const v = cell->value;
-	if (v == NULL)
-		return FALSE;
-	switch (v->type) {
-	case VALUE_BOOLEAN : return !v->v_bool.val;
-	case VALUE_INTEGER : return v->v_int.val == 0;
-	case VALUE_FLOAT :
-	{
-		gnm_float res = v->v_float.val;
-		return (-1e-10 < res && res < 1e-10);
-	}
-
-	default :
-		return FALSE;
-	}
-}
-
-GnmExprArray const *
-cell_is_array (GnmCell const *cell)
-{
-	if (cell != NULL && cell_has_expr (cell) &&
-	    cell->base.expression->any.oper == GNM_EXPR_OP_ARRAY)
-		return &cell->base.expression->array;
-	return NULL;
+	return v && VALUE_IS_NUMBER (v) && gnm_abs (value_get_as_float (v)) < 64 * GNM_EPSILON;
 }
 
 gboolean
-cell_is_partial_array (GnmCell const *cell)
+cell_array_bound (GnmCell const *cell, GnmRange *res)
 {
-	GnmExprArray const *ref = cell_is_array (cell);
-	return ref != NULL && (ref->cols > 1 || ref->rows > 1);
+	GnmExprTop const *texpr;
+	GnmExprArrayCorner const *array;
+
+	if (NULL == cell || !cell_has_expr (cell))
+		return FALSE;
+
+	g_return_val_if_fail (res != NULL, FALSE);
+
+	texpr = cell->base.texpr;
+	if (gnm_expr_top_is_array_elem (texpr)) {
+		cell = sheet_cell_get (cell->base.sheet,
+			cell->pos.col - texpr->expr->array_elem.x,
+			cell->pos.row - texpr->expr->array_elem.y);
+
+		g_return_val_if_fail (cell != NULL, FALSE);
+		g_return_val_if_fail (cell_has_expr (cell), FALSE);
+
+		texpr = cell->base.texpr;
+	}
+
+	array = gnm_expr_top_get_array_corner (texpr);
+	if (!array)
+		return FALSE;
+
+	range_init (res, cell->pos.col, cell->pos.row,
+		cell->pos.col + array->cols - 1,
+		cell->pos.row + array->rows - 1);
+	return TRUE;
+}
+
+GnmExprArrayCorner const *
+cell_is_array_corner (GnmCell const *cell)
+{
+	return cell && cell_has_expr (cell)
+		? gnm_expr_top_get_array_corner (cell->base.texpr)
+		: NULL;
+}
+
+/**
+ * cell_is_array :
+ * @cell : #GnmCell const *
+ *
+ * Return TRUE is @cell is part of an array
+ **/
+gboolean
+cell_is_array (GnmCell const *cell)
+{
+	return cell != NULL && cell_has_expr (cell) &&
+		(gnm_expr_top_get_array_corner (cell->base.texpr) ||
+		 gnm_expr_top_is_array_elem (cell->base.texpr));
+}
+
+/**
+ * cell_is_nonsingleton_array :
+ * @cell : #GnmCell const *
+ *
+ * Return TRUE is @cell is part of an array larger than 1x1
+ **/
+gboolean
+cell_is_nonsingleton_array (GnmCell const *cell)
+{
+	GnmExprArrayCorner const *corner = cell_is_array_corner (cell);
+
+	return corner && (corner->cols > 1 || corner->rows > 1);
 }
 
 /***************************************************************************/
@@ -552,7 +469,7 @@ cell_render_value (GnmCell *cell, gboolean allow_variable_width)
 	g_return_if_fail (cell != NULL);
 
 	sheet = cell->base.sheet;
-	rv = rendered_value_new (cell, cell_get_mstyle (cell),
+	rv = rendered_value_new (cell, cell_get_style (cell),
 				 allow_variable_width,
 				 sheet->context,
 				 sheet->last_zoom_factor_used);
@@ -580,9 +497,120 @@ cell_get_rendered_text  (GnmCell *cell)
 	return g_strdup (rendered_value_get_text (cell->rendered_value));
 }
 
+/**
+ * cell_get_render_color:
+ * @cell: the cell from which we want to pull the color from
+ *
+ * The returned value is a pointer to a PangoColor describing
+ * the foreground colour.
+ */
+GOColor
+cell_get_render_color (GnmCell const *cell)
+{
+	g_return_val_if_fail (cell != NULL, 0);
 
-GnmStyle *
-cell_get_mstyle (GnmCell const *cell)
+	/* A precursor to just in time rendering Ick! */
+	if (cell->rendered_value == NULL)
+		cell_render_value ((GnmCell *)cell, TRUE);
+
+	return cell->rendered_value->go_fore_color;
+}
+
+/**
+ * cell_get_entered_text:
+ * @cell: the cell from which we want to pull the content from
+ *
+ * This returns a g_malloc()ed region of memory with a text representation
+ * of the cell contents.
+ *
+ * This will return a text expression if the cell contains a formula, or
+ * a string representation of the value.
+ */
+char *
+cell_get_entered_text (GnmCell const *cell)
+{
+	g_return_val_if_fail (cell != NULL, NULL);
+
+	if (cell_has_expr (cell)) {
+		GnmParsePos pp;
+		GString *res = g_string_new ("=");
+
+		gnm_expr_top_as_gstring (res, cell->base.texpr,
+					 parse_pos_init_cell (&pp, cell),
+					 cell->base.sheet->convs);
+		return g_string_free (res, FALSE);
+	}
+
+	if (cell->value != NULL) {
+		if (VALUE_IS_STRING (cell->value)) {
+			/* Try to be reasonably smart about adding a leading quote */
+			char const *tmp = cell->value->v_str.val->str;
+
+			if (tmp[0] != '\'' && !gnm_expr_char_start_p (tmp)) {
+				GnmValue *val = format_match_number (tmp,
+					cell_get_format	(cell),
+					workbook_date_conv (cell->base.sheet->workbook));
+				if (val == NULL)
+					return g_strdup (tmp);
+				value_release (val);
+			}
+			return g_strconcat ("\'", tmp, NULL);
+		}
+		return format_value (NULL, cell->value, NULL, -1,
+			workbook_date_conv (cell->base.sheet->workbook));
+	}
+
+	g_warning ("A cell with no expression, and no value ??");
+	return g_strdup ("<ERROR>");
+}
+
+
+/*
+ * Return the height of the rendered layout after rotation.
+ */
+int
+cell_rendered_height (GnmCell const *cell)
+{
+	const RenderedValue *rv;
+
+	g_return_val_if_fail (cell != NULL, 0);
+
+	rv = cell->rendered_value;
+	if (!rv)
+		return 0;
+
+	return PANGO_PIXELS (cell->rendered_value->layout_natural_height);
+}
+
+/*
+ * Return the width of the rendered layout after rotation.
+ */
+int
+cell_rendered_width (GnmCell const *cell)
+{
+	const RenderedValue *rv;
+
+	g_return_val_if_fail (cell != NULL, 0);
+
+	rv = cell->rendered_value;
+	if (!rv)
+		return 0;
+
+	return PANGO_PIXELS (cell->rendered_value->layout_natural_width);
+}
+
+int
+cell_rendered_offset (GnmCell const * cell)
+{
+	if (!cell || !cell->rendered_value)
+		return 0;
+
+	return (cell->rendered_value->indent_left +
+		cell->rendered_value->indent_right);
+}
+
+GnmStyle const *
+cell_get_style (GnmCell const *cell)
 {
 	g_return_val_if_fail (cell != NULL, NULL);
 	return sheet_style_get (cell->base.sheet,
@@ -596,19 +624,19 @@ cell_get_mstyle (GnmCell const *cell)
  *
  * Get the display format.  If the assigned format is General,
  * the format of the value will be used.
- */
+ **/
 GOFormat *
 cell_get_format (GnmCell const *cell)
 {
 	GOFormat *fmt;
 
-	g_return_val_if_fail (cell != NULL, style_format_general ());
+	g_return_val_if_fail (cell != NULL, go_format_general ());
 
-	fmt = gnm_style_get_format (cell_get_mstyle (cell));
+	fmt = gnm_style_get_format (cell_get_style (cell));
 
-	g_return_val_if_fail (fmt != NULL, style_format_general ());
+	g_return_val_if_fail (fmt != NULL, go_format_general ());
 
-	if (style_format_is_general (fmt) &&
+	if (go_format_is_general (fmt) &&
 	    cell->value != NULL && VALUE_FMT (cell->value))
 		fmt = VALUE_FMT (cell->value);
 
@@ -631,7 +659,6 @@ cell_set_format (GnmCell *cell, char const *format)
 
 	g_return_if_fail (mstyle != NULL);
 
-	cell_dirty (cell);
 	gnm_style_set_format_text (mstyle, format);
 
 	r.start = r.end = cell->pos;
@@ -656,70 +683,13 @@ void
 cell_convert_expr_to_value (GnmCell *cell)
 {
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (cell_has_expr(cell));
+	g_return_if_fail (cell_has_expr (cell));
 
 	/* Clipboard cells, e.g., are not attached to a sheet.  */
 	if (cell_expr_is_linked (cell))
 		dependent_unlink (CELL_TO_DEP (cell));
 
-	gnm_expr_unref (cell->base.expression);
-	cell->base.expression = NULL;
-
-	cell_dirty (cell);
+	gnm_expr_top_unref (cell->base.texpr);
+	cell->base.texpr = NULL;
 }
 
-GnmComment *
-cell_has_comment_pos (Sheet const *sheet, GnmCellPos const *pos)
-{
-	GnmRange r;
-	GSList *comments;
-	GnmComment *res;
-
-	r.start = r.end = *pos;
-	comments = sheet_objects_get (sheet, &r, CELL_COMMENT_TYPE);
-	if (!comments)
-		return NULL;
-
-	/* This assumes just one comment per cell.  */
-	res = comments->data;
-	g_slist_free (comments);
-	return res;
-}
-
-
-GnmComment *
-cell_has_comment (GnmCell const *cell)
-{
-	return cell_has_comment_pos (cell->base.sheet, &cell->pos);
-}
-
-/****************************************************************************/
-
-void
-cell_init (void)
-{
-#if USE_CELL_POOL
-	cell_pool = go_mem_chunk_new ("cell pool",
-				       sizeof (GnmCell),
-				       128 * 1024 - 128);
-#endif
-}
-
-#if USE_CELL_POOL
-static void
-cb_cell_pool_leak (gpointer data, G_GNUC_UNUSED gpointer user)
-{
-	GnmCell const *cell = data;
-	fprintf (stderr, "Leaking cell %p at %s\n", cell, cellpos_as_string (&cell->pos));
-}
-#endif
-
-void
-cell_shutdown (void)
-{
-#if USE_CELL_POOL
-	go_mem_chunk_foreach_leak (cell_pool, cb_cell_pool_leak, NULL);
-	go_mem_chunk_destroy (cell_pool, FALSE);
-	cell_pool = NULL;
-#endif
-}
