@@ -20,7 +20,7 @@
  * USA
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "cmd-edit.h"
 
@@ -287,7 +287,7 @@ sv_select_cur_inputs (SheetView *sv)
  * When pasting a copy the destination can be a singleton, or an integer
  * multiple of the size of the source.  This is not tested here.
  * Full undo support.
- */
+ **/
 void
 cmd_paste (WorkbookControl *wbc, GnmPasteTarget const *pt)
 {
@@ -329,7 +329,7 @@ cmd_paste (WorkbookControl *wbc, GnmPasteTarget const *pt)
 			return;
 		}
 
-		rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
+		rinfo.reloc_type = GNM_EXPR_RELOCATE_MOVE_RANGE;
 		rinfo.origin = *src_range;
 		rinfo.col_offset = dst.start.col - rinfo.origin.start.col;
 		rinfo.row_offset = dst.start.row - rinfo.origin.start.row;
@@ -394,7 +394,7 @@ cmd_shift_rows (WorkbookControl *wbc, Sheet *sheet,
 	GnmExprRelocateInfo rinfo;
 	char *desc;
 
-	rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
+	rinfo.reloc_type = GNM_EXPR_RELOCATE_MOVE_RANGE;
 	rinfo.col_offset = count;
 	rinfo.row_offset = 0;
 	rinfo.origin_sheet = rinfo.target_sheet = sheet;
@@ -433,7 +433,7 @@ cmd_shift_cols (WorkbookControl *wbc, Sheet *sheet,
 	GnmExprRelocateInfo rinfo;
 	char *desc;
 
-	rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
+	rinfo.reloc_type = GNM_EXPR_RELOCATE_MOVE_RANGE;
 	rinfo.col_offset = 0;
 	rinfo.row_offset = count;
 	rinfo.origin_sheet = rinfo.target_sheet = sheet;
