@@ -22,7 +22,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -1202,4 +1202,3 @@ dialog_cell_sort (WorkbookControlGUI *wbcg)
 
 	gtk_widget_show (state->dialog);
 }
-
