@@ -55,7 +55,7 @@
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 enum {
@@ -70,7 +70,6 @@ typedef struct {
 	GtkWidget	*dialog;
 	GtkWidget	*notebook;
 	GtkTextView	*description;
-	GSList		*pages;
 	GtkTreeStore    *store;
 	GtkTreeView     *view;
 	Workbook	*wb;
@@ -422,7 +421,7 @@ cb_pref_font_hf_set_fonts (GOConfNode *node, char const *key, GtkWidget *page)
 
 static gboolean
 cb_pref_font_hf_has_changed (G_GNUC_UNUSED FontSelector *fs,
-			  GnmStyle *mstyle, PrefState *state)
+			     GnmStyle *mstyle, PrefState *state)
 {
 	gnm_gconf_set_hf_font (mstyle);
 	return TRUE;
@@ -537,6 +536,10 @@ pref_sort_page_initializer (PrefState *state,
 				 page, row++, 
 				 gnm_gconf_set_sort_ascending,
 				 _("Sort Ascending"));
+	bool_pref_create_widget (node, GNM_CONF_SORT_DEFAULT_HAS_HEADER,
+				 page, row++, 
+				 gnm_gconf_set_sort_has_header,
+				 _("Sort Area Has a HEADER"));
 	go_conf_free_node (node);
 
 	gtk_widget_show_all (page);
@@ -659,8 +662,8 @@ pref_file_page_initializer (PrefState *state,
 
 static void
 pref_screen_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
-		     G_GNUC_UNUSED GtkNotebook *notebook,
-		     G_GNUC_UNUSED gint page_num)
+		       G_GNUC_UNUSED GtkNotebook *notebook,
+		       G_GNUC_UNUSED gint page_num)
 {
 	dialog_pref_load_description (state,
 				      _("The items on this page are related to "
@@ -669,9 +672,9 @@ pref_screen_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
 
 static GtkWidget *
 pref_screen_page_initializer (PrefState *state,
-			    G_GNUC_UNUSED gpointer data,
-			    G_GNUC_UNUSED GtkNotebook *notebook,
-			    G_GNUC_UNUSED gint page_num)
+			      G_GNUC_UNUSED gpointer data,
+			      G_GNUC_UNUSED GtkNotebook *notebook,
+			      G_GNUC_UNUSED gint page_num)
 {
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
@@ -743,8 +746,8 @@ pref_tool_page_initializer (PrefState *state,
 
 static void
 pref_copypaste_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
-		     G_GNUC_UNUSED GtkNotebook *notebook,
-		     G_GNUC_UNUSED gint page_num)
+			  G_GNUC_UNUSED GtkNotebook *notebook,
+			  G_GNUC_UNUSED gint page_num)
 {
 	dialog_pref_load_description (state,
 				      _("The items on this page are "
@@ -754,9 +757,9 @@ pref_copypaste_page_open (PrefState *state, G_GNUC_UNUSED gpointer data,
 
 static GtkWidget *
 pref_copypaste_page_initializer (PrefState *state,
-			    G_GNUC_UNUSED gpointer data,
-			    G_GNUC_UNUSED GtkNotebook *notebook,
-			    G_GNUC_UNUSED gint page_num)
+				 G_GNUC_UNUSED gpointer data,
+				 G_GNUC_UNUSED GtkNotebook *notebook,
+				 G_GNUC_UNUSED gint page_num)
 {
 	GtkWidget *page = gtk_table_new (2, 2, FALSE);
 	gint row = 0;
@@ -840,11 +843,17 @@ cb_preferences_destroy (PrefState *state)
 {
 	go_conf_sync (state->root);
 
+	if (state->store) {
+		g_object_unref (state->store);
+		state->store = NULL;
+	}
+
 	if (state->gui != NULL) {
 		g_object_unref (G_OBJECT (state->gui));
 		state->gui = NULL;
 	}
 	state->dialog = NULL;
+
 	g_free (state);
 
 	gnm_app_set_pref_dialog (NULL);
@@ -901,9 +910,8 @@ dialog_preferences (WorkbookControlGUI *wbcg, gint page)
 	state->gui = gui;
 	state->dialog     = glade_xml_get_widget (gui, "preferences");
 	state->notebook   = glade_xml_get_widget (gui, "notebook");
-	state->pages      = NULL;
 	state->description = GTK_TEXT_VIEW (glade_xml_get_widget (gui, "description"));
-	state->wb	  = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb	  = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 
 	state->view = GTK_TREE_VIEW(glade_xml_get_widget (gui, "itemlist"));
 	state->store = gtk_tree_store_new (NUM_COLUMNS,
@@ -953,8 +961,6 @@ dialog_preferences (WorkbookControlGUI *wbcg, gint page)
 							       GTK_NOTEBOOK (state->notebook), i);
 		GtkWidget *label = NULL;
 
-		state->pages = g_slist_append (state->pages, page);
-
 		if (this_page->icon_name)
 			label = gtk_image_new_from_stock (this_page->icon_name,
 							  GTK_ICON_SIZE_BUTTON);
