@@ -20,12 +20,13 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialog-stf.h"
 #include <gui-util.h>
 #include <workbook.h>
 #include <gtk/gtkframe.h>
+#include <goffice/utils/go-glib-extras.h>
 
 /*************************************************************************************************
  * MISC UTILITY FUNCTIONS
@@ -44,14 +45,22 @@ main_page_set_encoding (StfDialogData *pagedata, const char *enc)
 	utf8_data = g_convert (pagedata->raw_data, pagedata->raw_data_len,
 			       "UTF-8", enc,
 			       &bytes_read, &bytes_written, &error);
-	if (error) {
+
+	/*
+	 * It _is_ possible to get NULL error, but not have valid UTF-8.
+	 * Specifically observed with UTF-16BE.
+	 */
+	if (error || !g_utf8_validate (utf8_data, -1, NULL)) {
 		g_free (utf8_data);
-		g_error_free (error);
-		/* FIXME: What to do with error?  */
+		if (error) {
+			/* FIXME: What to do with error?  */
+			g_error_free (error);
+		}
 		return FALSE;
 	}
 
-	if (!charmap_selector_set_encoding (pagedata->main.charmap_selector, enc)) {
+
+	if (!go_charmap_sel_set_encoding (pagedata->main.charmap_selector, enc)) {
 		g_free (utf8_data);
 		return FALSE;
 	}
@@ -147,7 +156,9 @@ main_page_import_range_changed (StfDialogData *data)
 	data->cur = stf_parse_find_line (data->parseoptions, data->utf8_data, startrow - 1);
 	data->cur_end = stf_parse_find_line (data->parseoptions, data->utf8_data, stoprow);
 
-	linescaption = g_strdup_printf (_("%d of %d lines to import"),
+	linescaption = g_strdup_printf (ngettext("%d of %d line to import",
+						 "%d of %d lines to import",
+						 renderdata->lines->len),
 					(stoprow - startrow) + 1,
 					renderdata->lines->len);
 	gtk_label_set_text (data->main.main_lines, linescaption);
@@ -159,23 +170,23 @@ main_page_import_range_changed (StfDialogData *data)
  *************************************************************************************************/
 
 static void
-encodings_changed_cb (CharmapSelector *cs, char const *new_charmap,
+encodings_changed_cb (GOCharmapSel *cs, char const *new_charmap,
 		      StfDialogData *pagedata)
 {
 	if (main_page_set_encoding (pagedata, new_charmap)) {
-		main_page_import_range_changed (pagedata);
 		main_page_update_preview (pagedata);
+		main_page_import_range_changed (pagedata);
 	} else {
-		const char *name = charmap_selector_get_encoding_name (cs, new_charmap);
+		const char *name = go_charmap_sel_get_encoding_name (cs, new_charmap);
 		char *msg = g_strdup_printf
 			(_("The data is not valid in encoding %s; "
 			   "please select another encoding."),
 			 name ? name : new_charmap);
-		gnumeric_notice (GTK_WINDOW (pagedata->dialog),
+		go_gtk_notice_dialog (GTK_WINDOW (pagedata->dialog),
 				 GTK_MESSAGE_ERROR, msg);
 		g_free (msg);
 
-		charmap_selector_set_encoding (pagedata->main.charmap_selector,
+		go_charmap_sel_set_encoding (pagedata->main.charmap_selector,
 					       pagedata->encoding);
 	}
 }
@@ -269,7 +280,7 @@ main_page_parseoptions_to_gui (StfDialogData *pagedata)
 	StfParseOptions_t *po = pagedata->parseoptions;
 
 #if 0
-	charmap_selector_set_encoding (pagedata->main.charmap_selector, pagedata->encoding);
+	go_charmap_sel_set_encoding (pagedata->main.charmap_selector, pagedata->encoding);
 #endif
 
 	switch (po->parsetype) {
@@ -283,7 +294,7 @@ main_page_parseoptions_to_gui (StfDialogData *pagedata)
 		break;
 	default:
 		break;
-	};
+	}
 
 	{
 		gboolean lb_unix = FALSE, lb_windows = FALSE, lb_mac = FALSE;
@@ -326,9 +337,11 @@ stf_dialog_main_page_init (GladeXML *gui, StfDialogData *pagedata)
 {
 	RenderData_t *renderdata;
 	GtkTreeViewColumn *column;
-	char const *locale_encoding;
+	const char *encoding_guess;
 
-	g_get_charset (&locale_encoding);
+	encoding_guess = go_guess_encoding (pagedata->raw_data, pagedata->raw_data_len,
+					    "ASCII",
+					    NULL);
 
 	pagedata->main.main_separated = GTK_RADIO_BUTTON (glade_xml_get_widget (gui, "main_separated"));
 	pagedata->main.main_fixed     = GTK_RADIO_BUTTON (glade_xml_get_widget (gui, "main_fixed"));
@@ -340,13 +353,13 @@ stf_dialog_main_page_init (GladeXML *gui, StfDialogData *pagedata)
 	pagedata->main.line_break_windows = GTK_CHECK_BUTTON (glade_xml_get_widget (gui, "line_break_windows"));
 	pagedata->main.line_break_mac     = GTK_CHECK_BUTTON (glade_xml_get_widget (gui, "line_break_mac"));
 
-	pagedata->main.charmap_selector = CHARMAP_SELECTOR (charmap_selector_new (CHARMAP_SELECTOR_TO_UTF8));
+	pagedata->main.charmap_selector = GO_CHARMAP_SEL (go_charmap_sel_new (GO_CHARMAP_SEL_TO_UTF8));
 	if (!main_page_set_encoding (pagedata, pagedata->encoding) &&
-	    !main_page_set_encoding (pagedata, locale_encoding) &&
-	    !main_page_set_encoding (pagedata, "ASCII") &&
-	    !main_page_set_encoding (pagedata, "ISO-8859-1") &&
-	    !main_page_set_encoding (pagedata, "UTF-8"))
+	    !main_page_set_encoding (pagedata, encoding_guess)) {
 		g_warning ("This is not good -- failed to find a valid encoding of data!");
+		pagedata->raw_data_len = 0;
+		main_page_set_encoding (pagedata, "ASCII");
+	}
 	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (gui, "encoding_hbox")),
 			   GTK_WIDGET (pagedata->main.charmap_selector));
 	gtk_widget_show_all (GTK_WIDGET (pagedata->main.charmap_selector));
