@@ -90,11 +90,12 @@ void 	 gnm_expr_list_unref	  (GnmExprList *list);
 
 /*****************************************************************************/
 
-#define GNM_EXPR_TOP_MAGIC 0x01123622
+#define GNM_EXPR_TOP_MAGIC 0x42
 #define IS_GNM_EXPR_TOP(et) ((et) && (et)->magic == GNM_EXPR_TOP_MAGIC)
 
 struct _GnmExprTop {
-	guint32 magic;
+	unsigned magic : 8;
+	unsigned hash : 24;  /* Zero meaning not yet computed.  */
 	guint32 refcount;
 	GnmExpr const *expr;
 };
@@ -107,6 +108,7 @@ GnmExprTop const *gnm_expr_top_new_array_elem	(int x, int y);
 void		gnm_expr_top_ref		(GnmExprTop const *texpr);
 void		gnm_expr_top_unref		(GnmExprTop const *texpr);
 gboolean	gnm_expr_top_equal		(GnmExprTop const *te1, GnmExprTop const *te2);
+guint           gnm_expr_top_hash               (GnmExprTop const *texpr);
 gboolean	gnm_expr_top_is_shared		(GnmExprTop const *texpr);
 gboolean	gnm_expr_top_is_err		(GnmExprTop const *texpr, GnmStdError e);
 gboolean 	gnm_expr_top_is_rangeref	(GnmExprTop const *texpr);
@@ -119,6 +121,7 @@ void		gnm_expr_top_get_boundingbox	(GnmExprTop const *texpr, GnmRange *bound);
 gboolean	gnm_expr_top_contains_subtotal	(GnmExprTop const *texpr);
 GSList	       *gnm_expr_top_referenced_sheets	(GnmExprTop const *texpr);
 GnmExpr const  *gnm_expr_top_first_funcall	(GnmExprTop const *texpr);
+GnmExprTop const *gnm_expr_top_transpose        (GnmExprTop const *texpr);
 
 struct _GnmExprRelocateInfo {
 	GnmParsePos pos;
