@@ -10,7 +10,7 @@
  *  Jody Goldberg   (jody@gnome.org)
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 #include "gnumeric.h"
 #include "func.h"
@@ -1050,7 +1050,9 @@ function_call_with_exprs (FunctionEvalInfo *ei,
 		return value_new_error_NA (ei->pos);
 
 	args = g_alloca (sizeof (GnmValue *) * fn_def->fn.args.max_args);
-	iter_count = (flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR) ? 0 : -1;
+	iter_count = (ei->pos->array != NULL &&
+		      (flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR))
+		? 0 : -1;
 
 	for (i = 0; i < argc; i++) {
 		char arg_type = fn_def->fn.args.arg_types[i];
@@ -1063,8 +1065,6 @@ function_call_with_exprs (FunctionEvalInfo *ei,
 				GnmCellRef r;
 				gnm_cellref_make_abs (&r, &expr->cellref.ref, ei->pos);
 				args[i] = value_new_cellrange_unsafe (&r, &r);
-				/* TODO decide on the semantics of these argument types */
-#warning "Do we need to force an eval here?"
 			} else {
 				tmp = args[i] = gnm_expr_eval (expr, ei->pos,
 					GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
@@ -1223,7 +1223,7 @@ function_call_with_exprs (FunctionEvalInfo *ei,
 						} else if (VALUE_IS_ERROR (elem)) {
 							err = elem;
 							break;
-						} else if (!VALUE_IS_STRING (tmp))
+						} else if (!VALUE_IS_STRING (elem))
 							break;
 					} else if (elem == NULL) {
 						args [iter_item[i]] = iter_args [i] = value_new_empty ();
@@ -1318,25 +1318,26 @@ typedef struct {
 	gboolean           ignore_subtotal;
 } IterateCallbackClosure;
 
-/*
+/**
  * cb_iterate_cellrange:
  *
  * Helper routine used by the function_iterate_do_value routine.
  * Invoked by the sheet cell range iterator.
- */
+ **/
 static GnmValue *
-cb_iterate_cellrange (Sheet *sheet, int col, int row,
-		      GnmCell *cell, gpointer user_data)
+cb_iterate_cellrange (GnmCellIter const *iter, gpointer user)
+		      
 {
-	IterateCallbackClosure *data = user_data;
+	IterateCallbackClosure *data = user;
+	GnmCell  *cell;
 	GnmValue *res;
 	GnmEvalPos ep;
 
-	if (cell == NULL) {
-		ep.sheet = sheet;
+	if (NULL == (cell = iter->cell)) {
+		ep.sheet = iter->pp.sheet;
 		ep.dep = NULL;
-		ep.eval.col = col;
-		ep.eval.row = row;
+		ep.eval.col = iter->pp.eval.col;
+		ep.eval.row = iter->pp.eval.row;
 		return (*data->callback)(&ep, NULL, data->closure);
 	}
 
@@ -1483,13 +1484,15 @@ function_iterate_argument_values (GnmEvalPos const	*ep,
 
 		/* We need a cleaner model of what to do here.  For now it
 		 * seems as if var-arg functions treat explict ranges as special cases
-		 * 	SUM(Range)
+		 * 	SUM(Range) or
+		 * 	SUM(INDIRECT(foo))
 		 * will pass the range not do implicit intersection in non-array mode
 		 * 	SUM(Range=3)
 		 * will do implicit intersection in non-array mode */
 		if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT)
 			val = value_dup (expr->constant.value);
-		else if (ep->array != NULL)
+		else if (ep->array != NULL ||
+			 GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL)
 			val = gnm_expr_eval (expr, ep,
 				GNM_EXPR_EVAL_PERMIT_EMPTY | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 		else
@@ -1501,7 +1504,6 @@ function_iterate_argument_values (GnmEvalPos const	*ep,
 
 		if (strict && VALUE_IS_ERROR (val)) {
 			/* Be careful not to make VALUE_TERMINATE into a real value */
-			/* FIXME : Make the new position of the error here */
 			return val;
 		}
 
