@@ -8,9 +8,6 @@
  *     Miguel de Icaza (miguel@kernel.org)
  *     Jody Goldberg (jody@gnome.org)
  *
- * Port to Maemo:
- * 	Eduardo Lima  (eduardo.lima@indt.org.br)
- * 	Renato Araujo (renato.filho@indt.org.br)
  */
 
 #include <gnumeric-config.h>
@@ -306,22 +303,16 @@ item_grid_draw_merged_range (GdkDrawable *drawable, ItemGrid *ig,
 
 	if (cell != NULL) {
 		ColRowInfo const * const ri = cell->row_info;
-		ColRowInfo const * const ci = cell->col_info;
 
 		if (ri->needs_respan)
-			row_calc_spans ((ColRowInfo *)ri, sheet);
+			row_calc_spans ((ColRowInfo *)ri, cell->pos.row, sheet);
 
-		/* FIXME : get the margins from the far col/row too */
 		if (sheet->text_is_rtl)
 			cell_draw (cell, ig->gc.cell, drawable,
-				   r, t,
-				   l - r - (ci->margin_b + ci->margin_a + 1),
-				   b - t - (ri->margin_b + ri->margin_a + 1), -1);
+				r, t, l - r, b - t, -1);
 		else
-		cell_draw (cell, ig->gc.cell, drawable,
-			   l, t,
-			   r - l - (ci->margin_b + ci->margin_a + 1),
-			   b - t - (ri->margin_b + ri->margin_a + 1), -1);
+			cell_draw (cell, ig->gc.cell, drawable,
+				l, t, r - l, b - t, -1);
 	}
 	style_border_draw_diag (style, drawable, l, t, r, b);
 }
@@ -509,7 +500,7 @@ item_grid_draw (FooCanvasItem *item, GdkDrawable *drawable, GdkEventExpose *expo
 		 * will ever get flagged.
 		 */
 		if (ri->needs_respan)
-			row_calc_spans ((ColRowInfo *)ri, sheet);
+			row_calc_spans ((ColRowInfo *)ri, row, sheet);
 
 		/* look for merges that start on this row, on the first painted row
 		 * also check for merges that start above. */
@@ -635,11 +626,11 @@ plain_draw : /* a quick hack to deal with 142267 */
 
 			/* Is this part of a span?
 			 * 1) There are cells allocated in the row
-			 *       (indicated by ri->pos != -1)
+			 *       (indicated by ri->spans != NULL)
 			 * 2) Look in the rows hash table to see if
 			 *    there is a span descriptor.
 			 */
-			if (ri->pos == -1 || NULL == (span = row_span_get (ri, col))) {
+			if (NULL == ri->spans || NULL == (span = row_span_get (ri, col))) {
 
 				/* If it is being edited pretend it is empty to
 				 * avoid problems with the a long cells
@@ -649,22 +640,22 @@ plain_draw : /* a quick hack to deal with 142267 */
 				GnmCell const *cell = sheet_cell_get (sheet, col, row);
 				if (!cell_is_empty (cell) && cell != edit_cell)
 					cell_draw (cell, ig->gc.cell, drawable,
-						   x, y, -1, -1, -1);
+						   x, y, ci->size_pixels,
+						   ri->size_pixels, -1);
 
 			/* Only draw spaning cells after all the backgrounds
 			 * that we are going to draw have been drawn.  No need
-			 * to draw the edit cell, or blanks.
-			 */
+			 * to draw the edit cell, or blanks. */
 			} else if (edit_cell != span->cell &&
 				   (col == span->right || col == end_col)) {
 				GnmCell const *cell = span->cell;
 				int const start_span_col = span->left;
 				int const end_span_col = span->right;
 				int real_x = x;
-				int center_offset = cell->col_info->size_pixels/2;
-				/* TODO : Use the spanning margins */
-				int tmp_width = ci->size_pixels -
-					ci->margin_b - ci->margin_a;
+				ColRowInfo const *cell_col =
+					sheet_col_get_info (sheet, cell->pos.col);
+				int center_offset = cell_col->size_pixels/2;
+				int tmp_width = ci->size_pixels;
 
 				if (col != cell->pos.col)
 					style = sheet_style_get (sheet,
@@ -672,8 +663,7 @@ plain_draw : /* a quick hack to deal with 142267 */
 
 				/* x, y are relative to this cell origin, but the cell
 				 * might be using columns to the left (if it is set to right
-				 * justify or center justify) compute the pixel difference
-				 */
+				 * justify or center justify) compute the pixel difference */
 				if (start_span_col != cell->pos.col)
 					center_offset += scg_colrow_distance_get (
 						gcanvas->simple.scg, TRUE,
@@ -698,7 +688,8 @@ plain_draw : /* a quick hack to deal with 142267 */
 				}
 
 				cell_draw (cell, ig->gc.cell, drawable,
-					   real_x, y, tmp_width, -1, center_offset);
+					   real_x, y, tmp_width,
+					   ri->size_pixels, center_offset);
 			} else if (col != span->left)
 				sr.vertical [col] = NULL;
 
@@ -775,7 +766,7 @@ ig_obj_create_begin (ItemGrid *ig, GdkEventButton *event)
 
 	coords[0] = coords[2] = event->x;
 	coords[1] = coords[3] = event->y;
-	sheet_object_anchor_init (&anchor, NULL, NULL, NULL, SO_DIR_DOWN_RIGHT);
+	sheet_object_anchor_init (&anchor, NULL, NULL, NULL, GOD_ANCHOR_DIR_DOWN_RIGHT);
 	scg_object_coords_to_anchor (ig->scg, coords, &anchor);
 	sheet_object_set_anchor (so, &anchor);
 	sheet_object_set_sheet (so, sc_sheet (SHEET_CONTROL (ig->scg)));
@@ -897,9 +888,6 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 
 			if ((ig->last_click_time + double_click_time) > event->time &&
 			    wbcg_edit_start (scg->wbcg, FALSE, FALSE)) {
-#ifdef USE_HILDON
-				gtk_im_context_focus_in (GNM_CANVAS (canvas)->im_context);
-#endif
 				break;
 			}
 		}
@@ -952,7 +940,7 @@ cb_cursor_come_to_rest (ItemGrid *ig)
 	GnmHLink *link;
 	int x, y;
 	GnmCellPos pos;
-	const char *tiptext;
+	char const *tiptext;
 
 	/* Be anal and look it up in case something has destroyed the link
 	 * since the last motion */
@@ -1022,7 +1010,7 @@ item_grid_event (FooCanvasItem *item, GdkEvent *event)
 
 		default:
 			g_assert_not_reached ();
-		};
+		}
 
 		ig->selecting = ITEM_GRID_NO_SELECTION;
 		gnm_simple_canvas_ungrab (item, event->button.time);
@@ -1066,7 +1054,7 @@ item_grid_event (FooCanvasItem *item, GdkEvent *event)
 			break;
 		default:
 			g_assert_not_reached ();
-		};
+		}
 
 		gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
 			GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y |
