@@ -19,6 +19,7 @@
 #include "gutils.h"
 #include "gnumeric-gconf.h"
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-font.h>
 
 #include <stdio.h>
 
@@ -1019,7 +1020,7 @@ gnm_style_set_font_name (GnmStyle *style, char const *name)
 	gnm_style_pango_clear (style);
 }
 
-const char *
+char const *
 gnm_style_get_font_name (GnmStyle const *style)
 {
 	g_return_val_if_fail (elem_is_set (style, MSTYLE_FONT_NAME), NULL);
@@ -1155,7 +1156,7 @@ gnm_style_set_format (GnmStyle *style, GOFormat *format)
 }
 
 void
-gnm_style_set_format_text (GnmStyle *style, const char *format)
+gnm_style_set_format_text (GnmStyle *style, char const *format)
 {
 	GOFormat *sf;
 
@@ -1477,6 +1478,7 @@ gnm_style_get_pango_attrs (GnmStyle const *style,
 
 	((GnmStyle *)style)->pango_attrs = l = pango_attr_list_new ();
 	((GnmStyle *)style)->pango_attrs_zoom = zoom;
+	((GnmStyle *)style)->pango_attrs_height = -1;
 
 	/* Foreground colour.  */
 	/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
@@ -1515,7 +1517,7 @@ gnm_style_get_pango_attrs (GnmStyle const *style,
 
 	{
 		GnmFont *font = gnm_style_get_font (style, context, zoom);
-		add_attr (l, pango_attr_font_desc_new (font->pango.font_descr));
+		add_attr (l, pango_attr_font_desc_new (font->go.font->desc));
 	}
 
 	add_attr (l, pango_attr_scale_new (zoom));
@@ -1553,6 +1555,28 @@ gnm_style_generate_attrs_full (GnmStyle const *style)
 	return l;
 }
 
+int
+gnm_style_get_pango_height (GnmStyle const *style,
+			    PangoContext *context,
+			    float zoom)
+{
+	PangoAttrList *attrs = gnm_style_get_pango_attrs (style, context, zoom);
+
+	if (style->pango_attrs_height == -1) {
+		int h;
+		PangoLayout *layout = pango_layout_new (context);
+		pango_layout_set_attributes (layout, attrs);
+		pango_layout_set_text (layout, "+1.23456789E-01", -1);
+		pango_layout_get_pixel_size (layout, NULL, &h);
+		g_object_unref (layout);
+		((GnmStyle *)style)->pango_attrs_height = h;
+	}
+
+	pango_attr_list_unref (attrs);
+	return style->pango_attrs_height;
+}
+
+
 void
 gnm_style_set_from_pango_attribute (GnmStyle *style, PangoAttribute const *attr)
 {
