@@ -10,7 +10,7 @@
  * Handles printing of Sheets.
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "print.h"
 
@@ -35,6 +35,7 @@
 #include "ranges.h"
 #include "style-font.h"
 #include "gnumeric-gconf.h"
+#include <goffice/utils/go-font.h>
 
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprint/gnome-print-config.h>
@@ -288,7 +289,7 @@ ensure_decoration_layout (PrintJobInfo *pj)
 			 1.);
 
 		pj->decoration_layout = layout;
-		pango_layout_set_font_description (layout, font->pango.font_descr);
+		pango_layout_set_font_description (layout, font->go.font->desc);
 		gnm_style_unref (style);
 	}
 	return pj->decoration_layout;
@@ -470,15 +471,9 @@ setup_scale (PrintJobInfo const *pj, Sheet const *sheet)
 }
 
 static GnmValue *
-cb_range_empty (Sheet *sheet, int col, int row, GnmCell *cell, gpointer flags)
+cb_range_empty (GnmCellIter const *iter, gpointer flags)
 {
-	ColRowInfo const *cri = sheet_col_get_info (sheet, col);
-	if (!cri->visible)
-		return NULL;
-	cri = sheet_row_get_info (sheet, row);
-	if (!cri->visible)
-		return NULL;
-	return VALUE_TERMINATE;
+	return (iter->ci->visible && iter->ri->visible) ? VALUE_TERMINATE : NULL;
 }
 
 /**
@@ -602,9 +597,10 @@ print_page (PrintJobInfo const *pj, Sheet const *sheet, GnmRange *range,
 		x += 1.;
 		y += 1.;
 	} else {
-		/* If there are no grids ignore the leading cell margins */
-		x -= sheet->cols.default_style.margin_a;
-		y -= sheet->rows.default_style.margin_a;
+		/* If there are no grids alias back to avoid a penalty for the
+		 * margins of the leading gridline */
+		x -= GNM_COL_MARGIN;
+		y -= GNM_ROW_MARGIN;
 	}
 
 	/* Note: we cannot have spaces in page numbers.  */
@@ -1043,9 +1039,8 @@ compute_sheet_pages (PageCountInfo *pc, Sheet const *sheet)
 	}
 
 	if (pj->range != PRINT_SHEET_SELECTION) {
-		r = sheet_get_extent (sheet, TRUE);
-		if (pi->print_even_if_only_styles)
-			sheet_style_get_extent (sheet, &r, NULL);
+		r = sheet_get_printarea	(sheet,
+					 pi->print_even_if_only_styles);
 	} else
 		r = pc->r;
 
@@ -1097,10 +1092,8 @@ print_sheet (PrintJobInfo *pj, Sheet const *sheet)
 			return;
 	}
 
-	extent = sheet_get_extent (sheet, TRUE);
-	if (pi->print_even_if_only_styles)
-		sheet_style_get_extent (sheet, &extent, NULL);
-
+	extent = sheet_get_printarea (sheet,
+				      pi->print_even_if_only_styles);
 	print_sheet_range (pj, sheet, &extent, TRUE);
 }
 
