@@ -1,8 +1,9 @@
-/* vim: set sw=8: */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * ranges.c: various functions for common operations on cell ranges.
  *
  * Author:
+ *   Jody Goldberg   (jody@gnome.org)
  *   Miguel de Icaza (miguel@gnu.org).
  *   Michael Meeks   (mmeeks@gnu.org).
  */
@@ -26,7 +27,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 #undef RANGE_DEBUG
@@ -71,6 +72,16 @@ range_init_cellpos (GnmRange *r, GnmCellPos const *start, GnmCellPos const *end)
 
 	return r;
 }
+GnmRange *
+range_init_cellpos_size (GnmRange *r,
+			 GnmCellPos const *start, int cols, int rows)
+{
+	r->start = *start;
+	r->end.col = start->col + cols - 1;
+	r->end.row = start->row + rows - 1;
+
+	return r;
+}
 
 GnmRange *
 range_init (GnmRange *r, int start_col, int start_row,
@@ -86,60 +97,18 @@ range_init (GnmRange *r, int start_col, int start_row,
 	return r;
 }
 
-
 /**
- * range_parse:
- * @sheet: the sheet where the cell range is evaluated
- * @range: a range specification (ex: "A1", "A1:C3").
- * @strict: if FALSE, allow extra characters after range text.
+ * range_parse :
+ * @r : #GnmRange
+ * @text :
  *
- * Returns a (GnmValue *) of type VALUE_CELLRANGE if the @range was
- * succesfully parsed or NULL on failure.
- */
-GnmValue *
-range_parse (Sheet *sheet, char const *range, gboolean strict)
-{
-	GnmCellRef a, b;
-	GnmCellPos tmp;
-
-	g_return_val_if_fail (range != NULL, NULL);
-
-	range = cellpos_parse (range, &tmp, FALSE);
-	if (!range)
-		return NULL;
-
-	a.sheet = sheet;
-	a.col = tmp.col;
-	a.col_relative = 0;
-	a.row = tmp.row;
-	a.row_relative = 0;
-
-	if (*range == ':') {
-		range = cellpos_parse (range + 1, &tmp, strict);
-		if (!range)
-			return NULL;
-		b.sheet = sheet;
-		b.col = tmp.col;
-		b.col_relative = 0;
-		b.row = tmp.row;
-		b.row_relative = 0;
-	} else if (strict && *range)
-		return NULL;
-	else
-		b = a;
-
-	/*
-	 * We can dummy out the calling cell because we know that both
-	 * refs use the same mode.  This will handle inversions.
-	 */
-	return value_new_cellrange (&a, &b, 0, 0);
-}
-
-/* Pulled from dialog-analysis-tools.c
- * Should be merged with range_parse
- */
+ * Parse a simple range (no abs/rel refs, no sheet refs)
+ * Store the result in @res.
+ *
+ * Returns TRUE on success.
+ **/
 gboolean
-parse_range (char const *text, GnmRange *r)
+range_parse (GnmRange *r, char const *text)
 {
 	text = cellpos_parse (text, &r->start, FALSE);
 	if (!text)
@@ -162,7 +131,7 @@ parse_range (char const *text, GnmRange *r)
  * @ranges: a list of value ranges to destroy.
  *
  * Destroys a list of ranges returned from parse_cell_range_list
- */
+ **/
 void
 range_list_destroy (GSList *ranges)
 {
@@ -194,70 +163,6 @@ range_as_string (GnmRange const *src)
 	return buffer;
 }
 
-/**
- * range_has_header:
- * @sheet: Sheet to check
- * @src: GnmRange to check
- * @top: Flag
- *
- * This routine takes a sheet and a range and checks for a header row
- * in the range.  If top is true it looks for a header row from the top
- * and if false it looks for a header col from the left
- *
- * Return value: Whether or not the range has a header
- **/
-gboolean
-range_has_header (Sheet const *sheet, GnmRange const *src,
-		  gboolean top, gboolean ignore_styles)
-{
-	GnmCell const *a, *b;
-	int length, i;
-
-	/* There is only one row or col */
-	if (top) {
-		if (src->end.row <= src->start.row)
-			return FALSE;
-		length = src->end.col - src->start.col + 1;
-	} else {
-		if (src->end.col <= src->start.col)
-			return FALSE;
-		length = src->end.row - src->start.row + 1;
-	}
-
-	for (i = 0; i < length; i++) {
-		if (top) {
-			a = sheet_cell_get (sheet,
-				src->start.col + i, src->start.row);
-			b = sheet_cell_get (sheet,
-				src->start.col + i, src->start.row + 1);
-		} else {
-			a = sheet_cell_get (sheet,
-				src->start.col, src->start.row + i);
-			b = sheet_cell_get (sheet,
-				src->start.col + 1, src->start.row + i);
-		}
-
-		/* be anal */
-		if (a == NULL || a->value == NULL || b == NULL || b->value == NULL)
-			continue;
-
-		if (VALUE_IS_NUMBER (a->value)) {
-			if (!VALUE_IS_NUMBER (b->value))
-				return TRUE;
-			/* check for style differences */
-		} else if (a->value->type != b->value->type)
-			return TRUE;
-
-		/* Look for style differences */
-		if (!ignore_styles &&
-		    !gnm_style_equal_header (cell_get_style (a),
-					     cell_get_style (b), top))
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
 void
 range_dump (GnmRange const *src, char const *suffix)
 {
@@ -491,7 +396,7 @@ range_split_ranges (GnmRange const *hard, GnmRange const *soft)
  * Copies the @a range.
  *
  * Return value: A copy of the GnmRange.
- */
+ **/
 GnmRange *
 range_dup (GnmRange const *a)
 {
@@ -592,53 +497,6 @@ range_normalize (GnmRange *src)
 	}
 }
 
-static GnmValue *
-cb_find_extents (Sheet *sheet, int col, int row, GnmCell *cell,
-		 GnmCellPos *extent)
-{
-	if (extent->col < col)
-		extent->col = col;
-	if (extent->row < row)
-		extent->row = row;
-	return NULL;
-}
-
-/**
- * range_trim:
- * @sheet: sheet cells are contained on
- * @r:	   range to trim empty cells from
- * @cols:  trim from right
- * @rows:  trim from bottom
- *
- * This removes empty rows/cols from the
- * right hand or bottom edges of the range
- * depending on the value of @cols or @rows.
- *
- * Return value: TRUE if the range was totally empty.
- **/
-gboolean
-range_trim (Sheet const *sheet, GnmRange *r,
-	    gboolean cols, gboolean rows)
-{
-	GnmCellPos extent = { -1, -1 };
-
-	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
-	g_return_val_if_fail (r != NULL, TRUE);
-
-	sheet_foreach_cell_in_range (
-		(Sheet *)sheet, CELL_ITER_IGNORE_BLANK,
-		r->start.col, r->start.row, r->end.col, r->end.row,
-		(CellIterFunc) cb_find_extents, &extent);
-
-	if (extent.col < 0 || extent.row < 0)
-		return TRUE;
-	if (cols)
-		r->end.col = extent.col;
-	if (rows)
-		r->end.row = extent.row;
-	return FALSE;
-}
-
 /**
  * range_union:
  * @a: range a
@@ -711,7 +569,7 @@ range_is_full (GnmRange const *r, gboolean is_cols)
  *
  * Clip the range to the area of the sheet with content.
  * WARNING THIS IS EXPENSIVE!
- */
+ **/
 void
 range_clip_to_finite (GnmRange *range, Sheet *sheet)
 {
@@ -776,7 +634,7 @@ range_translate (GnmRange *range, int col_offset, int row_offset)
  *
  * Silently clip a range to ensure that it does not contain areas
  * outside the valid bounds.  Does NOT fix inverted ranges.
- */
+ **/
 void
 range_ensure_sanity (GnmRange *range)
 {
@@ -795,7 +653,7 @@ range_ensure_sanity (GnmRange *range)
  * @range : the range to check
  *
  * Generate warnings if the range is out of bounds or inverted.
- */
+ **/
 gboolean
 range_is_sane (GnmRange const *range)
 {
@@ -956,12 +814,12 @@ max_range_name_width (void)
 }
 
 /**
- * char const *undo_cell_pos_name
+ * undo_cell_pos_name:
  * @sheet:
  * @pos:
  *
  * Returns the range name depending on the preference setting.
- */
+ **/
 char *
 undo_cell_pos_name (Sheet const *sheet, GnmCellPos const *pos)
 {
@@ -1044,14 +902,14 @@ range_list_name_try (GString *names, Sheet const *sheet, GSList const *ranges,
 
 
 /**
- * char const *undo_range_list_name
+ * undo_range_list_name:
  * @sheet:
  * @ranges : GSList containing GnmRange *'s
  *
  * Returns the range list name depending on the preference setting.
  * (The result will be something like: "A1:C3, D4:E5"). The string will be
  * truncated to max_descriptor_width.
- */
+ **/
 char *
 undo_range_list_name (Sheet const *sheet, GSList const *ranges)
 {
@@ -1108,7 +966,7 @@ undo_range_list_name (Sheet const *sheet, GSList const *ranges)
  * results.
  *
  * Returns a GSList containing Values of type VALUE_CELLRANGE, or NULL on failure
- */
+ **/
 GSList *
 global_range_list_parse (Sheet *sheet, char const *str)
 {
@@ -1120,14 +978,12 @@ global_range_list_parse (Sheet *sheet, char const *str)
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	g_return_val_if_fail (str != NULL, NULL);
 
-	texpr = gnm_expr_parse_str
-		(str,
+	texpr = gnm_expr_parse_str (str,
 		 parse_pos_init_sheet (&pp, sheet),
 		 GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES |
 		 GNM_EXPR_PARSE_PERMIT_MULTIPLE_EXPRESSIONS |
 		 GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS,
-		 gnm_expr_conventions_default,
-		 NULL);
+		 NULL, NULL);
 
 	if (texpr != NULL)  {
 		if (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_SET) {
