@@ -397,7 +397,7 @@ cell_comment_set_cell (GnmComment *cc, GnmCellPos const *pos)
 
 	r.start = r.end = *pos;
 	sheet_object_anchor_init (&anchor, &r, NULL,
-		anchor_types, SO_DIR_DOWN_RIGHT);
+		anchor_types, GOD_ANCHOR_DIR_DOWN_RIGHT);
 	sheet_object_set_anchor (SHEET_OBJECT (cc), &anchor);
 }
 
