@@ -3,7 +3,7 @@
 /*
  * sheet-view.c:
  *
- * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -20,7 +20,7 @@
  * USA
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include "gnumeric.h"
 
@@ -123,13 +123,6 @@ sv_attach_control (SheetView *sv, SheetControl *sc)
 	sv_init_sc (sv, sc);
 }
 
-SheetControl *
-sv_get_control (SheetView const *sv, WorkbookControl const *wbc)
-{
-	SHEET_VIEW_FOREACH_CONTROL (sv, sc, if (sc_wbc (sc) == wbc) return sc;);
-	return NULL;
-}
-
 void
 sv_detach_control (SheetControl *sc)
 {
@@ -397,7 +390,7 @@ sv_selection_cut (SheetView *sv, WorkbookControl *wbc)
  * @move_col :
  * @move_row :
  * @bound    : An optionally NULL range that should contain all the supplied points
- */
+ **/
 void
 sv_cursor_set (SheetView *sv,
 	       GnmCellPos const *edit,
@@ -454,6 +447,7 @@ sv_set_edit_pos (SheetView *sv, GnmCellPos const *pos)
 	g_return_if_fail (pos->row < SHEET_MAX_ROWS);
 
 	old = sv->edit_pos;
+	sv->first_tab_col = -1; /* invalidate */
 
 	if (old.col != pos->col || old.row != pos->row) {
 		GnmRange const *merged = sheet_merge_is_corner (sv->sheet, &old);
@@ -618,9 +612,9 @@ sv_update (SheetView *sv)
 }
 
 static GnmValue *
-fail_if_not_selected (Sheet *sheet, int col, int row, GnmCell *cell, void *sv)
+cb_fail_if_not_selected (GnmCellIter const *iter, gpointer sv)
 {
-	if (!sv_is_pos_selected (sv, col, row))
+	if (!sv_is_pos_selected (sv, iter->pp.eval.col, iter->pp.eval.row))
 		return VALUE_TERMINATE;
 	else
 		return NULL;
@@ -667,7 +661,7 @@ sv_is_region_empty_or_selected (SheetView const *sv, GnmRange const *r)
 	return sheet_foreach_cell_in_range (
 		sv->sheet, CELL_ITER_IGNORE_NONEXISTENT,
 		r->start.col, r->start.row, r->end.col, r->end.row,
-		fail_if_not_selected, (gpointer)sv) == NULL;
+		cb_fail_if_not_selected, (gpointer)sv) == NULL;
 }
 
 /**
@@ -677,7 +671,8 @@ sv_is_region_empty_or_selected (SheetView const *sv, GnmRange const *r)
  * @unfrozen : top left corner of the unfrozen region
  *
  * By definition the unfrozen region must be below the frozen.
- */
+ * If @frozen == @unfrozen or @frozen == NULL unfreeze
+ **/
 void
 sv_freeze_panes (SheetView *sv,
 		 GnmCellPos const *frozen,
@@ -692,7 +687,8 @@ sv_freeze_panes (SheetView *sv,
 
 		/* Just in case */
 		if (unfrozen->col != (SHEET_MAX_COLS-1) &&
-		    unfrozen->row != (SHEET_MAX_ROWS-1)) {
+		    unfrozen->row != (SHEET_MAX_ROWS-1) &&
+		    !gnm_cellpos_equal (frozen, unfrozen)) {
 			sv->frozen_top_left = *frozen;
 			sv->unfrozen_top_left = *unfrozen;
 			if (sv->frozen_top_left.col == sv->unfrozen_top_left.col)
