@@ -27,7 +27,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "stf-parse.h"
 
@@ -39,9 +39,9 @@
 #include "number-match.h"
 #include "gutils.h"
 #include "parse-util.h"
-#include "gnm-format.h"
 #include <goffice/utils/datetime.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-format.h>
 
 #include <stdlib.h>
 #include <locale.h>
@@ -77,13 +77,13 @@ typedef struct {
  * not deal very well with that.
  */
 static inline int
-my_garray_len (const GArray *a)
+my_garray_len (GArray const *a)
 {
 	return (int)a->len;
 }
 
 static inline int
-my_gptrarray_len (const GPtrArray *a)
+my_gptrarray_len (GPtrArray const *a)
 {
 	return (int)a->len;
 }
@@ -102,7 +102,7 @@ comp_term (gchar const *s, gchar const *term)
 static int
 compare_terminator (char const *s, StfParseOptions_t *parseoptions)
 {
-	const guchar *us = (const guchar *)s;
+	guchar const *us = (guchar const *)s;
 	GSList *l;
 
 	if (*us > parseoptions->compiled_terminator.max ||
@@ -110,8 +110,8 @@ compare_terminator (char const *s, StfParseOptions_t *parseoptions)
 		return 0;
 
 	for (l = parseoptions->terminator; l; l = l->next) {
-		const char *term = l->data;
-		const char *d = s;
+		char const *term = l->data;
+		char const *d = s;
 
 		while (*term) {
 			if (*d != *term)
@@ -156,9 +156,10 @@ stf_parse_options_new (void)
 	parseoptions->splitpositions = NULL;
 	stf_parse_options_fixed_splitpositions_clear (parseoptions);
 
-	parseoptions->stringindicator        = '"';
+	parseoptions->stringindicator = '"';
 	parseoptions->indicator_2x_is_single = TRUE;
-	parseoptions->duplicates             = FALSE;
+	parseoptions->duplicates = FALSE;
+	parseoptions->trim_seps = FALSE;
 
 	parseoptions->sep.str = NULL;
 	parseoptions->sep.chr = NULL;
@@ -264,30 +265,6 @@ stf_parse_options_add_line_terminator (StfParseOptions_t *parseoptions, char con
 	compile_terminators (parseoptions);
 }
 
-/**
- * stf_parse_options_remove_line_terminator:
- *
- * This will remove from the line terminators, in both the Fixed width and CSV delimited importers
- * this indicates the end of a row.
- *
- **/
-void
-stf_parse_options_remove_line_terminator (StfParseOptions_t *parseoptions, char const *terminator)
-{
-	GSList*    in_list;
-
-	g_return_if_fail (parseoptions != NULL);
-
-	in_list = g_slist_find_custom (parseoptions->terminator, terminator, go_str_compare);
-
-	if (in_list) {
-		char *s = in_list->data;
-		GO_SLIST_REMOVE (parseoptions->terminator, in_list->data);
-		g_free (s);
-		compile_terminators (parseoptions);
-	}
-}
-
 /**
  * stf_parse_options_clear_line_terminator:
  *
@@ -374,6 +351,19 @@ stf_parse_options_csv_set_duplicates (StfParseOptions_t *parseoptions, gboolean
 	parseoptions->duplicates = duplicates;
 }
 
+/**
+ * stf_parse_options_csv_set_trim_seps:
+ * @trim_seps : a boolean value indicating whether we want to ignore
+ *               separators at the beginning of lines
+ **/
+void
+stf_parse_options_csv_set_trim_seps (StfParseOptions_t *parseoptions, gboolean const trim_seps)
+{
+	g_return_if_fail (parseoptions != NULL);
+
+	parseoptions->trim_seps = trim_seps;
+}
+
 /**
  * stf_parse_options_fixed_splitpositions_clear:
  *
@@ -665,6 +655,8 @@ stf_parse_csv_line (Source_t *src, StfParseOptions_t *parseoptions)
 	g_return_val_if_fail (parseoptions != NULL, NULL);
 
 	line = g_ptr_array_new ();
+	if (parseoptions->trim_seps)
+		stf_parse_eat_separators (src, parseoptions);
 	while (*src->position != '\0' && !compare_terminator (src->position, parseoptions)) {
 		char *field = stf_parse_csv_cell (src, parseoptions);
 		if (parseoptions->duplicates)
@@ -811,7 +803,7 @@ stf_parse_general (StfParseOptions_t *parseoptions,
 GPtrArray *
 stf_parse_lines (StfParseOptions_t *parseoptions,
 		 GStringChunk *lines_chunk,
-		 const char *data,
+		 char const *data,
 		 int maxlines, gboolean with_lineno)
 {
 	GPtrArray *lines;
@@ -821,7 +813,7 @@ stf_parse_lines (StfParseOptions_t *parseoptions,
 
 	lines = g_ptr_array_new ();
 	while (*data) {
-		const char *data0 = data;
+		char const *data0 = data;
 		GPtrArray *line = g_ptr_array_new ();
 
 		if (with_lineno) {
@@ -853,9 +845,9 @@ stf_parse_lines (StfParseOptions_t *parseoptions,
 	return lines;
 }
 
-const char *
+char const *
 stf_parse_find_line (StfParseOptions_t *parseoptions,
-		     const char *data,
+		     char const *data,
 		     int line)
 {
 	while (line > 0) {
@@ -1197,7 +1189,7 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	GODateConventions const *date_conv = wb ? workbook_date_conv (wb) : &default_conv;
 
 	GnmCellRegion *cr;
-	GSList	 *content = NULL;
+	GSList	 *contents = NULL;
 	unsigned int row, colhigh = 0;
 	GStringChunk *lines_chunk;
 	GPtrArray *lines;
@@ -1234,8 +1226,8 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 
 					ccopy = gnm_cell_copy_new (targetcol, row);
 					ccopy->val  = v;
-					ccopy->expr = NULL;
-					content = g_slist_prepend (content, ccopy);
+					ccopy->texpr = NULL;
+					contents = g_slist_prepend (contents, ccopy);
 					targetcol++;
 					if (targetcol > colhigh)
 						colhigh = targetcol;
@@ -1249,7 +1241,7 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	END_LOCALE_SWITCH;
 
 	cr = cellregion_new (NULL);
-	cr->content = content;
+	cr->contents = contents;
 	cr->cols    = (colhigh > 0) ? colhigh : 1;
 	cr->rows    = row;
 
@@ -1322,9 +1314,9 @@ stf_parse_next_token (char const *data, StfParseOptions_t *parseoptions, StfToke
 }
 
 static int
-int_sort (const void *a, const void *b)
+int_sort (void const *a, void const *b)
 {
-	return *(const int *)a - *(const int *)b;
+	return *(int const *)a - *(int const *)b;
 }
 
 
@@ -1341,7 +1333,7 @@ count_character (GPtrArray *lines, gunichar c, double quantile)
 	for (lno = cno = 0; lno < lines->len; lno++) {
 		int count = 0;
 		GPtrArray *boxline = g_ptr_array_index (lines, lno);
-		const char *line = g_ptr_array_index (boxline, 0);
+		char const *line = g_ptr_array_index (boxline, 0);
 
 		/* Ignore empty lines.  */
 		if (*line == 0)
@@ -1373,7 +1365,7 @@ count_character (GPtrArray *lines, gunichar c, double quantile)
 
 
 StfParseOptions_t *
-stf_parse_options_guess (const char *data)
+stf_parse_options_guess (char const *data)
 {
 	StfParseOptions_t *res;
 	GStringChunk *lines_chunk;
@@ -1408,23 +1400,33 @@ stf_parse_options_guess (const char *data)
 		if (count_character (lines, (c = sepchar), 0.5) > 0 ||
 		    count_character (lines, (c = format_get_col_sep ()), 0.5) > 0 ||
 		    count_character (lines, (c = ':'), 0.5) > 0 ||
+		    count_character (lines, (c = ','), 0.5) > 0 ||
 		    count_character (lines, (c = ';'), 0.5) > 0 ||
 		    count_character (lines, (c = '|'), 0.5) > 0 ||
 		    count_character (lines, (c = '!'), 0.5) > 0 ||
 		    count_character (lines, (c = ' '), 0.5) > 0) {
 			char sep[7];
 			sep[g_unichar_to_utf8 (c, sep)] = 0;
+			if (c == ' ')
+				strcat (sep, "\t");
 			stf_parse_options_csv_set_separators (res, sep, NULL);
 		}
 	}
 
 	if (1) {
 		/* Separated */
+		gboolean dups =
+			res->sep.chr &&
+			strchr (res->sep.chr, ' ') != NULL;
+		gboolean trim =
+			res->sep.chr &&
+			strchr (res->sep.chr, ' ') != NULL;
 
 		stf_parse_options_set_type (res, PARSE_TYPE_CSV);
 		stf_parse_options_set_trim_spaces (res, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
 		stf_parse_options_csv_set_indicator_2x_is_single (res, TRUE);
-		stf_parse_options_csv_set_duplicates (res, FALSE);
+		stf_parse_options_csv_set_duplicates (res, dups);
+		stf_parse_options_csv_set_trim_seps (res, trim);
 
 		stf_parse_options_csv_set_stringindicator (res, '"');
 	} else {
