@@ -9,7 +9,7 @@
  *
  * (C) Copyright 2000, 2001 by Jukka-Pekka Iivonen <jiivonen@hutcs.cs.hut.fi>
  * (C) Copyright 2001, 2002 by Andreas J. Guelzow  <aguelzow@taliesin.ca>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,7 +22,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include <gnumeric-config.h>
@@ -41,8 +41,10 @@
 #include "gnm-format.h"
 #include "sheet-object-cell-comment.h"
 #include "style-color.h"
+#include <goffice/app/go-doc.h>
 
-#include <glib/gi18n.h>
+#include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <time.h>
 #include <parse-util.h>
@@ -57,8 +59,8 @@
  **/
 
 data_analysis_output_t *
-dao_init (data_analysis_output_t *dao, 
-				  data_analysis_output_type_t type)
+dao_init (data_analysis_output_t *dao,
+	  data_analysis_output_type_t type)
 {
 	if (dao == NULL)
 		dao = g_new (data_analysis_output_t, 1);
@@ -85,7 +87,7 @@ dao_load_from_value (data_analysis_output_t *dao,
 		     GnmValue *output_range)
 {
 	g_return_val_if_fail (output_range != NULL, dao);
-	g_return_val_if_fail 
+	g_return_val_if_fail
 		(output_range->type == VALUE_CELLRANGE, dao);
 
 	dao->start_col = output_range->v_range.cell.a.col;
@@ -95,7 +97,7 @@ dao_load_from_value (data_analysis_output_t *dao,
 	dao->rows = output_range->v_range.cell.b.row
 		- output_range->v_range.cell.a.row + 1;
 	dao->sheet = output_range->v_range.cell.a.sheet;
-	
+
 	return dao;
 }
 
@@ -113,7 +115,7 @@ dao_range_name (data_analysis_output_t *dao)
 {
 	GnmRange range;
 	range_init (&range, dao->start_col, dao->start_row,
-		    dao->start_col + dao->cols - 1, 
+		    dao->start_col + dao->cols - 1,
 		    dao->start_row + dao->rows - 1);
 
 	return undo_range_name (dao->sheet, &range);
@@ -187,7 +189,7 @@ dao_adjust (data_analysis_output_t *dao, gint cols, gint rows)
 		if (rows != -1)
 			dao->rows = MIN (rows, dao->rows);
 	}
-	
+
 	if (dao->cols > max_cols)
 		dao->cols = max_cols;
 	if (dao->rows > max_rows)
@@ -198,7 +200,7 @@ dao_adjust (data_analysis_output_t *dao, gint cols, gint rows)
  * dao_prepare_output:
  * @dao:
  * @name:
- * 
+ *
  * prepares the output by creating a new sheet or workbook as appropriate
  *
  **/
@@ -213,9 +215,9 @@ dao_prepare_output (WorkbookControl *wbc, data_analysis_output_t *dao,
 		dao->wbc = wbc;
 
 	if (dao->type == NewSheetOutput) {
-		Workbook *wb = wb_control_workbook (dao->wbc);
+		Workbook *wb = wb_control_get_workbook (dao->wbc);
 		char *name_with_counter = g_strdup_printf ("%s (1)", name);
-		unique_name = workbook_sheet_get_free_name 
+		unique_name = workbook_sheet_get_free_name
 			(wb, name_with_counter, FALSE, TRUE);
 		g_free (name_with_counter);
 	        dao->sheet = sheet_new (wb, unique_name);
@@ -247,7 +249,7 @@ dao_prepare_output (WorkbookControl *wbc, data_analysis_output_t *dao,
  * @cmd:
  *
  * format's the output range according to the settings
- * 
+ *
  *
  **/
 gboolean
@@ -257,7 +259,7 @@ dao_format_output (data_analysis_output_t *dao, char const *cmd)
 	GnmRange range;
 
 	range_init (&range, dao->start_col, dao->start_row,
-		    dao->start_col + dao->cols - 1, 
+		    dao->start_col + dao->cols - 1,
 		    dao->start_row + dao->rows - 1);
 
 	if (dao->type == RangeOutput
@@ -304,11 +306,12 @@ dao_cell_is_visible (data_analysis_output_t *dao, int col, int row)
  *
  */
 
-void 
+void
 dao_set_cell_expr (data_analysis_output_t *dao, int col, int row,
 		   GnmExpr const *expr)
 {
         GnmCell *cell;
+	GnmExprTop const *texpr;
 
 	col += dao->offset_col;
 	row += dao->offset_row;
@@ -319,20 +322,21 @@ dao_set_cell_expr (data_analysis_output_t *dao, int col, int row,
 	if (dao->type == RangeOutput &&
 	    (dao->cols > 1 || dao->rows > 1) &&
 	    (col >= dao->cols || row >= dao->rows)) {
-		gnm_expr_unref (expr);
+		gnm_expr_free (expr);
 	        return;
 	}
 
 	col += dao->start_col;
 	row += dao->start_row;
 	if (col >= SHEET_MAX_COLS || row >= SHEET_MAX_ROWS) {
-		gnm_expr_unref (expr);
+		gnm_expr_free (expr);
 		return;
 	}
 
 	cell = sheet_cell_fetch (dao->sheet, col, row);
-	cell_set_expr (cell, expr);
-	gnm_expr_unref (expr);				
+	texpr = gnm_expr_top_new (expr);
+	cell_set_expr (cell, texpr);
+	gnm_expr_top_unref (texpr);
 }
 
 
@@ -347,7 +351,7 @@ dao_set_cell_expr (data_analysis_output_t *dao, int col, int row,
  *
  * Note: the rows/cols specification for all dao_set_cell_...
  *       commands are relative to the location of the output
- * 
+ *
  *
  **/
 
@@ -378,7 +382,7 @@ dao_set_cell_value (data_analysis_output_t *dao, int col, int row, GnmValue *v)
 
 	cell = sheet_cell_fetch (dao->sheet, col, row);
 
-	sheet_cell_set_value (cell, v);       
+	sheet_cell_set_value (cell, v);
 }
 
 /**
@@ -389,7 +393,7 @@ dao_set_cell_value (data_analysis_output_t *dao, int col, int row, GnmValue *v)
  * @text:
  *
  * set cell to a string
- * 
+ *
  *
  **/
 void
@@ -411,7 +415,7 @@ dao_set_cell (data_analysis_output_t *dao, int col, int row, const char *text)
  * @row:
  * @fmt:
  * @...:
- * 
+ *
  * create format string and set cell.
  *
  **/
@@ -439,7 +443,7 @@ dao_set_cell_printf (data_analysis_output_t *dao, int col, int row,
  * @v:
  *
  * set cell to a float
- * 
+ *
  *
  **/
 void
@@ -457,7 +461,7 @@ dao_set_cell_float (data_analysis_output_t *dao, int col, int row, gnm_float v)
  * @v:
  *
  * set cell to an int
- * 
+ *
  *
  **/
 void
@@ -474,7 +478,7 @@ dao_set_cell_int (data_analysis_output_t *dao, int col, int row, int v)
  * @row:
  *
  * set cell to NA
- * 
+ *
  *
  **/
 void
@@ -492,7 +496,7 @@ dao_set_cell_na (data_analysis_output_t *dao, int col, int row)
  * @is_valid:
  *
  * set cell to a float or NA as appropraite
- * 
+ *
  *
  **/
 void
@@ -512,7 +516,7 @@ dao_set_cell_float_na (data_analysis_output_t *dao, int col, int row,
  * @col:
  * @row:
  * @comment:
- * 
+ *
  * set a cell comment
  *
  **/
@@ -548,7 +552,7 @@ dao_set_cell_comment (data_analysis_output_t *dao, int col, int row,
  * @col:
  *
  * fits a column to the content
- * 
+ *
  *
  **/
 static void
@@ -558,7 +562,9 @@ dao_autofit_column (data_analysis_output_t *dao, int col)
 
 	actual_col = dao->start_col + col;
 
-	ideal_size = sheet_col_size_fit_pixels (dao->sheet, actual_col);
+	ideal_size = sheet_col_size_fit_pixels (dao->sheet, actual_col,
+						0, SHEET_MAX_ROWS - 1,
+						FALSE);
 	if (ideal_size == 0)
 	        return;
 
@@ -573,7 +579,7 @@ dao_autofit_column (data_analysis_output_t *dao, int col)
  * @to:
  *
  * fits all columns to their content
- * 
+ *
  *
  **/
 void
@@ -594,7 +600,7 @@ dao_autofit_these_columns (data_analysis_output_t *dao, int from, int to)
  * @to:
  *
  * fits all columns to their content
- * 
+ *
  *
  **/
 void
@@ -613,7 +619,7 @@ dao_autofit_columns (data_analysis_output_t *dao)
  * @style:
  *
  * sets the given cell range to bold
- * 
+ *
  *
  **/
 static void
@@ -653,7 +659,7 @@ dao_set_style (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * sets the given cell range to bold
- * 
+ *
  *
  **/
 void
@@ -681,7 +687,7 @@ dao_set_bold (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * sets the given cell range to underlined
- * 
+ *
  *
  **/
 void
@@ -709,7 +715,7 @@ dao_set_underlined (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * sets the given cell range to italic
- * 
+ *
  *
  **/
 void
@@ -731,7 +737,7 @@ dao_set_italic (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * set the given cell range to percent format
- * 
+ *
  *
  **/
 void
@@ -752,7 +758,7 @@ dao_set_percent (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * set the given cell range to date format
- * 
+ *
  *
  **/
 void
@@ -773,7 +779,7 @@ dao_set_date (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * set the given cell range to given background and text colors
- * 
+ *
  *
  **/
 void
@@ -800,7 +806,7 @@ dao_set_colors (data_analysis_output_t *dao, int col1, int row1,
  * @row2:
  *
  * set the given horizontal and vertical alignment to a cell range
- * 
+ *
  *
  **/
 void
@@ -822,7 +828,7 @@ dao_set_align (data_analysis_output_t *dao, int col1, int row1,
  * @dao:
  * @is_cols:
  *
- * 
+ *
  *
  **/
 ColRowStateList *
@@ -852,7 +858,7 @@ dao_get_colrow_state_list (data_analysis_output_t *dao, gboolean is_cols)
  * @is_cols:
  * @list:
  *
- * 
+ *
  *
  **/
 void
@@ -862,8 +868,8 @@ dao_set_colrow_state_list (data_analysis_output_t *dao, gboolean is_cols,
 	g_return_if_fail (list);
 
 	if (dao->type == RangeOutput)
-		colrow_set_states (dao->sheet, is_cols, 
-				   is_cols ? dao->start_col : dao->start_row, 
+		colrow_set_states (dao->sheet, is_cols,
+				   is_cols ? dao->start_col : dao->start_row,
 				   list);
 }
 
@@ -871,16 +877,23 @@ void
 dao_append_date (GString *buf)
 {
 	GDate     date;
-	GTimeVal  t;
 	struct tm tm_s;
 	gchar     *tmp;
+	time_t    now;
 
+#ifdef HAVE_G_DATE_SET_TIME_T
+	now = time (NULL);
+	g_date_set_time_t (&date, now);
+#else
+	GTimeVal  t;
 	g_get_current_time (&t);
-	g_date_set_time (&date, t.tv_sec);
+	now = t.tv_sec;
+ 	g_date_set_time (&date, t.tv_sec);
+#endif
 	g_date_to_struct_tm (&date, &tm_s);
-	tm_s.tm_sec  = t.tv_sec % 60;
-	tm_s.tm_min  = (t.tv_sec / 60) % 60;
-	tm_s.tm_hour = (t.tv_sec / 3600) % 24;
+	tm_s.tm_sec  = now % 60;
+	tm_s.tm_min  = (now / 60) % 60;
+	tm_s.tm_hour = (now / 3600) % 24;
 	tmp = asctime (&tm_s);
 	g_string_append (buf, tmp);
 }
@@ -892,7 +905,7 @@ dao_append_date (GString *buf)
  * @title:
  * @sheet:
  *
- * 
+ *
  *
  **/
 void
@@ -903,13 +916,13 @@ dao_write_header (data_analysis_output_t *dao, const gchar *toolname,
 	const char *uri;
 
 	buf = g_string_new (NULL);
-	g_string_append_printf (buf, "%s %s %s %s", 
+	g_string_append_printf (buf, "%s %s %s %s",
 		_("Gnumeric "), toolname, VERSION, title);
 	dao_set_cell (dao, 0, 0, buf->str);
 	g_string_free (buf, FALSE);
 
 	buf = g_string_new (NULL);
-	uri = workbook_get_uri (sheet->workbook);
+	uri = go_doc_get_uri (GO_DOC (sheet->workbook));
 	g_string_append_printf (buf, "%s [%s]%s", _("Worksheet:"),
 				uri,
 				sheet->name_quoted);
@@ -967,14 +980,14 @@ dao_find_name (Sheet *sheet, int col, int row)
 	return str;
 }
 
-gboolean 
+gboolean
 dao_put_formulas (data_analysis_output_t *dao)
 {
 	g_return_val_if_fail (dao != NULL, FALSE);
 	return dao->put_formulas;
 }
 
-void 
+void
 dao_convert_to_values (data_analysis_output_t *dao)
 {
 	int row, col;
@@ -985,7 +998,7 @@ dao_convert_to_values (data_analysis_output_t *dao)
 	workbook_recalc (dao->sheet->workbook);
 	for (row = 0; row < dao->rows; row++) {
 		for (col = 0; col < dao->cols; col++) {
-			GnmCell *cell = sheet_cell_get (dao->sheet, 
+			GnmCell *cell = sheet_cell_get (dao->sheet,
 				dao->start_col + col, dao->start_row + row);
 			if (cell != NULL && cell_has_expr (cell))
 				cell_convert_expr_to_value (cell);
@@ -998,12 +1011,12 @@ dao_redraw_respan (data_analysis_output_t *dao)
 {
 	GnmRange r;
 
-	range_init (&r, dao->start_col, dao->start_row, 
-		    dao->start_col + dao->cols - 1, 
+	range_init (&r, dao->start_col, dao->start_row,
+		    dao->start_col + dao->cols - 1,
 		    dao->start_row + dao->rows - 1);
-	sheet_range_calc_spans (dao->sheet, &r, 
+	sheet_range_calc_spans (dao->sheet, &r,
 				SPANCALC_RESIZE | SPANCALC_RENDER);
 	sheet_region_queue_recalc (dao->sheet, &r);
-	dao_convert_to_values (dao);	
+	dao_convert_to_values (dao);
 	sheet_redraw_range (dao->sheet, &r);
 }
