@@ -23,7 +23,7 @@
 */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "reports.h"
 
 #include "parse-util.h"
@@ -302,25 +302,15 @@ save_original_values (SolverResults          *res,
 	res->original_value_of_obj_fn = value_get_as_float (cell->value);
 }
 
-/*
- * Restores the original values of the input variables.
- */
 static void
 restore_original_values (SolverResults *res)
 {
-  
-	GSList   *inputs = res->param->input_cells;
-	GnmCell  *cell;
-	int      i = 0;
+	GSList *ptr;
+	int     i = 0;
 
-	while (inputs != NULL) {
-	        cell = (GnmCell *) inputs->data;
-
-		sheet_cell_set_value (cell,
-				      value_new_float(res->original_values[i]));
-		inputs = inputs->next;
-		i++;
-	}
+	for (ptr = res->param->input_cells; ptr != NULL ; ptr = ptr->next)
+		sheet_cell_set_value (ptr->data,
+			value_new_float (res->original_values[i++]));
 }
 
 /************************************************************************
@@ -470,7 +460,7 @@ lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 		g_get_current_time (&cur_time);
 		if (cur_time.tv_sec - start.tv_sec >
 		    param->options.max_time_sec) {
-			*errmsg = solver_max_time_err;
+			*errmsg = SOLVER_MAX_TIME_ERR;
 			solver_results_free (res);
 			return NULL;
 		}
@@ -760,14 +750,12 @@ solver_lp_copy (const SolverParameters *src_param, Sheet *new_sheet)
 
 	/* Copy the input cell list */
 	for (inputs = src_param->input_cells; inputs ; inputs = inputs->next) {
-		GnmCell *cell = inputs->data;
-		GnmCell *new_cell;
-
-		new_cell = cell_copy (cell);
-		new_cell->base.sheet = new_sheet;
+		GnmCell const *old_cell = inputs->data;
+		GnmCell *new_cell = sheet_cell_fetch (new_sheet,
+						      old_cell->pos.col,
+						      old_cell->pos.row);
 		dst_param->input_cells =
-		        g_slist_prepend (dst_param->input_cells,
-					 (gpointer) new_cell);
+			g_slist_prepend (dst_param->input_cells, new_cell);
 	}
 	dst_param->input_cells = g_slist_reverse (dst_param->input_cells);
 
@@ -837,17 +825,14 @@ solver_insert_cols (Sheet *sheet, int col, int count)
 
 	/* Adjust the input range. */
 	input_range = value_new_cellrange_str (sheet, param->input_entry_str);
-	if (input_range != NULL) {
-	        if (input_range->v_range.cell.a.col >= col) {
-		        range.start.col = input_range->v_range.cell.a.col +
-			        count;
-		        range.start.row = input_range->v_range.cell.a.row;
-		        range.end.col   = input_range->v_range.cell.b.col +
-			        count;
-		        range.end.row   = input_range->v_range.cell.b.row;
-			param->input_entry_str =
-			        g_strdup (global_range_name (sheet, &range));
-		}
+	if (input_range != NULL &&
+	    input_range->v_range.cell.a.col >= col) {
+		range.start.col = input_range->v_range.cell.a.col + count;
+		range.start.row = input_range->v_range.cell.a.row;
+		range.end.col   = input_range->v_range.cell.b.col + count;
+		range.end.row   = input_range->v_range.cell.b.row;
+		param->input_entry_str = g_strdup (
+			global_range_name (sheet, &range));
 	}
 
 	/* Adjust the constraints. */
@@ -880,21 +865,17 @@ solver_delete_rows (Sheet *sheet, int row, int count)
 
 	/* Adjust the input range. */
 	input_range = value_new_cellrange_str (sheet, param->input_entry_str);
-	if (input_range != NULL) {
-	        if (input_range->v_range.cell.a.row >= row) {
-		        range.start.col = input_range->v_range.cell.a.col;
-		        range.start.row = input_range->v_range.cell.a.row -
-			        count;
-		        range.end.col   = input_range->v_range.cell.b.col;
-		        range.end.row   = input_range->v_range.cell.b.row -
-			        count;
-			if (range.start.row < row || range.end.row < row)
-			        param->input_entry_str = g_strdup ("");
-			else
-			        param->input_entry_str =
-				         g_strdup (global_range_name (sheet,
-								      &range));
-		}
+	if (input_range != NULL &&
+	    input_range->v_range.cell.a.row >= row) {
+		range.start.col = input_range->v_range.cell.a.col;
+		range.start.row = input_range->v_range.cell.a.row - count;
+		range.end.col   = input_range->v_range.cell.b.col;
+		range.end.row   = input_range->v_range.cell.b.row - count;
+		if (range.start.row < row || range.end.row < row)
+			param->input_entry_str = g_strdup ("");
+		else
+			param->input_entry_str = g_strdup (
+				global_range_name (sheet, &range));
 	}
 
 	/* Adjust the constraints. */
