@@ -4,6 +4,7 @@
  * wbc-gtk.c: A raw gtk based WorkbookControl
  *
  * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2006 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -21,7 +22,6 @@
  *
  * Port to Maemo:
  * 	Eduardo Lima  (eduardo.lima@indt.org.br)
- * 	Renato Araujo (renato.filho@indt.org.br)
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -49,27 +49,15 @@
 #include <goffice/utils/go-file.h>
 #include <goffice/app/go-cmd-context-impl.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkaccellabel.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhandlebox.h>
-#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtk.h>
 #include "gdk/gdkkeysyms.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <errno.h>
 #include <string.h>
 
 #ifdef USE_HILDON
-	#include <gtk/gtktable.h>
-	#include <gtk/gtktoolbutton.h>
-	#include <goffice/gtk/go-combo-text.h>
-	#include <hildon-lgpl/hildon-widgets/hildon-app.h>
-	#include <hildon-lgpl/hildon-widgets/hildon-appview.h>
+#include <hildon-widgets/hildon-window.h>
+#include <hildon-widgets/hildon-program.h>
 #endif
 
 #define CHECK_MENU_UNDERLINES
@@ -85,7 +73,8 @@ struct _WBCgtk {
 		guint		  merge_id;
 	} file_history, toolbar, windows;
 
-	GOActionComboStack	*undo_action, *redo_action;
+	GOActionComboStack	*undo_haction, *redo_haction;
+	GtkAction		*undo_vaction, *redo_vaction;
 	GOActionComboColor	*fore_color, *back_color;
 	GOActionComboText	*font_name, *font_size, *zoom;
 	GOActionComboPixmaps	*borders, *halignment, *valignment;
@@ -100,7 +89,7 @@ struct _WBCgtk {
 		GtkToggleAction	 *top, *center, *bottom;
 	} v_align;
 
-	GtkWidget *menu_zone, *toolbar_zone, *everything;
+	GtkWidget *menu_zone, *everything, *toolbar_zones[4];
 	GHashTable *custom_uis;
 };
 typedef WorkbookControlGUIClass WBCgtkClass;
@@ -138,6 +127,11 @@ wbc_gtk_create_status_area (WorkbookControlGUI *wbcg, GtkWidget *progress,
 	g_hash_table_insert (wbcg->toggle_for_fullscreen,
 		g_strdup ("ViewStatusbar"),
 		gtk_action_group_get_action (gtk->actions, "ViewStatusbar"));
+
+#ifdef USE_HILDON
+	g_hash_table_remove (wbcg->toggle_for_fullscreen, "ViewStatusbar");
+	gtk_widget_hide (gtk->status_area);
+#endif
 }
 
 /*****************************************************************************/
@@ -182,6 +176,9 @@ wbc_gtk_init_zoom (WBCgtk *gtk)
 	gtk->zoom = g_object_new (go_action_combo_text_get_type (),
 				  "name", "Zoom",
 				  "label", _("_Zoom"),
+				  "visible-vertical", FALSE,
+				  "tooltip", _("Zoom"),
+				  "stock-id", GTK_STOCK_ZOOM_IN,
 				  NULL);
 #ifdef USE_HILDON
 	go_action_combo_text_set_width (gtk->zoom,  "100000000%");
@@ -249,7 +246,7 @@ cb_border_activated (GOActionComboPixmaps *a, WorkbookControl *wbc)
 	case 11 : /* left */
 		borders[STYLE_BORDER_LEFT] = style_border_fetch (STYLE_BORDER_THIN,
 			 sheet_style_get_auto_pattern_color (sheet),
-			 style_border_get_orientation (MSTYLE_BORDER_LEFT));
+			 style_border_get_orientation (STYLE_BORDER_LEFT));
 		break;
 
 	case 12 : /* none */
@@ -260,7 +257,7 @@ cb_border_activated (GOActionComboPixmaps *a, WorkbookControl *wbc)
 	case 13 : /* right */
 		borders[STYLE_BORDER_RIGHT] = style_border_fetch (STYLE_BORDER_THIN,
 			 sheet_style_get_auto_pattern_color (sheet),
-			 style_border_get_orientation (MSTYLE_BORDER_RIGHT));
+			 style_border_get_orientation (STYLE_BORDER_RIGHT));
 		break;
 
 	case 21 : /* all */
@@ -323,7 +320,9 @@ wbc_gtk_init_borders (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->borders),
 		      "label", _("Borders"),
 		      "tooltip", _("Borders"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 #if 0
 	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Foreground"));
 	go_combo_pixmaps_select (gtk->borders, 1); /* default to none */
@@ -373,11 +372,11 @@ wbc_gtk_init_alignments (WBCgtk *gtk)
 	gtk->halignment = go_action_combo_pixmaps_new ("HAlignmentSelector",
 						       halignment_combo_info, 3, 1);
 	g_object_set (G_OBJECT (gtk->halignment),
-		      "label", _("Horzontal Alignment"),
-		      "tooltip", _("Horzontal Alignment"),
+		      "label", _("Horizontal Alignment"),
+		      "tooltip", _("Horizontal Alignment"),
 		      NULL);
 #if 0
-	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horzontal Alignment"));
+	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horizontal Alignment"));
 	go_combo_pixmaps_select (gtk->halignment, 1); /* default to none */
 #endif
 	g_signal_connect (G_OBJECT (gtk->halignment),
@@ -392,7 +391,7 @@ wbc_gtk_init_alignments (WBCgtk *gtk)
 		      "tooltip", _("Vertical Alignment"),
 		      NULL);
 #if 0
-	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horzontal Alignment"));
+	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horizontal Alignment"));
 	go_combo_pixmaps_select (gtk->valignment, 1); /* default to none */
 #endif
 	g_signal_connect (G_OBJECT (gtk->valignment),
@@ -407,7 +406,7 @@ static GOActionComboStack *
 ur_stack (WorkbookControl *wbc, gboolean is_undo)
 {
 	WBCgtk *gtk = (WBCgtk *)wbc;
-	return is_undo ? gtk->undo_action : gtk->redo_action;
+	return is_undo ? gtk->undo_haction : gtk->redo_haction;
 }
 
 static void
@@ -429,45 +428,6 @@ wbc_gtk_undo_redo_push (WorkbookControl *wbc, gboolean is_undo,
 	go_action_combo_stack_push (ur_stack (wbc, is_undo), text, key);
 }
 
-static GOActionComboStack *
-create_undo_redo (WBCgtk *gtk, char const *name, char const *tooltip,
-		  char const *stock_id, char const *accel)
-{
-	GOActionComboStack *res = g_object_new (go_action_combo_stack_get_type (),
-		"name",		name,
-		"tooltip",  	_(tooltip),
-		"stock-id",	stock_id,
-		"sensitive",	FALSE,
-		NULL);
-	
-#ifdef USE_HILDON
-	gtk_action_group_add_action (gtk->actions, GTK_ACTION (res));
-#else
-	gtk_action_group_add_action_with_accel (gtk->actions,
-		GTK_ACTION (res), accel);
-#endif
-	return res;
-}
-
-
-static void
-cb_undo_activated (GOActionComboStack *a, WorkbookControl *wbc)
-{
-	unsigned n = workbook_find_command (wb_control_workbook (wbc), TRUE,
-		go_action_combo_stack_selection (a));
-	while (n-- > 0)
-		command_undo (wbc);
-}
-
-static void
-cb_redo_activated (GOActionComboStack *a, WorkbookControl *wbc)
-{
-	unsigned n = workbook_find_command (wb_control_workbook (wbc), FALSE,
-		go_action_combo_stack_selection (a));
-	while (n-- > 0)
-		command_redo (wbc);
-}
-
 static void
 cb_chain_sensitivity (GtkAction *src, G_GNUC_UNUSED GParamSpec *pspec,
 		      GtkAction *action)
@@ -486,24 +446,85 @@ cb_chain_sensitivity (GtkAction *src, G_GNUC_UNUSED GParamSpec *pspec,
 	g_object_set (action, "sensitive", new_val, NULL);
 }
 
+static void
+create_undo_redo (GOActionComboStack **haction, char const *hname,
+		  GCallback hcb, 
+		  GtkAction **vaction, char const *vname,
+		  GCallback vcb, 
+		  WBCgtk *gtk,
+		  char const *tooltip,
+		  char const *stock_id, char const *accel)
+{
+	*haction = g_object_new
+		(go_action_combo_stack_get_type (),
+		 "name", hname,
+		 "tooltip", tooltip,
+		 "stock-id", stock_id,
+		 "sensitive", FALSE,
+		 "visible-vertical", FALSE,
+		 NULL);
+	gtk_action_group_add_action_with_accel (gtk->actions,
+		GTK_ACTION (*haction), accel);
+	g_signal_connect (G_OBJECT (*haction), "activate", hcb, gtk);
+
+	*vaction = gtk_action_new (vname, NULL, tooltip, stock_id);
+	g_object_set (G_OBJECT (*vaction),
+		      "sensitive", FALSE,
+		      "visible-horizontal", FALSE,
+		      NULL);
+	gtk_action_group_add_action (gtk->actions, GTK_ACTION (*vaction));
+	g_signal_connect_swapped (G_OBJECT (*vaction), "activate", vcb, gtk);
+
+	g_signal_connect (G_OBJECT (*haction),
+			  "notify::sensitive",
+			  G_CALLBACK (cb_chain_sensitivity),
+			  *vaction);
+}
+
+
+static void
+cb_undo_activated (GOActionComboStack *a, WorkbookControl *wbc)
+{
+	unsigned n = workbook_find_command (wb_control_get_workbook (wbc), TRUE,
+		go_action_combo_stack_selection (a));
+	while (n-- > 0)
+		command_undo (wbc);
+}
+
+static void
+cb_redo_activated (GOActionComboStack *a, WorkbookControl *wbc)
+{
+	unsigned n = workbook_find_command (wb_control_get_workbook (wbc), FALSE,
+		go_action_combo_stack_selection (a));
+	while (n-- > 0)
+		command_redo (wbc);
+}
+
 static void
 wbc_gtk_init_undo_redo (WBCgtk *gtk)
 {
-	gtk->undo_action = create_undo_redo (gtk, N_("Undo"),
-		N_("Undo the last action"), GTK_STOCK_UNDO, "<control>z");
-	g_signal_connect (G_OBJECT (gtk->undo_action),
-		"activate",
-		G_CALLBACK (cb_undo_activated), gtk);
-	g_signal_connect (G_OBJECT (gtk->undo_action),
-		"notify::sensitive",
-		G_CALLBACK (cb_chain_sensitivity),
-		gtk_action_group_get_action (gtk->permanent_actions, "Repeat"));
+	GtkAction *repeat =
+		gtk_action_group_get_action (gtk->permanent_actions, "Repeat");
 
-	gtk->redo_action = create_undo_redo (gtk, N_("Redo"),
-		N_("Redo the undone action"), GTK_STOCK_REDO, "<control>y");
-	g_signal_connect (G_OBJECT (gtk->redo_action),
-		"activate",
-		G_CALLBACK (cb_redo_activated), gtk);
+	create_undo_redo (&gtk->undo_haction, "Undo",
+			  G_CALLBACK (cb_undo_activated),
+			  &gtk->undo_vaction, "VUndo",
+			  G_CALLBACK (command_undo),
+			  gtk,
+			  _("Undo the last action"),
+			  GTK_STOCK_UNDO, "<control>z");
+	g_signal_connect (G_OBJECT (gtk->undo_haction),
+			  "notify::sensitive",
+			  G_CALLBACK (cb_chain_sensitivity),
+			  repeat);
+
+	create_undo_redo (&gtk->redo_haction, "Redo",
+			  G_CALLBACK (cb_redo_activated),
+			  &gtk->redo_vaction, "VRedo",
+			  G_CALLBACK (command_redo),
+			  gtk,
+			  _("Redo the undone action"),
+			  GTK_STOCK_REDO, "<control>y");
 }
 
 /****************************************************************************/
@@ -557,7 +578,9 @@ wbc_gtk_init_color_fore (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->fore_color),
 		      "label", _("Foreground"),
 		      "tooltip", _("Foreground"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 	g_signal_connect (G_OBJECT (gtk->fore_color),
 		"activate",
 		G_CALLBACK (cb_fore_color_changed), gtk);
@@ -606,7 +629,9 @@ wbc_gtk_init_color_back (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->back_color),
 		      "label", _("Background"),
 		      "tooltip", _("Background"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 	g_object_connect (G_OBJECT (gtk->back_color),
 		"signal::activate", G_CALLBACK (cb_back_color_changed), gtk,
 		"signal::display-custom-dialog", G_CALLBACK (cb_custom_color_created), gtk,
@@ -651,8 +676,13 @@ wbc_gtk_init_font_name (WBCgtk *gtk)
 	gtk->font_name = g_object_new (go_action_combo_text_get_type (),
 				       "name", "FontName",
 				       "case-sensitive", FALSE,
+				       "stock-id", GTK_STOCK_SELECT_FONT,
+				       "visible-vertical", FALSE,
+				       "tooltip", _("Font"),
 				       NULL);
 
+	/* TODO: Create vertical version of this.  */
+
 	context = gtk_widget_get_pango_context
 		(GTK_WIDGET (wbcg_toplevel (WORKBOOK_CONTROL_GUI (gtk))));
 	families = go_fonts_list_families (context);
@@ -705,7 +735,14 @@ wbc_gtk_init_font_size (WBCgtk *gtk)
 
 	gtk->font_size = g_object_new (go_action_combo_text_get_type (),
 				       "name", "FontSize",
+				       "stock-id", GTK_STOCK_SELECT_FONT,
+				       "visible-vertical", FALSE,
+				       "label", _("Font Size"),
+				       "tooltip", _("Font Size"),
 				       NULL);
+
+	/* TODO: Create vertical version of this.  */
+
 	font_sizes = go_fonts_list_sizes ();
 	for (ptr = font_sizes; ptr != NULL ; ptr = ptr->next) {
 		int psize = GPOINTER_TO_INT (ptr->data);
@@ -714,7 +751,11 @@ wbc_gtk_init_font_size (WBCgtk *gtk)
 		g_free (size_text);
 	}
 	g_slist_free (font_sizes);
+#ifdef USE_HILDON
+	go_action_combo_text_set_width (gtk->font_size,  "888888");
+#else
 	go_action_combo_text_set_width (gtk->font_size, "888");
+#endif
 	g_signal_connect (G_OBJECT (gtk->font_size),
 		"activate",
 		G_CALLBACK (cb_font_size_changed), gtk);
@@ -754,7 +795,7 @@ wbc_gtk_reload_recent_file_menu (WorkbookControlGUI const *wbcg)
 	for (i = 1; ptr != NULL ; ptr = ptr->next, i++) {
 		GtkActionEntry entry;
 		GtkAction *action;
-		const char *uri = ptr->data;
+		char const *uri = ptr->data;
 		char *name = g_strdup_printf ("FileHistoryEntry%d", i);
 		char *label = history_item_label (uri, i);
 		char *filename = go_filename_from_uri (uri);
@@ -787,7 +828,11 @@ wbc_gtk_reload_recent_file_menu (WorkbookControlGUI const *wbcg)
 	while (i-- > 1) {
 		char *name = g_strdup_printf ("FileHistoryEntry%d", i);
 		gtk_ui_manager_add_ui (gtk->ui, gtk->file_history.merge_id,
-			"/menubar/File/FileHistory", 
+#ifdef USE_HILDON
+			"/popup/File/FileHistory",
+#else
+			"/menubar/File/FileHistory",
+#endif
 			name, name, GTK_UI_MANAGER_AUTO, TRUE);
 		g_free (name);
 	}
@@ -806,6 +851,35 @@ wbc_gtk_set_action_sensitivity (WorkbookControlGUI const *wbcg,
 	g_object_set (G_OBJECT (a), "sensitive", sensitive, NULL);
 }
 
+#ifdef USE_HILDON
+static void
+wbc_gtk_hildon_set_action_sensitive (WBCgtk      *gtk,
+				     const gchar *action,
+				     gboolean     sensitive)
+{
+	GtkWidget * toolbar = gtk_ui_manager_get_widget (gtk->ui, "/StandardToolbar");
+	GList * children = gtk_container_get_children (GTK_CONTAINER (toolbar));
+	GList * l;
+	
+	for (l = children; l != NULL; l = g_list_next (l)) {
+		if (GTK_IS_SEPARATOR_TOOL_ITEM (l->data) == FALSE) {
+			gchar * label = NULL;
+			g_object_get (GTK_TOOL_ITEM (l->data), "label", &label, NULL);
+			
+			if (label != NULL && strstr (label, action) != NULL) {
+				g_object_set (G_OBJECT (l->data), "sensitive", sensitive, NULL);
+				g_free(label);
+				break;
+			}
+
+			g_free(label);
+		}
+	}
+
+	g_list_free (children);
+}
+#endif
+
 /* NOTE : The semantics of prefix and suffix seem contrived.  Why are we
  * handling it at this end ?  That stuff should be done in the undo/redo code
  **/
@@ -820,47 +894,20 @@ wbc_gtk_set_action_label (WorkbookControlGUI const *wbcg,
 	GtkAction *a = gtk_action_group_get_action (gtk->actions, action);
 
 	if (prefix != NULL) {
-		gchar    *text;
-		gboolean  sensitive = TRUE;
-
-		if (suffix == NULL) {
-#ifndef USE_HILDON
-			suffix = _("Nothing");
-#endif
-			sensitive = FALSE;
-		}
+		char *text;
+		gboolean is_suffix = (suffix != NULL);
 
 #ifdef USE_HILDON
-		text = g_strdup (prefix);
-
-		{
-			/* Set hildon toolbuttons (in)sensitive */
-			GtkToolbar * toolbar = hildon_appview_get_toolbar (HILDON_APPVIEW (wbcg->toplevel));
-			GList * l;
-			
-			for (l = gtk_container_get_children (GTK_CONTAINER (toolbar)); l; l = l->next) {
-				if (GTK_IS_TOOL_BUTTON (l->data)) {
-					gchar * label;
-					g_object_get (l->data, "label", &label, NULL);
-					
-					if (strstr (label, action) != NULL) {
-						g_object_set (G_OBJECT (l->data), "sensitive", sensitive, NULL);
-						g_free(label);
-						break;
-					}
-
-					g_free(label);
-				}
-			}
-		}
-#else
-		text = g_strdup_printf ("%s : %s", prefix, suffix);
+		wbc_gtk_hildon_set_action_sensitive (gtk, action, is_suffix);
 #endif
-		g_object_set (G_OBJECT (a),
-			      "label",	   text,
-			      "sensitive", sensitive,
-			      NULL);
-		g_free (text);
+
+		text = is_suffix ? g_strdup_printf ("%s: %s", prefix, suffix) : (char *) prefix;
+			g_object_set (G_OBJECT (a),
+				      "label",	   text,
+				      "sensitive", is_suffix,
+				      NULL);
+		if (is_suffix)
+			g_free (text);
 	} else
 		g_object_set (G_OBJECT (a), "label", suffix, NULL);
 
@@ -1001,12 +1048,12 @@ cb_handlebox_dock_status (GtkHandleBox *hb,
 
 #ifdef CHECK_MENU_UNDERLINES
 
-static const char *
+static char const *
 get_accel_label (GtkMenuItem *item, guint *key)
 {
 	GList *children = gtk_container_get_children (GTK_CONTAINER (item));
 	GList *l;
-	const char *res = NULL;
+	char const *res = NULL;
 
 	*key = GDK_VoidSymbol;
 	for (l = children; l; l = l->next) {
@@ -1024,7 +1071,7 @@ get_accel_label (GtkMenuItem *item, guint *key)
 }
 
 static void
-check_underlines (GtkWidget *w, const char *path)
+check_underlines (GtkWidget *w, char const *path)
 {
 	GList *children = gtk_container_get_children (GTK_CONTAINER (w));
 	GHashTable *used = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify)g_free);
@@ -1034,7 +1081,7 @@ check_underlines (GtkWidget *w, const char *path)
 		GtkMenuItem *item = GTK_MENU_ITEM (l->data);
 		GtkWidget *sub = gtk_menu_item_get_submenu (item);
 		guint key;
-		const char *label = get_accel_label (item, &key);
+		char const *label = get_accel_label (item, &key);
 
 		if (sub) {
 			char *newpath = g_strconcat (path, *path ? "->" : "", label, NULL);
@@ -1043,7 +1090,7 @@ check_underlines (GtkWidget *w, const char *path)
 		}
 
 		if (key != GDK_VoidSymbol) {
-			const char *prev = g_hash_table_lookup (used, GUINT_TO_POINTER (key));
+			char const *prev = g_hash_table_lookup (used, GUINT_TO_POINTER (key));
 			if (prev) {
 				/* xgettext: Translators: if this warning shows up when
 				 * running Gnumeric in your locale, the underlines need
@@ -1091,10 +1138,10 @@ regenerate_window_menu (WBCgtk *gtk, Workbook *wb, unsigned i)
 		if (i >= 20)
 			return i;
 		if (IS_WORKBOOK_CONTROL_GUI (wbc) &&
-			(basename = go_basename_from_uri (wb->uri)) != NULL) {
+			(basename = go_basename_from_uri (GO_DOC (wb)->uri)) != NULL) {
 			GString *label = g_string_new (NULL);
 			char *name;
-			const char *s;
+			char const *s;
 			GtkActionEntry entry;
 
 			if (i < 10) g_string_append_c (label, '_');
@@ -1130,10 +1177,14 @@ regenerate_window_menu (WBCgtk *gtk, Workbook *wb, unsigned i)
 static void
 cb_regenerate_window_menu (WBCgtk *gtk)
 {
-	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (gtk));
+	Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (gtk));
 	GList const *ptr;
 	unsigned i;
 
+	/* This can happen during exit.  */
+	if (!wb)
+		return;
+
 	if (gtk->windows.merge_id != 0)
 		gtk_ui_manager_remove_ui (gtk->ui, gtk->windows.merge_id);
 	gtk->windows.merge_id = gtk_ui_manager_new_merge_id (gtk->ui);
@@ -1154,7 +1205,11 @@ cb_regenerate_window_menu (WBCgtk *gtk)
 	while (i-- > 1) {
 		char *name = g_strdup_printf ("WindowListEntry%d", i);
 		gtk_ui_manager_add_ui (gtk->ui, gtk->windows.merge_id,
-			"/menubar/View/Windows", 
+#ifdef USE_HILDON
+			"/popup/View/Windows",
+#else
+			"/menubar/View/Windows",
+#endif
 			name, name, GTK_UI_MANAGER_AUTO, TRUE);
 		g_free (name);
 	}
@@ -1232,6 +1287,158 @@ cb_init_extra_ui (GnmAppExtraUI *extra_ui, WBCgtk *gtk)
 /****************************************************************************/
 /* Toolbar menu */
 
+static void
+set_toolbar_style_for_position (GtkToolbar *tb, GtkPositionType pos)
+{
+	GtkHandleBox *hdlbox = GTK_HANDLE_BOX (GTK_WIDGET (tb)->parent);
+
+	static const GtkPositionType hdlpos[] = {
+		GTK_POS_TOP, GTK_POS_TOP,
+		GTK_POS_LEFT, GTK_POS_LEFT
+	};
+	static const GtkOrientation orientations[] = {
+		GTK_ORIENTATION_VERTICAL, GTK_ORIENTATION_VERTICAL,
+		GTK_ORIENTATION_HORIZONTAL, GTK_ORIENTATION_HORIZONTAL
+	};
+
+	gtk_toolbar_set_orientation (tb, orientations[pos]);
+	gtk_handle_box_set_handle_position (hdlbox, hdlpos[pos]);
+}
+
+static void
+set_toolbar_position (GtkToolbar *tb, GtkPositionType pos, WBCgtk *gtk)
+{
+	GtkWidget *hdlbox = GTK_WIDGET (tb)->parent;
+	GtkContainer *zone = GTK_CONTAINER (GTK_WIDGET (hdlbox)->parent);
+	GtkContainer *new_zone = GTK_CONTAINER (gtk->toolbar_zones[pos]);
+	char const *name = g_object_get_data (G_OBJECT (hdlbox), "name");
+
+	if (zone == new_zone)
+		return;
+
+	g_object_ref (hdlbox);
+	if (zone)
+		gtk_container_remove (zone, hdlbox);
+	set_toolbar_style_for_position (tb, pos);
+	gtk_container_add (new_zone, hdlbox);
+	g_object_unref (hdlbox);
+
+	if (zone)
+		gnm_gconf_set_toolbar_position (name, pos);
+}
+
+static void
+cb_set_toolbar_position (GtkMenuItem *item, WBCgtk *gtk)
+{
+	GtkToolbar *tb = g_object_get_data (G_OBJECT (item), "toolbar");
+	GtkPositionType side = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "side"));
+
+	set_toolbar_position (tb, side, gtk);
+}
+
+static void
+cb_tcm_reattach (GtkWidget *widget, GtkHandleBox *hdlbox)
+{
+	GdkEvent *event = gdk_event_new (GDK_DELETE);
+	event->any.type = GDK_DELETE;
+	event->any.window = g_object_ref (hdlbox->float_window);
+	event->any.send_event = TRUE;
+	gtk_main_do_event (event);
+	gdk_event_free (event);
+}
+
+static void
+cb_tcm_hide (GtkWidget *widget, GtkHandleBox *hdlbox)
+{
+	if (hdlbox->child_detached)
+		cb_tcm_reattach (widget, hdlbox);
+	gtk_widget_hide (GTK_WIDGET (hdlbox));
+}
+
+static void
+toolbar_context_menu (GtkToolbar *tb, WBCgtk *gtk, GdkEventButton *event_button)
+{
+	GtkHandleBox *hdlbox = GTK_HANDLE_BOX (GTK_WIDGET (tb)->parent);
+	GtkWidget *zone = GTK_WIDGET (hdlbox)->parent;
+	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *item;
+
+	static struct {
+		char const *text;
+		GtkPositionType pos;
+	} const pos_items[] = {
+		{ N_("Display above sheets"), GTK_POS_TOP },
+		{ N_("Display to the left of sheets"), GTK_POS_LEFT },
+		{ N_("Display to the right of sheets"), GTK_POS_RIGHT }
+	};
+
+	if (hdlbox->child_detached) {
+		item = gtk_menu_item_new_with_label (_("Reattach to main window"));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (cb_tcm_reattach),
+				  hdlbox);
+	} else {
+		size_t ui;
+		GSList *group = NULL;
+
+		for (ui = 0; ui < G_N_ELEMENTS (pos_items); ui++) {
+			char const *text = _(pos_items[ui].text);
+			GtkPositionType pos = pos_items[ui].pos;
+
+			item = gtk_radio_menu_item_new_with_label (group, text);
+			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
+
+			GTK_CHECK_MENU_ITEM (item)->active =
+				(zone == gtk->toolbar_zones[pos]);
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			g_object_set_data (G_OBJECT (item), "toolbar", tb);
+			g_object_set_data (G_OBJECT (item), "side", GINT_TO_POINTER (pos));
+			g_signal_connect (G_OBJECT (item), "activate",
+					  G_CALLBACK (cb_set_toolbar_position),
+					  gtk);
+		}
+	}
+
+	item = gtk_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	gtk_widget_set_sensitive (item, FALSE);
+
+	item = gtk_menu_item_new_with_label (_("Hide"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (cb_tcm_hide),
+			  hdlbox);
+
+	gtk_widget_show_all (menu);
+	gnumeric_popup_menu (GTK_MENU (menu), event_button);
+}
+
+static gboolean
+cb_toolbar_button_press (GtkToolbar *tb, GdkEventButton *event, WBCgtk *gtk)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		toolbar_context_menu (tb, gtk, event);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+cb_handlebox_button_press (GtkHandleBox *hdlbox, GdkEventButton *event, WBCgtk *gtk)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		GtkToolbar *tb = GTK_TOOLBAR (gtk_bin_get_child (GTK_BIN (hdlbox)));
+		toolbar_context_menu (tb, gtk, event);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 static void
 cb_toolbar_activate (GtkToggleAction *action, WorkbookControlGUI *wbcg)
 {
@@ -1244,7 +1451,7 @@ cb_handlebox_visible (GtkWidget *box, G_GNUC_UNUSED GParamSpec *pspec,
 {
 	GtkToggleAction *toggle_action = g_object_get_data (
 		G_OBJECT (box), "toggle_action");
-	const char *name = g_object_get_data (G_OBJECT (box), "name");
+	char const *name = g_object_get_data (G_OBJECT (box), "name");
 	gboolean visible = GTK_WIDGET_VISIBLE (box);
 
 	gtk_toggle_action_set_active (toggle_action, visible);
@@ -1258,33 +1465,49 @@ cb_add_menus_toolbars (G_GNUC_UNUSED GtkUIManager *ui,
 	if (GTK_IS_TOOLBAR (w)) {
 		WorkbookControlGUI *wbcg = (WorkbookControlGUI *)gtk;
 		char const *name = gtk_widget_get_name (w);
-#ifdef USE_HILDON
-		if (0 == strcmp (name, "HildonToolbar")) {
-			hildon_appview_set_toolbar (HILDON_APPVIEW (wbcg->toplevel), GTK_TOOLBAR (w));
-			gtk_widget_show_all (w);
-		} else if (0 == strcmp (name, "HildonTopToolbar")) {
-			gtk_table_attach (GTK_TABLE (wbcg->table), w, 1, 2, 0, 1,
-					  GTK_FILL | GTK_EXPAND , 0, 0, 0);
-		}
-#else
-		GtkWidget *box = gtk_handle_box_new ();
 		GtkToggleActionEntry entry;
 		char *toggle_name = g_strdup_printf ("ViewMenuToolbar%s", name);
 		char *tooltip = g_strdup_printf (_("Show/Hide toolbar %s"), _(name));
 		gboolean visible = gnm_gconf_get_toolbar_visible (name);
 
+#ifdef USE_HILDON
+		hildon_window_add_toolbar (HILDON_WINDOW (wbcg_toplevel (wbcg)), GTK_TOOLBAR (w));
+
+		gtk_widget_show_all (w);
+		g_hash_table_insert (wbcg->visibility_widgets,
+			g_strdup (toggle_name), g_object_ref (w));
+#else
+		GtkWidget *box = gtk_handle_box_new ();
+		GtkPositionType pos = gnm_gconf_get_toolbar_position (name);
+		g_signal_connect (G_OBJECT (w),
+				  "button_press_event",
+				  G_CALLBACK (cb_toolbar_button_press),
+				  gtk);
+		g_signal_connect (G_OBJECT (box),
+				  "button_press_event",
+				  G_CALLBACK (cb_handlebox_button_press),
+				  gtk);
+
 		gtk_container_add (GTK_CONTAINER (box), w);
-		gtk_toolbar_set_show_arrow (GTK_TOOLBAR (w), TRUE);
-		gtk_toolbar_set_style (GTK_TOOLBAR (w), GTK_TOOLBAR_ICONS);
 		gtk_widget_show_all (box);
 		if (!visible)
 			gtk_widget_hide (box);
-		gtk_box_pack_start (GTK_BOX (gtk->toolbar_zone), box, FALSE, FALSE, 0);
+		set_toolbar_position (GTK_TOOLBAR (w), pos, gtk);
+
 		g_object_connect (box,
 			"signal::notify::visible", G_CALLBACK (cb_handlebox_visible), wbcg,
 			"signal::child_attached", G_CALLBACK (cb_handlebox_dock_status), GINT_TO_POINTER (TRUE),
 			"signal::child_detached", G_CALLBACK (cb_handlebox_dock_status), GINT_TO_POINTER (FALSE),
 			NULL);
+		g_object_set_data_full (G_OBJECT (box), "name",
+					g_strdup (name),
+					(GDestroyNotify)g_free);
+
+		g_hash_table_insert (wbcg->visibility_widgets,
+			g_strdup (toggle_name), g_object_ref (box));
+#endif
+		gtk_toolbar_set_show_arrow (GTK_TOOLBAR (w), TRUE);
+		gtk_toolbar_set_style (GTK_TOOLBAR (w), GTK_TOOLBAR_ICONS);
 
 		entry.name = toggle_name;
 		entry.stock_id = NULL;
@@ -1295,17 +1518,15 @@ cb_add_menus_toolbars (G_GNUC_UNUSED GtkUIManager *ui,
 		entry.is_active = visible;
 		gtk_action_group_add_toggle_actions (gtk->toolbar.actions,
 			&entry, 1, (WorkbookControlGUI *)wbcg);
-		gtk_ui_manager_add_ui (gtk->ui, gtk->toolbar.merge_id,
-			"/menubar/View/Toolbars", 
-			toggle_name, toggle_name, GTK_UI_MANAGER_AUTO, FALSE);
 		g_object_set_data (G_OBJECT (box), "toggle_action",
 			gtk_action_group_get_action (gtk->toolbar.actions, toggle_name));
-		g_object_set_data_full (G_OBJECT (box), "name",
-					g_strdup (name),
-					(GDestroyNotify)g_free);
-
-		g_hash_table_insert (wbcg->visibility_widgets,
-			g_strdup (toggle_name), g_object_ref (box));
+		gtk_ui_manager_add_ui (gtk->ui, gtk->toolbar.merge_id,
+#ifdef USE_HILDON
+			"/popup/View/Toolbars",
+#else
+			"/menubar/View/Toolbars",
+#endif
+			toggle_name, toggle_name, GTK_UI_MANAGER_AUTO, FALSE);
 		g_hash_table_insert (wbcg->toggle_for_fullscreen,
 			g_strdup (toggle_name),
 			gtk_action_group_get_action (gtk->toolbar.actions,
@@ -1313,7 +1534,6 @@ cb_add_menus_toolbars (G_GNUC_UNUSED GtkUIManager *ui,
 
 		g_free (tooltip);
 		g_free (toggle_name);
-#endif
 	} else {
 		gtk_box_pack_start (GTK_BOX (gtk->menu_zone), w, FALSE, TRUE, 0);
 		gtk_widget_show_all (w);
@@ -1404,6 +1624,12 @@ cb_wbcg_window_state_event (GtkWidget           *widget,
 	wbcg->toggle_for_fullscreen = tmp;
 }
 
+#ifndef HILDON
+char const *uifilename = NULL;
+GtkActionEntry const *extra_actions = NULL;
+int nb_extra_actions = 0;
+#endif
+
 static void
 wbc_gtk_init (GObject *obj)
 {
@@ -1435,48 +1661,71 @@ wbc_gtk_init (GObject *obj)
 	char		   *uifile;
 	unsigned	    i;
 	GError *error = NULL;
-
-	gtk->menu_zone = gtk_vbox_new (TRUE, 0);
-	gtk->toolbar_zone = gtk_vbox_new (FALSE, 0);
-	gtk->everything = gtk_vbox_new (FALSE, 0);
+	GtkWidget *hbox;
 
 #ifdef USE_HILDON
-	{ GtkWidget *appview = hildon_appview_new ("Gnumeric");
-	wbcg->hildon_app = hildon_app_new_with_appview (HILDON_APPVIEW (appview));
-	wbcg_set_toplevel (wbcg, appview); }
+	static HildonProgram * hildon_program = NULL;
+#endif
+
+	gtk->menu_zone = gtk_vbox_new (TRUE, 0);
+	gtk->everything = gtk_vbox_new (FALSE, 0);
+
+	gtk->toolbar_zones[GTK_POS_TOP] = gtk_vbox_new (FALSE, 0);
+	gtk->toolbar_zones[GTK_POS_BOTTOM] = NULL;
+	gtk->toolbar_zones[GTK_POS_LEFT] = gtk_hbox_new (FALSE, 0);
+	gtk->toolbar_zones[GTK_POS_RIGHT] = gtk_hbox_new (FALSE, 0);
+
+#ifdef USE_HILDON
+	if (hildon_program == NULL)
+		hildon_program = HILDON_PROGRAM (hildon_program_get_instance ());
+	else 
+		g_object_ref (hildon_program);
+
+	wbcg->hildon_prog = hildon_program;
+
+	wbcg_set_toplevel (wbcg, hildon_window_new ());
+	hildon_program_add_window (wbcg->hildon_prog, HILDON_WINDOW (wbcg_toplevel (wbcg)));
 #else
 	wbcg_set_toplevel (wbcg, gtk_window_new (GTK_WINDOW_TOPLEVEL));
+#endif
+
 	g_signal_connect (wbcg_toplevel (wbcg), "window_state_event",
 			  G_CALLBACK (cb_wbcg_window_state_event),
 			  wbcg);
+
+#ifndef USE_HILDON
 	gtk_box_pack_start (GTK_BOX (gtk->everything),
 		gtk->menu_zone, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (gtk->everything),
-		gtk->toolbar_zone, FALSE, TRUE, 0);
+		gtk->toolbar_zones[GTK_POS_TOP], FALSE, TRUE, 0);
 #endif
 
 	gtk_window_set_title (wbcg_toplevel (wbcg), "Gnumeric");
 	gtk_window_set_wmclass (wbcg_toplevel (wbcg), "Gnumeric", "Gnumeric");
 
-#ifdef USE_HILDON
-	gtk_widget_show_all (GTK_WIDGET (wbcg_toplevel (wbcg)));
-#endif
 #if 0
 	bonobo_dock_set_client_area (BONOBO_DOCK (gtk->dock), wbcg->table);
 #endif
-	gtk_box_pack_start (GTK_BOX (gtk->everything),
-		wbcg->table, TRUE, TRUE, 0);
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk->toolbar_zones[GTK_POS_LEFT], FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), wbcg->table, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk->toolbar_zones[GTK_POS_RIGHT], FALSE, TRUE, 0);
+
+	gtk_box_pack_start (GTK_BOX (gtk->everything), hbox, TRUE, TRUE, 0);
 	gtk_widget_show_all (gtk->everything);
 
 #warning "TODO split into smaller chunks"
 	gtk->permanent_actions = gtk_action_group_new ("PermanentActions");
-	gtk_action_group_set_translation_domain (gtk->permanent_actions, NULL);
+	gtk_action_group_set_translation_domain (gtk->permanent_actions, GETTEXT_PACKAGE);
 	gtk->actions = gtk_action_group_new ("Actions");
-	gtk_action_group_set_translation_domain (gtk->actions, NULL);
+	gtk_action_group_set_translation_domain (gtk->actions, GETTEXT_PACKAGE);
 	gtk->font_actions = gtk_action_group_new ("FontActions");
-	gtk_action_group_set_translation_domain (gtk->font_actions, NULL);
+	gtk_action_group_set_translation_domain (gtk->font_actions, GETTEXT_PACKAGE);
 
 	wbcg_register_actions (wbcg, gtk->permanent_actions, gtk->actions, gtk->font_actions);
+	if (extra_actions)
+		gtk_action_group_add_actions (gtk->actions, extra_actions, nb_extra_actions, wbcg);
 
 	for (i = G_N_ELEMENTS (toggles); i-- > 0 ; ) {
 		act = gtk_action_group_get_action (
@@ -1507,11 +1756,13 @@ wbc_gtk_init (GObject *obj)
 	gtk_window_add_accel_group (wbcg_toplevel (wbcg), 
 		gtk_ui_manager_get_accel_group (gtk->ui));
 
-#ifdef USE_HILDON	
+#ifdef USE_HILDON
 	uifile = g_build_filename (gnm_sys_data_dir (), "HILDON_Gnumeric-gtk.xml", NULL);
 #else
-	uifile = g_build_filename (gnm_sys_data_dir (), "GNOME_Gnumeric-gtk.xml", NULL);
+	uifile = g_build_filename (gnm_sys_data_dir (),
+		(uifilename? uifilename: "GNOME_Gnumeric-gtk.xml"), NULL);
 #endif
+
 	if (!gtk_ui_manager_add_ui_from_file (gtk->ui, uifile, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
@@ -1543,26 +1794,33 @@ wbc_gtk_init (GObject *obj)
 
 	gtk_ui_manager_ensure_update (gtk->ui);
 
-#ifdef USE_HILDON
-	{
-		GtkWidget *hildonMenu = GTK_WIDGET (hildon_appview_get_menu (HILDON_APPVIEW (wbcg->toplevel)));
-		GList *list = gtk_container_get_children (GTK_CONTAINER (gtk->menu_zone));
-		
-		for (list = gtk_container_get_children (GTK_CONTAINER (list->data)); list; list = list->next) {
-			gtk_widget_reparent (GTK_WIDGET (list->data), hildonMenu);
-		}
-
-		gtk_widget_show_all (GTK_WIDGET (hildonMenu));
-	}
-#endif
-
 	gtk_container_add (GTK_CONTAINER (wbcg->toplevel), gtk->everything);
 
+	/*
+	 * The following line updates the menus before the check so we
+	 * avoid problems like #324692.  We could move it inside the
+	 * conditional, but I don't like doing active things like this
+	 * in there.
+	 */
+	wb_control_undo_redo_labels (WORKBOOK_CONTROL (wbcg), "", "");
+
 #ifdef CHECK_MENU_UNDERLINES
 	gtk_container_foreach (GTK_CONTAINER (gtk->menu_zone),
 			       (GtkCallback)check_underlines,
 			       (gpointer)"");
 #endif
+
+#ifdef USE_HILDON
+	hildon_window_set_menu (HILDON_WINDOW (wbcg_toplevel (wbcg)),
+				GTK_MENU (gtk_ui_manager_get_widget (gtk->ui, "/popup")));
+
+	gtk_widget_show_all (wbcg->toplevel);
+	
+	wbc_gtk_set_toggle_action_state (wbcg, "ViewMenuToolbarFormatToolbar", FALSE);
+	wbc_gtk_set_toggle_action_state (wbcg, "ViewMenuToolbarObjectToolbar", FALSE);
+	wbc_gtk_set_toggle_action_state (wbcg, "ViewSheets", FALSE);
+	wbc_gtk_set_toggle_action_state (wbcg, "ViewStatusbar", FALSE);
+#endif
 }
 
 static void
