@@ -9,7 +9,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "gnumeric-expr-entry.h"
 
@@ -34,6 +34,7 @@
 #include <gtk/gtkcelleditable.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtktogglebutton.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
@@ -145,6 +146,132 @@ gee_destroy (GtkObject *object)
 	parent_class->destroy (object);
 }
 
+static void
+cb_icon_clicked (GtkButton *icon,
+		 GnmExprEntry *entry)
+{
+	GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
+
+	/* TODO special-case GnmExprEntry being directly packed
+	 * into a GtkWindow. Currently, we just use it in
+	 * GtkDialogs so the current window child widget
+	 * is never identical to the entry when it is
+	 * not rolled up.
+	 */
+
+	if (toplevel != NULL && (GTK_WIDGET_FLAGS (toplevel) & GTK_TOPLEVEL)) {
+		GtkWidget *old_entry_parent;
+		GtkWidget *old_toplevel_child;
+		GParamSpec **container_props_pspec;
+		GValueArray *container_props;
+
+		g_assert (GTK_IS_WINDOW (toplevel));
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (icon))) {
+			int n, width, height;
+
+			/* roll-up request */
+
+			old_toplevel_child = gtk_bin_get_child (GTK_BIN (toplevel));
+			g_assert (GTK_IS_WIDGET (old_toplevel_child));
+
+			old_entry_parent = gtk_widget_get_parent (GTK_WIDGET (entry));
+			g_assert (GTK_IS_CONTAINER (old_entry_parent));
+
+			g_object_set_data_full (G_OBJECT (entry), "old_entry_parent",
+						gtk_widget_ref (old_entry_parent),
+						(GDestroyNotify) gtk_widget_unref);
+
+			g_return_if_fail ((GtkWidget *) entry != old_toplevel_child);
+
+			g_object_set_data_full (G_OBJECT (entry), "old_toplevel_child",
+						gtk_widget_ref (old_toplevel_child),
+						(GDestroyNotify) gtk_widget_unref);
+
+			gtk_window_get_size (GTK_WINDOW (toplevel), &width, &height);
+			g_object_set_data (G_OBJECT (entry), "old_window_width", GUINT_TO_POINTER (width));
+			g_object_set_data (G_OBJECT (entry), "old_window_height", GUINT_TO_POINTER (height));
+
+			container_props = NULL;
+
+			container_props_pspec = gtk_container_class_list_child_properties
+					(G_OBJECT_GET_CLASS (old_entry_parent), &n);
+			if (container_props_pspec != NULL) {
+				int i;
+
+				g_assert (n >= 1);
+
+				container_props = g_value_array_new (n);
+
+				for (i = 0; i < n; i++) {
+					GValue value = { 0 };
+					g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (container_props_pspec[i]));
+
+					gtk_container_child_get_property (GTK_CONTAINER (old_entry_parent), GTK_WIDGET (entry),
+									  g_param_spec_get_name (container_props_pspec[i]),
+									  &value);
+					g_value_array_append (container_props, &value);
+				}
+			}
+
+			g_object_set_data_full (G_OBJECT (entry), "container_props",
+						container_props,
+						(GDestroyNotify) g_value_array_free);
+			g_object_set_data_full (G_OBJECT (entry), "container_props_pspec",
+						container_props_pspec,
+						(GDestroyNotify) g_free);
+
+			gtk_container_remove (GTK_CONTAINER (toplevel), old_toplevel_child);
+			gtk_widget_reparent (GTK_WIDGET (entry), toplevel);
+
+			gtk_widget_grab_focus (GTK_WIDGET (entry->entry));
+			gtk_window_resize (GTK_WINDOW (toplevel), 1, 1);
+
+		} else {
+			int i;
+
+			/* reset rolled-up window */
+
+			old_toplevel_child = g_object_get_data (G_OBJECT (entry), "old_toplevel_child");
+			g_assert (GTK_IS_WIDGET (old_toplevel_child));
+
+			old_entry_parent = g_object_get_data (G_OBJECT (entry), "old_entry_parent");
+			g_assert (GTK_IS_CONTAINER (old_entry_parent));
+
+			gtk_widget_ref (GTK_WIDGET (entry));
+			gtk_container_remove (GTK_CONTAINER (toplevel), GTK_WIDGET (entry));
+			gtk_container_add (GTK_CONTAINER (toplevel), old_toplevel_child);
+			gtk_container_add (GTK_CONTAINER (old_entry_parent), GTK_WIDGET (entry));
+			gtk_widget_unref (GTK_WIDGET (entry));
+
+			container_props = g_object_get_data (G_OBJECT (entry), "container_props");
+			container_props_pspec = g_object_get_data (G_OBJECT (entry), "container_props_pspec");
+
+			g_assert (!(container_props != NULL && container_props_pspec == NULL));
+			g_assert (!(container_props == NULL && container_props_pspec != NULL));
+
+			if (container_props_pspec != NULL) {
+				for (i = 0; container_props_pspec[i] != NULL; i++) {
+					gtk_container_child_set_property (GTK_CONTAINER (old_entry_parent), GTK_WIDGET (entry),
+									  g_param_spec_get_name (container_props_pspec[i]),
+									  g_value_array_get_nth (container_props, i));
+				}
+			}
+
+			gtk_window_resize (GTK_WINDOW (toplevel),
+					   GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (entry), "old_window_width")),
+					   GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (entry), "old_window_height")));
+
+			g_object_set_data (G_OBJECT (entry), "old_entry_parent", NULL);
+			g_object_set_data (G_OBJECT (entry), "old_toplevel_child", NULL);
+			g_object_set_data (G_OBJECT (entry), "container_props", NULL);
+			g_object_set_data (G_OBJECT (entry), "container_props_pspec", NULL);
+		}
+	} else {
+		g_warning ("GnmExprEntry button was clicked, but entry has no toplevel parent.");
+	}
+}
+
 static void
 gee_set_property (GObject      *object,
 		  guint         prop_id,
@@ -160,10 +287,14 @@ gee_set_property (GObject      *object,
 	case PROP_WITH_ICON:
 		if (g_value_get_boolean (value)) {
 			if (gee->icon == NULL) {
-				gee->icon = gtk_image_new_from_stock (
-					"Gnumeric_ExprEntry", GTK_ICON_SIZE_BUTTON);
+				gee->icon = gtk_toggle_button_new ();
+				gtk_container_add (GTK_CONTAINER (gee->icon),
+						   gtk_image_new_from_stock ("Gnumeric_ExprEntry",
+							   		     GTK_ICON_SIZE_BUTTON));
 				gtk_box_pack_start (GTK_BOX (gee), gee->icon, FALSE, FALSE, 0);
-				gtk_widget_show (gee->icon);
+				gtk_widget_show_all (gee->icon);
+				g_signal_connect (gee->icon, "clicked",
+						  G_CALLBACK (cb_icon_clicked), gee);
 			}
 		} else if (gee->icon != NULL)
 			gtk_object_destroy (GTK_OBJECT (gee->icon));
@@ -260,6 +391,7 @@ cb_gee_key_press_event (GtkEntry	  *entry,
 		Rangesel *rs = &gee->rangesel;
 		gboolean abs_cols = (gee->flags & GNM_EE_ABS_COL);
 		gboolean abs_rows = (gee->flags & GNM_EE_ABS_ROW);
+		gboolean c, r;
 
 		/* FIXME: since the range can't have changed we should just be able to */
 		/*        look it up rather than reparse */
@@ -272,21 +404,21 @@ cb_gee_key_press_event (GtkEntry	  *entry,
 		if (!rs->is_valid || rs->text_start >= rs->text_end)
 			return TRUE;
 
-		/* rows must be absolute */
+		c = rs->ref.a.col_relative;
+		r = rs->ref.a.row_relative;
 		if (abs_rows) {
 			if (abs_cols)
 				return TRUE;
-			rs->ref.b.col_relative = rs->ref.a.col_relative =
-				!rs->ref.a.col_relative;
-		} else if (abs_cols)
-			rs->ref.b.row_relative = rs->ref.a.row_relative =
-				!rs->ref.a.row_relative;
-		else {
-			/* It's late. I'm doing this the straightforward way. */
-			rs->ref.b.row_relative = rs->ref.a.row_relative =
-				(rs->ref.a.row_relative != rs->ref.a.col_relative);
-			rs->ref.b.col_relative = rs->ref.a.col_relative =
-				!rs->ref.a.col_relative;
+			gnm_cellref_set_row_ar (&rs->ref.a, &gee->pp, !c);
+			gnm_cellref_set_col_ar (&rs->ref.b, &gee->pp, !c);
+		} else if (abs_cols) {
+			gnm_cellref_set_row_ar (&rs->ref.a, &gee->pp, !r);
+			gnm_cellref_set_row_ar (&rs->ref.b, &gee->pp, !r);
+		} else {
+			gnm_cellref_set_col_ar (&rs->ref.a, &gee->pp, !c);
+			gnm_cellref_set_col_ar (&rs->ref.b, &gee->pp, !c);
+			gnm_cellref_set_row_ar (&rs->ref.a, &gee->pp, c^r);
+			gnm_cellref_set_row_ar (&rs->ref.b, &gee->pp, c^r);
 		}
 
 		gee_rangesel_update_text (gee);
@@ -340,6 +472,20 @@ cb_gee_key_press_event (GtkEntry	  *entry,
 		return TRUE;
 	}
 
+	case GDK_KP_Separator:
+	case GDK_KP_Decimal: {
+		GtkEditable *editable = GTK_EDITABLE (entry);
+		gint start, end, l;
+		GString const* s = format_get_decimal ();
+		gchar const* decimal = s->str;
+		l = s->len;
+		gtk_editable_get_selection_bounds (editable, &start, &end);
+		gtk_editable_delete_text (editable, start, end);
+		gtk_editable_insert_text (editable, decimal, l, &start);
+		gtk_editable_set_position (editable, start);
+		return TRUE;
+	}
+
 	default:
 		break;
 	}
@@ -387,6 +533,16 @@ gee_init (GnmExprEntry *gee)
 	gee_rangesel_reset (gee);
 
 	gee->entry = GTK_ENTRY (gtk_entry_new ());
+
+#ifdef USE_HILDON
+	g_object_set (G_OBJECT (gee->entry), "autocap", FALSE, NULL);
+#endif
+
+	/* Disable selecting the entire content when the widget gets focus */
+	g_object_set (gtk_widget_get_settings (GTK_WIDGET (gee->entry)),
+		      "gtk-entry-select-on-focus", FALSE,
+		      NULL);
+
 	g_signal_connect (G_OBJECT (gee->entry),
 		"activate",
 		G_CALLBACK (cb_entry_activate), gee);
@@ -505,8 +661,8 @@ gee_cell_editable_init (GtkCellEditableIface *iface)
 }
 
 GSF_CLASS_FULL (GnmExprEntry, gnm_expr_entry,
-		gee_class_init, gee_init,
-		GTK_TYPE_HBOX, 0,
+		NULL, NULL, gee_class_init, NULL,
+		gee_init, GTK_TYPE_HBOX, 0,
 		GSF_INTERFACE (gee_cell_editable_init, GTK_TYPE_CELL_EDITABLE))
 
 /**
@@ -884,7 +1040,7 @@ gnm_expr_entry_thaw (GnmExprEntry *gee)
 				break;
 		case GTK_UPDATE_CONTINUOUS:
 			g_signal_emit (G_OBJECT (gee), signals [UPDATE], 0, FALSE);
-		};
+		}
 	}
 }
 
@@ -994,8 +1150,8 @@ gnm_expr_entry_load_from_dep (GnmExprEntry *gee, GnmDependent const *dep)
 	/* We have nowhere to store the text while frozen. */
 	g_return_if_fail (gee->freeze_count == 0);
 
-	if (dep->expression != NULL) {
-		char *text = gnm_expr_as_string (dep->expression,
+	if (dep->texpr != NULL) {
+		char *text = gnm_expr_top_as_string (dep->texpr,
 			parse_pos_init_dep (&pp, dep), gnm_expr_conventions_default);
 
 		gee_rangesel_reset (gee);
@@ -1009,7 +1165,7 @@ gnm_expr_entry_load_from_dep (GnmExprEntry *gee, GnmDependent const *dep)
 /**
  * gnm_expr_entry_load_from_expr
  * @gee: a #GnmExprEntry
- * @expr: An expression
+ * @texpr: An expression
  * @pp  : The parse position
  *
  * Sets the text of the entry, and removes saved information about earlier
@@ -1017,14 +1173,15 @@ gnm_expr_entry_load_from_dep (GnmExprEntry *gee, GnmDependent const *dep)
  **/
 void
 gnm_expr_entry_load_from_expr (GnmExprEntry *gee,
-			       GnmExpr const *expr, GnmParsePos const *pp)
+			       GnmExprTop const *texpr,
+			       GnmParsePos const *pp)
 {
 	g_return_if_fail (IS_GNM_EXPR_ENTRY (gee));
 	/* We have nowhere to store the text while frozen. */
 	g_return_if_fail (gee->freeze_count == 0);
 
-	if (expr != NULL) {
-		char *text = gnm_expr_as_string (expr, pp,
+	if (texpr != NULL) {
+		char *text = gnm_expr_top_as_string (texpr, pp,
 				gnm_expr_conventions_default);
 		gee_rangesel_reset (gee);
 		gtk_entry_set_text (gee->entry, text);
@@ -1118,8 +1275,8 @@ gnm_expr_entry_get_rangesel (GnmExprEntry const *gee,
 
 	gee_prepare_range (gee, &ref);
 	if (r != NULL) {
-		cellref_get_abs_pos (&ref.a, &gee->pp.eval, &r->start);
-		cellref_get_abs_pos (&ref.b, &gee->pp.eval, &r->end);
+		gnm_cellpos_init_cellref (&r->start, &ref.a, &gee->pp.eval);
+		gnm_cellpos_init_cellref (&r->end, &ref.b, &gee->pp.eval);
 		range_normalize (r);
 	}
 
@@ -1208,14 +1365,14 @@ gnm_expr_entry_can_rangesel (GnmExprEntry *gee)
  * Attempts to parse the content of the entry line honouring
  * the flags.
  */
-GnmExpr const *
+GnmExprTop const *
 gnm_expr_entry_parse (GnmExprEntry *gee, GnmParsePos const *pp,
 		      GnmParseError *perr, gboolean start_sel,
 		      GnmExprParseFlags flags)
 {
 	char const *text;
 	char *str;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 
 	g_return_val_if_fail (IS_GNM_EXPR_ENTRY (gee), NULL);
 
@@ -1231,26 +1388,26 @@ gnm_expr_entry_parse (GnmExprEntry *gee, GnmParsePos const *pp,
 	if (!(gee->flags & GNM_EE_SHEET_OPTIONAL))
 		flags |= GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES;
 
-	expr = gnm_expr_parse_str (text, pp, flags, gnm_expr_conventions_default, perr);
-	if (expr == NULL)
+	texpr = gnm_expr_parse_str (text, pp, flags, NULL, perr);
+	if (texpr == NULL)
 		return NULL;
 
 	if (gee->flags & GNM_EE_SINGLE_RANGE) {
-		GnmValue *range = gnm_expr_get_range (expr) ;
+		GnmValue *range = gnm_expr_top_get_range (texpr);
 		if (range == NULL) {
 			if (perr != NULL) {
 				perr->err = g_error_new (1, PERR_SINGLE_RANGE,
 					_("Expecting a single range"));
 				perr->begin_char = perr->end_char   = 0;
 			}
-			gnm_expr_unref (expr);
+			gnm_expr_top_unref (texpr);
 			return NULL;
 		}
 		value_release (range);
 	}
 
 	/* Reset the entry in case something changed */
-	str = gnm_expr_as_string (expr, pp, gnm_expr_conventions_default);
+	str = gnm_expr_top_as_string (texpr, pp, gnm_expr_conventions_default);
 	if (strcmp (str, text)) {
 		SheetControlGUI *scg = wbcg_cur_scg (gee->wbcg);
 		Rangesel const *rs = &gee->rangesel;
@@ -1263,7 +1420,7 @@ gnm_expr_entry_parse (GnmExprEntry *gee, GnmParsePos const *pp,
 	}
 	g_free (str);
 
-	return expr;
+	return texpr;
 }
 
 /**
