@@ -35,8 +35,10 @@ void	  gnm_expr_entry_set_flags	(GnmExprEntry *e,
 					 GnmExprEntryFlags mask);
 void	  gnm_expr_entry_set_scg	(GnmExprEntry *e,
 					 SheetControlGUI *scg);
+void	  gnm_expr_entry_set_parsepos	(GnmExprEntry *e,
+					 GnmParsePos const *pp);
 GtkEntry *gnm_expr_entry_get_entry	(GnmExprEntry *e);
-gboolean  gnm_expr_entry_get_rangesel	(GnmExprEntry *e,
+gboolean  gnm_expr_entry_get_rangesel	(GnmExprEntry const *e,
 					 GnmRange *r, Sheet **sheet);
 void	  gnm_expr_expr_find_range	(GnmExprEntry *e);
 void	  gnm_expr_entry_rangesel_stop	(GnmExprEntry *e,
@@ -51,19 +53,19 @@ gboolean  gnm_expr_entry_is_cell_ref	(GnmExprEntry *e,
 char const *gnm_expr_entry_get_text	  (GnmExprEntry const *ee);
 GnmValue   *gnm_expr_entry_parse_as_value (GnmExprEntry *ee, Sheet *sheet);
 GSList	   *gnm_expr_entry_parse_as_list  (GnmExprEntry *ee, Sheet *sheet);
-GnmExpr const  *gnm_expr_entry_parse	  (GnmExprEntry *e,
-					   ParsePos const *pp,
-					   ParseError *perr, gboolean start_sel,
+GnmExprTop const *gnm_expr_entry_parse	  (GnmExprEntry *e,
+					   GnmParsePos const *pp,
+					   GnmParseError *perr, gboolean start_sel,
 					   GnmExprParseFlags flags);
 char    *gnm_expr_entry_global_range_name (GnmExprEntry *e, Sheet *sheet);
-void	 gnm_expr_entry_load_from_text	(GnmExprEntry *e, char const *str);
-void	 gnm_expr_entry_load_from_dep	(GnmExprEntry *e,
-					 Dependent const *dep);
-void	 gnm_expr_entry_load_from_expr	(GnmExprEntry *e,
-					 GnmExpr const *expr,
-					 ParsePos const *pp);
-gboolean gnm_expr_entry_load_from_range (GnmExprEntry *e,
-					 Sheet *sheet, GnmRange const *r);
+void	 gnm_expr_entry_load_from_text	  (GnmExprEntry *e, char const *str);
+void	 gnm_expr_entry_load_from_dep	  (GnmExprEntry *e,
+					   GnmDependent const *dep);
+void	 gnm_expr_entry_load_from_expr	  (GnmExprEntry *e,
+					   GnmExprTop const *texpr,
+					   GnmParsePos const *pp);
+gboolean gnm_expr_entry_load_from_range   (GnmExprEntry *e,
+					   Sheet *sheet, GnmRange const *r);
 
 void gnm_expr_entry_set_update_policy (GnmExprEntry *e,
 					    GtkUpdateType  policy);
