@@ -50,7 +50,7 @@
 
 #include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <errno.h>
 
@@ -486,10 +486,7 @@ workbook_cells (Workbook *wb, gboolean comments)
 
 	WORKBOOK_FOREACH_SHEET (wb, sheet, {
 		int oldlen = cells->len;
-		GPtrArray *scells =
-			sheet_cells (sheet,
-				     0, 0, SHEET_MAX_COLS, SHEET_MAX_ROWS,
-				     comments);
+		GPtrArray *scells = sheet_cells (sheet, comments);
 
 		g_ptr_array_set_size (cells, oldlen + scells->len);
 		memcpy (&g_ptr_array_index (cells, oldlen),
@@ -502,6 +499,28 @@ workbook_cells (Workbook *wb, gboolean comments)
 	return cells;
 }
 
+GnmExprSharer *
+workbook_share_expressions (Workbook *wb, gboolean freeit)
+{
+	GnmExprSharer *es = gnm_expr_sharer_new ();
+
+	WORKBOOK_FOREACH_DEPENDENT (wb, dep, {
+		if (dependent_is_cell (dep)) {
+			/* Hopefully safe, even when linked.  */
+			dep->texpr = gnm_expr_sharer_share (es, dep->texpr);
+		} else {
+			/* Not sure we want to touch this here.  */
+		}
+	});
+
+	if (freeit) {
+		gnm_expr_sharer_destroy (es);
+		es = NULL;
+	}
+
+	return es;
+}
+
 GSList *
 workbook_local_functions (Workbook const *wb)
 {
@@ -862,6 +881,8 @@ workbook_sheet_delete (Sheet *sheet)
         g_return_if_fail (IS_SHEET (sheet));
         g_return_if_fail (IS_WORKBOOK (sheet->workbook));
 
+	gnm_app_clipboard_invalidate_sheet (sheet);
+
 	wb = sheet->workbook;
 	sheet_index = sheet->index_in_wb;
 
@@ -944,11 +965,11 @@ workbook_sheet_move (Sheet *sheet, int direction)
  **/
 char *
 workbook_sheet_get_free_name (Workbook *wb,
-			      const char *base,
+			      char const *base,
 			      gboolean always_suffix,
 			      gboolean handle_counter)
 {
-	const char *name_format;
+	char const *name_format;
 	char *name, *base_name;
 	unsigned int i = 0;
 	int limit;
