@@ -9,7 +9,7 @@
  *   Jukka-Pekka Iivonen <jiivonen@hutcs.cs.hut.fi>
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "xml-io.h"
 #include "xml-sax.h"
@@ -81,9 +81,9 @@
 #define XML_INPUT_BUFFER_SIZE      4096
 #define N_ELEMENTS_BETWEEN_UPDATES 20
 
-#define CC2XML(s) ((const xmlChar *)(s))
+#define CC2XML(s) ((xmlChar const *)(s))
 #define C2XML(s) ((xmlChar *)(s))
-#define CXML2C(s) ((const char *)(s))
+#define CXML2C(s) ((char const *)(s))
 #define XML2C(s) ((char *)(s))
 
 /* ------------------------------------------------------------------------- */
@@ -577,7 +577,7 @@ xml_read_print_repeat_range (XmlParseContext *ctxt, xmlNodePtr tree,
 
 		if (s) {
 			GnmRange r;
-			if (parse_range (CXML2C (s), &r)) {
+			if (range_parse (&r, CXML2C (s))) {
 				range->range = r;
 				range->use   = TRUE;
 			}
@@ -1135,7 +1135,7 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	cell = sheet_cell_get (ctxt->sheet, col, row);
 	if ((is_new_cell = (cell == NULL)))
-		cell = sheet_cell_new (ctxt->sheet, col, row);
+		cell = sheet_cell_create (ctxt->sheet, col, row);
 	if (cell == NULL)
 		return NULL;
 
@@ -1450,7 +1450,7 @@ xml_read_sheet_filters (XmlParseContext *ctxt, xmlNode const *container)
 		area = xml_node_get_cstr (filter_node, "Area");
 		if (area == NULL)
 			continue;
-		if (parse_range (CXML2C (area), &r)) {
+		if (range_parse (&r, CXML2C (area))) {
 			filter = gnm_filter_new (ctxt->sheet, &r);
 			for (field = filter_node->xmlChildrenNode; field != NULL; field = field->next)
 				if (!xmlIsBlankNode (field))
@@ -1681,7 +1681,7 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 	tmp = (char *) xmlGetProp (tree, (xmlChar *)"ObjectBound");
 	if (tmp != NULL) {
 		GnmRange r;
-		if (parse_range (tmp, &r))
+		if (range_parse (&r, tmp))
 			so->anchor.cell_bound = r;
 		xmlFree (tmp);
 	}
@@ -1705,9 +1705,9 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 	}
 
 	if (xml_node_get_int (tree, "Direction", &tmp_int))
-		so->anchor.direction = tmp_int;
+		so->anchor.base.direction = tmp_int;
 	else
-		so->anchor.direction = SO_DIR_UNKNOWN;
+		so->anchor.base.direction = GOD_ANCHOR_DIR_UNKNOWN;
 
 	/* Do not assign to a sheet when extracting a cell region */
 	if (NULL != ctxt->sheet) {
@@ -1731,7 +1731,7 @@ xml_read_merged_regions (XmlParseContext const *ctxt, xmlNodePtr sheet)
 			xmlChar *content = xml_node_get_cstr (region, NULL);
 			GnmRange r;
 			if (content != NULL) {
-				if (parse_range (CXML2C (content), &r))
+				if (range_parse (&r, CXML2C (content)))
 					sheet_merge_add (ctxt->sheet, &r, FALSE, NULL);
 				xmlFree (content);
 			}
@@ -1755,91 +1755,77 @@ xml_read_styles (XmlParseContext *ctxt, xmlNodePtr tree)
 	}
 }
 
-/*
- * Create a ColRowInfo equivalent to the XML subtree of doc.
- */
-static int
+static void
 xml_read_colrow_info (XmlParseContext *ctxt, xmlNodePtr tree,
-		      ColRowInfo *info, double *size_pts)
+		      gboolean is_cols)
 {
-	int val, count;
+	ColRowInfo *cri;
+	double size_pts;
+	int pos, val;
+
+	if (!xml_node_get_int (tree, "No", &pos) ||
+	    NULL == (cri = sheet_colrow_fetch (ctxt->sheet, pos, is_cols)))
+		return;
 
-	info->size_pts = -1;
-	xml_node_get_int (tree, "No", &info->pos);
-	xml_node_get_double (tree, "Unit", size_pts);
-	if (xml_node_get_int (tree, "MarginA", &val))
-		info->margin_a = val;
-	if (xml_node_get_int (tree, "MarginB", &val))
-		info->margin_b = val;
 	if (xml_node_get_int (tree, "HardSize", &val))
-		info->hard_size = val;
+		cri->hard_size = val;
+	if (xml_node_get_double (tree, "Unit", &size_pts)) {
+		if (is_cols)
+			sheet_col_set_size_pts (ctxt->sheet,
+				pos, size_pts, cri->hard_size);
+		else
+			sheet_row_set_size_pts (ctxt->sheet,
+				pos, size_pts, cri->hard_size);
+	}
+
 	if (xml_node_get_int (tree, "Hidden", &val) && val)
-		info->visible = FALSE;
+		cri->visible = FALSE;
 	if (xml_node_get_int (tree, "Collapsed", &val) && val)
-		info->is_collapsed = TRUE;
+		cri->is_collapsed = TRUE;
 	if (xml_node_get_int (tree, "OutlineLevel", &val) && val > 0)
-		info->outline_level = val;
-	if (xml_node_get_int (tree, "Count", &count))
-		return count;
-	return 1;
+		cri->outline_level = val;
+
+	if (xml_node_get_int (tree, "Count", &val)) {
+		/* resize flags are already set only need to copy the sizes */
+		for ( ; --val > 0 ; )
+			colrow_copy (sheet_colrow_fetch (ctxt->sheet, ++pos, is_cols), cri);
+	}
 }
 
 static void
 xml_read_cols_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr cols, col;
-	double size_pts, tmp;
-	ColRowInfo *info;
-	int count, pos;
-	Sheet *sheet = ctxt->sheet;
+	double tmp;
 
 	cols = e_xml_get_child_by_name (tree, CC2XML ("Cols"));
 	if (cols == NULL)
 		return;
 
 	if (xml_node_get_double (cols, "DefaultSizePts", &tmp))
-		sheet_col_set_default_size_pts (sheet, tmp);
+		sheet_col_set_default_size_pts (ctxt->sheet, tmp);
 
 	for (col = cols->xmlChildrenNode; col; col = col->next)
-		if (!xmlIsBlankNode (col)) {
-			info = sheet_col_new (sheet);
-			count = xml_read_colrow_info (ctxt, col, info, &size_pts);
-			sheet_col_add (sheet, info);
-			sheet_col_set_size_pts (ctxt->sheet, info->pos, size_pts, info->hard_size);
-
-			/* resize flags are already set only need to copy the sizes */
-			for (pos = info->pos ; --count > 0 ; )
-				colrow_copy (sheet_col_fetch (ctxt->sheet, ++pos), info);
-		}
+		if (!xmlIsBlankNode (col))
+			xml_read_colrow_info (ctxt, col, TRUE);
 }
 
 static void
 xml_read_rows_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr rows, row;
-	double size_pts, tmp;
-	ColRowInfo *info;
-	int count, pos;
-	Sheet *sheet = ctxt->sheet;
+	double tmp;
 
 	rows = e_xml_get_child_by_name (tree, CC2XML ("Rows"));
 	if (rows == NULL)
 		return;
 
 	if (xml_node_get_double (rows, "DefaultSizePts", &tmp))
-		sheet_row_set_default_size_pts (sheet, tmp);
+		sheet_row_set_default_size_pts (ctxt->sheet, tmp);
 
 	for (row = rows->xmlChildrenNode; row; row = row->next)
-		if (!xmlIsBlankNode (row)) {
-			info = sheet_row_new (sheet);
-			count = xml_read_colrow_info (ctxt, row, info, &size_pts);
-			sheet_row_add (sheet, info);
-			sheet_row_set_size_pts (ctxt->sheet, info->pos, size_pts, info->hard_size);
-
-			/* resize flags are already set only need to copy the sizes */
-			for (pos = info->pos ; --count > 0 ; )
-				colrow_copy (sheet_row_fetch (ctxt->sheet, ++pos), info);
-		}
+		if (!xmlIsBlankNode (row))
+			xml_read_colrow_info (ctxt, row, FALSE);
 }
 
 static void
@@ -2058,8 +2044,10 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 						(value_new_string
 						 (gnm_expr_char_start_p
 						  (CXML2C (content))));
-					value_release (cc->val);
-					cc->val = value_new_empty ();
+					if (cc->val) {
+						value_release (cc->val);
+						cc->val = NULL;
+					}
 				}
 				g_ptr_array_add (ctxt->shared_exprs,
 						 (gpointer)cc->texpr);
@@ -2077,10 +2065,12 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 		} else {
 			g_warning ("XML-IO: Missing shared expression");
 		}
-		cc->val = value_new_empty ();
 	}
 	go_format_unref (value_fmt);
 
+	if (!cc->val)
+		cc->val = value_new_empty ();
+
 	cr->contents = g_slist_prepend (cr->contents, cc);
 }
 
@@ -2154,7 +2144,7 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 			if (!xmlIsBlankNode (l)) {
 				GnmRange r;
 				xmlChar *content = (char *)xmlNodeGetContent (l);
-				if (parse_range (CXML2C (content), &r))
+				if (range_parse (&r, CXML2C (content)))
 					cr->merged = g_slist_prepend (cr->merged,
 								      range_dup (&r));
 				xmlFree (content);
@@ -2444,7 +2434,7 @@ maybe_convert (GsfInput *input, gboolean quiet)
 	GString *buffer;
 	guint ui;
 	char *converted;
-	const char *encoding;
+	char const *encoding;
 	gboolean ok;
 
 	buf = gsf_input_read (input, strlen (noencheader), NULL);
@@ -2715,11 +2705,12 @@ xml_init (void)
 	xml_sax_prober.error      = (errorSAXFunc) xml_probe_problem;
 	xml_sax_prober.fatalError = (fatalErrorSAXFunc) xml_probe_problem;
 	xml_sax_prober.startElement = (startElementSAXFunc) xml_probe_start_element;
+#warning REMOVE for 2.0
 	go_file_opener_register (go_file_opener_new (
 		"Gnumeric_XmlIO:gnum_xml",
-		_("Gnumeric XML (*.gnumeric)"),
+		_("Gnumeric XML (*.gnumeric) Old slow importer"),
 		suffixes, mimes,
-		xml_probe, gnumeric_xml_read_workbook), 50);
+		xml_probe, gnumeric_xml_read_workbook), 40);
 
 	go_file_opener_register (go_file_opener_new (
 		"Gnumeric_XmlIO:xml_sax",
