@@ -4,7 +4,7 @@
  * xml-sax-write.c : export .gnumeric and the clipboard subset using a the sax
  * 			like wrappers in libgsf
  *
- * Copyright (C) 2003-20045Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -34,7 +34,6 @@
 #include <sheet.h>
 #include <sheet-view.h>
 #include <sheet-style.h>
-#include <summary.h>
 #include <style-color.h>
 #include <style-conditions.h>
 #include <expr.h>
@@ -62,6 +61,7 @@
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-output-gzip.h>
 #include <gsf/gsf-doc-meta-data.h>
+#include <gsf/gsf-opendoc-utils.h>
 #include <gsf/gsf-utils.h>
 #include <locale.h>
 
@@ -137,37 +137,10 @@ xml_write_attributes (GnmOutputXML *state)
 }
 
 static void
-xml_write_summary (GnmOutputXML *state)
+xml_write_meta_data (GnmOutputXML *state)
 {
-	SummaryInfo *summary_info = workbook_metadata (state->wb);
-	GList *items, *ptr;
-	SummaryItem *sit;
-
-	if (summary_info == NULL)
-		return;
-	items = summary_info_as_list (summary_info);
-	if (items == NULL)
-		return;
-
-	gsf_xml_out_start_element (state->output, GNM "Summary");
-	for (ptr = items ; ptr != NULL ; ptr = ptr->next) {
-		sit = items->data;
-		if (sit == NULL)
-			continue;
-		gsf_xml_out_start_element (state->output, GNM "Item");
-		gsf_xml_out_simple_element (state->output, GNM "name", sit->name);
-		if (sit->type == SUMMARY_INT) {
-			gsf_xml_out_simple_int_element (state->output,
-				GNM "val-int", sit->v.i);
-		} else {
-			char *text = summary_item_as_text (sit);
-			gsf_xml_out_simple_element (state->output, GNM "val-string", text);
-			g_free (text);
-		}
-		gsf_xml_out_end_element (state->output);	/* </Item> */
-	}
-	gsf_xml_out_end_element (state->output); /* </Summary> */
-	g_list_free (items);
+	gsf_opendoc_metadata_write (state->output, 
+		go_doc_get_meta_data (GO_DOC (state->wb)));
 }
 
 static void
@@ -259,7 +232,7 @@ xml_write_print_repeat_range (GnmOutputXML *state,
 	if (range->use) {
 		gsf_xml_out_start_element (state->output, name);
 		gsf_xml_out_add_cstr_unchecked (state->output, "value",
-			range_name (&range->range));
+			range_as_string (&range->range));
 		gsf_xml_out_end_element (state->output);
 	}
 }
@@ -391,10 +364,10 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 		gsf_xml_out_add_int (state->output, "Shade", gnm_style_get_pattern (style));
 	if (gnm_style_is_element_set (style, MSTYLE_INDENT))
 		gsf_xml_out_add_int (state->output, "Indent", gnm_style_get_indent (style));
-	if (gnm_style_is_element_set (style, MSTYLE_CONTENT_LOCKED))
-		gsf_xml_out_add_bool (state->output, "Locked", gnm_style_get_content_locked (style));
-	if (gnm_style_is_element_set (style, MSTYLE_CONTENT_HIDDEN))
-		gsf_xml_out_add_bool (state->output, "Hidden", gnm_style_get_content_hidden (style));
+	if (gnm_style_is_element_set (style, MSTYLE_CONTENTS_LOCKED))
+		gsf_xml_out_add_bool (state->output, "Locked", gnm_style_get_contents_locked (style));
+	if (gnm_style_is_element_set (style, MSTYLE_CONTENTS_HIDDEN))
+		gsf_xml_out_add_bool (state->output, "Hidden", gnm_style_get_contents_hidden (style));
 	if (gnm_style_is_element_set (style, MSTYLE_FONT_COLOR))
 		gnm_xml_out_add_color (state->output, "Fore", gnm_style_get_font_color (style));
 	if (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK))
@@ -412,7 +385,8 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 	    gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
 	    gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC) ||
 	    gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
-	    gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH)) {
+	    gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
+	    gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT)) {
 		char const *fontname;
 
 		gsf_xml_out_start_element (state->output, GNM "Font");
@@ -427,6 +401,8 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 			gsf_xml_out_add_int (state->output, "Underline", (int)gnm_style_get_font_uline (style));
 		if (gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH))
 			gsf_xml_out_add_int (state->output, "StrikeThrough", gnm_style_get_font_strike (style));
+		if (gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT))
+			gsf_xml_out_add_int (state->output, "Script", (int)gnm_style_get_font_script (style));
 
 		if (gnm_style_is_element_set (style, MSTYLE_FONT_NAME))
 			fontname = gnm_style_get_font_name (style);
@@ -474,13 +450,13 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 			gsf_xml_out_add_cstr (state->output, "Message", v->msg->str);
 
 		parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
-		if (v->expr[0] != NULL &&
-		    (tmp = gnm_expr_as_string (v->expr[0], &pp, state->exprconv)) != NULL) {
+		if (v->texpr[0] != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->texpr[0], &pp, state->exprconv)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 			g_free (tmp);
 		}
-		if (v->expr[1] != NULL &&
-		    (tmp = gnm_expr_as_string (v->expr[1], &pp, state->exprconv)) != NULL) {
+		if (v->texpr[1] != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->texpr[1], &pp, state->exprconv)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 			g_free (tmp);
 		}
@@ -507,13 +483,13 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 				gsf_xml_out_start_element (state->output, GNM "Condition");
 				gsf_xml_out_add_int (state->output, "Operator", cond->op);
 				parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
-				if (cond->expr[0] != NULL &&
-				    (tmp = gnm_expr_as_string (cond->expr[0], &pp, state->exprconv)) != NULL) {
+				if (cond->texpr[0] != NULL &&
+				    (tmp = gnm_expr_top_as_string (cond->texpr[0], &pp, state->exprconv)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 					g_free (tmp);
 				}
-				if (cond->expr[1] != NULL &&
-				    (tmp = gnm_expr_as_string (cond->expr[1], &pp, state->exprconv)) != NULL) {
+				if (cond->texpr[1] != NULL &&
+				    (tmp = gnm_expr_top_as_string (cond->texpr[1], &pp, state->exprconv)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 					g_free (tmp);
 				}
@@ -575,19 +551,19 @@ xml_write_styles (GnmOutputXML *state)
 
 typedef struct {
 	GnmOutputXML *state;
-	gboolean is_column;
-	ColRowInfo *previous;
-	int rle_count;
+	gboolean	is_column;
+	ColRowInfo const *prev;
+	int prev_pos, rle_count;
 } closure_write_colrow;
 
 static gboolean
-xml_write_colrow_info (ColRowInfo *info, closure_write_colrow *closure)
+xml_write_colrow_info (GnmColRowIter const *iter, closure_write_colrow *closure)
 {
-	ColRowInfo const *prev = closure->previous;
+	ColRowInfo const *prev = closure->prev;
 	GsfXMLOut *output = closure->state->output;
 
 	closure->rle_count++;
-	if (colrow_equal (prev, info))
+	if (NULL != iter && colrow_equal (prev, iter->cri))
 		return FALSE;
 
 	if (prev != NULL) {
@@ -596,10 +572,8 @@ xml_write_colrow_info (ColRowInfo *info, closure_write_colrow *closure)
 		else
 			gsf_xml_out_start_element (output, GNM "RowInfo");
 
-		gsf_xml_out_add_int (output, "No", prev->pos);
+		gsf_xml_out_add_int (output, "No", closure->prev_pos);
 		xml_out_add_points (output, "Unit", prev->size_pts);
-		gsf_xml_out_add_int (output, "MarginA", prev->margin_a);
-		gsf_xml_out_add_int (output, "MarginB", prev->margin_b);
 		if (prev->hard_size)
 			gsf_xml_out_add_bool (output, "HardSize", TRUE);
 		if (!prev->visible)
@@ -615,7 +589,10 @@ xml_write_colrow_info (ColRowInfo *info, closure_write_colrow *closure)
 	}
 
 	closure->rle_count = 0;
-	closure->previous = info;
+	if (NULL != iter) {
+		closure->prev = iter->cri;
+		closure->prev_pos = iter->pos;
+	}
 
 	return FALSE;
 }
@@ -629,7 +606,8 @@ xml_write_cols_rows (GnmOutputXML *state)
 		sheet_col_get_default_size_pts (state->sheet));
 	closure.state = state;
 	closure.is_column = TRUE;
-	closure.previous = NULL;
+	closure.prev = NULL;
+	closure.prev_pos = -1;
 	closure.rle_count = 0;
 	colrow_foreach (&state->sheet->cols, 0, SHEET_MAX_COLS-1,
 		(ColRowHandler)&xml_write_colrow_info, &closure);
@@ -641,7 +619,8 @@ xml_write_cols_rows (GnmOutputXML *state)
 		sheet_row_get_default_size_pts (state->sheet));
 	closure.state = state;
 	closure.is_column = FALSE;
-	closure.previous = NULL;
+	closure.prev = NULL;
+	closure.prev_pos = -1;
 	closure.rle_count = 0;
 	colrow_foreach (&state->sheet->rows, 0, SHEET_MAX_ROWS-1,
 		(ColRowHandler)&xml_write_colrow_info, &closure);
@@ -652,8 +631,8 @@ xml_write_cols_rows (GnmOutputXML *state)
 static void
 xml_write_selection_info (GnmOutputXML *state)
 {
-	GList *ptr, *copy;
-	const SheetView *sv = sheet_get_view (state->sheet, state->wb_view);
+	GSList *ptr, *copy;
+	SheetView const *sv = sheet_get_view (state->sheet, state->wb_view);
 	if (!sv) return;  /* Hidden.  */
 
 	gsf_xml_out_start_element (state->output, GNM "Selections");
@@ -661,44 +640,43 @@ xml_write_selection_info (GnmOutputXML *state)
 	gsf_xml_out_add_int (state->output, "CursorRow", sv->edit_pos_real.row);
 
 	/* Insert the selections in REVERSE order */
-	copy = g_list_copy (sv->selections);
-	ptr = g_list_reverse (copy);
+	copy = g_slist_copy (sv->selections);
+	ptr = g_slist_reverse (copy);
 	for (; ptr != NULL ; ptr = ptr->next) {
 		GnmRange const *r = ptr->data;
 		gsf_xml_out_start_element (state->output, GNM "Selection");
 		xml_out_add_range (state->output, r);
 		gsf_xml_out_end_element (state->output); /* </gnm:Selection> */
 	}
-	g_list_free (copy);
+	g_slist_free (copy);
 
 	gsf_xml_out_end_element (state->output); /* </gnm:Selections> */
 }
 
 static void
 xml_write_cell_and_position (GnmOutputXML *state,
-			     GnmExpr const *expr, GnmValue const *val,
+			     GnmExprTop const *texpr, GnmValue const *val,
 			     GnmParsePos const *pp)
 {
-	GnmExprArray const *ar = NULL;
 	gboolean write_contents = TRUE;
-	gboolean const is_shared_expr = (expr != NULL) &&
-		gnm_expr_is_shared (expr);
+	gboolean const is_shared_expr = (texpr != NULL) &&
+		gnm_expr_top_is_shared (texpr);
 
 	/* Only the top left corner of an array needs to be saved (>= 0.53) */
-	if (expr && NULL != (ar = gnm_expr_is_array (expr)) && (ar->y != 0 || ar->x != 0))
+	if (texpr && gnm_expr_top_is_array_elem (texpr))
 		return; /* DOM version would write <Cell Col= Row=/> */
 
 	gsf_xml_out_start_element (state->output, GNM "Cell");
-	gsf_xml_out_add_int (state->output, "Col", pp->eval.col);
 	gsf_xml_out_add_int (state->output, "Row", pp->eval.row);
+	gsf_xml_out_add_int (state->output, "Col", pp->eval.col);
 
 	/* As of version 0.53 we save the ID of shared expressions */
 	if (is_shared_expr) {
-		gpointer id = g_hash_table_lookup (state->expr_map, (gpointer) expr);
+		gpointer id = g_hash_table_lookup (state->expr_map, (gpointer) texpr);
 
 		if (id == NULL) {
 			id = GINT_TO_POINTER (g_hash_table_size (state->expr_map) + 1);
-			g_hash_table_insert (state->expr_map, (gpointer)expr, id);
+			g_hash_table_insert (state->expr_map, (gpointer)texpr, id);
 		} else
 			write_contents = FALSE;
 
@@ -707,15 +685,15 @@ xml_write_cell_and_position (GnmOutputXML *state,
 
 	/* As of version 0.53 we save the size of the array as attributes */
 	/* As of version 0.57 the attributes are in the Cell not the Content */
-	if (ar != NULL) {
-	        gsf_xml_out_add_int (state->output, "Rows", ar->rows);
-	        gsf_xml_out_add_int (state->output, "Cols", ar->cols);
+	if (texpr && gnm_expr_top_get_array_corner (texpr)) {
+	        gsf_xml_out_add_int (state->output, "Rows", texpr->expr->array_corner.rows);
+	        gsf_xml_out_add_int (state->output, "Cols", texpr->expr->array_corner.cols);
 	}
 
 	if (write_contents) {
 		GString *str = g_string_sized_new (1000);
 
-		if (NULL == expr) {
+		if (!texpr) {
 			if (val != NULL) {
 				gsf_xml_out_add_int (state->output, "ValueType", val->type);
 				if (VALUE_FMT (val) != NULL) {
@@ -729,7 +707,7 @@ xml_write_cell_and_position (GnmOutputXML *state,
 			}
 		} else {
 			g_string_append_c (str, '=');
-			gnm_expr_as_gstring (str, expr, pp, state->exprconv);
+			gnm_expr_top_as_gstring (str, texpr, pp, state->exprconv);
 		}
 
 		gsf_xml_out_add_cstr (state->output, NULL, str->str);
@@ -739,11 +717,10 @@ xml_write_cell_and_position (GnmOutputXML *state,
 }
 
 static GnmValue *
-cb_write_cell (Sheet *sheet, int col, int row, GnmCell const *cell, GnmOutputXML *state)
+cb_write_cell (GnmCellIter const *iter, GnmOutputXML *state)
 {
-	GnmParsePos pp;
-	xml_write_cell_and_position (state, cell->base.expression, cell->value,
-		parse_pos_init_cell (&pp, cell));
+	xml_write_cell_and_position (state,
+		iter->cell->base.texpr, iter->cell->value, &iter->pp);
 	return NULL;
 }
 
@@ -766,7 +743,7 @@ xml_write_merged_regions (GnmOutputXML *state)
 	gsf_xml_out_start_element (state->output, GNM "MergedRegions");
 	for (; ptr != NULL ; ptr = ptr->next)
 		gsf_xml_out_simple_element (state->output,
-			GNM "Merge", range_name (ptr->data));
+			GNM "Merge", range_as_string (ptr->data));
 	gsf_xml_out_end_element (state->output); /* </gnm:MergedRegions> */
 }
 
@@ -860,7 +837,7 @@ xml_write_sheet_filters (GnmOutputXML *state)
 		filter = ptr->data;
 		gsf_xml_out_start_element (state->output, GNM "Filter");
 		gsf_xml_out_add_cstr_unchecked (state->output, "Area",
-			range_name (&filter->r));
+			range_as_string (&filter->r));
 
 		for (i = filter->fields->len ; i-- > 0 ; ) {
 			cond = gnm_filter_get_condition (filter, i);
@@ -949,12 +926,7 @@ xml_write_solver (GnmOutputXML *state)
 static void
 xml_write_scenarios (GnmOutputXML *state)
 {
-	scenario_t const *s;
 	GList   *ptr;
-#if 0
-	GString  *name;
-	int       i, cols, rows;
-#endif
 
 	if (state->sheet->scenarios == NULL)
 		return;
@@ -962,7 +934,10 @@ xml_write_scenarios (GnmOutputXML *state)
 	gsf_xml_out_start_element (state->output, GNM "Scenarios");
 
 	for (ptr = state->sheet->scenarios ; ptr != NULL ; ptr = ptr->next) {
-	        s = (scenario_t const *)ptr->data;
+	        scenario_t const *s = (scenario_t const *)ptr->data;
+#if 0
+		int       i, cols, rows;
+#endif
 
 		gsf_xml_out_start_element (state->output, GNM "Scenario");
 		gsf_xml_out_add_cstr (state->output, "Name", s->name);
@@ -978,13 +953,15 @@ xml_write_scenarios (GnmOutputXML *state)
 		rows = range_height (&s->range);
 		cols = range_width (&s->range);
 		for (i = 0; i < cols * rows; i++) {
-			name = g_string_new (NULL);
+			GString  *name = g_string_new (NULL);
 			g_string_append_printf (name, "V%d", i);
 			xml_node_set_value (scen, name->str,
 					    s->changing_cells [i]);
-			g_string_free (name, FALSE);
+			g_string_free (name, TRUE);
 		}
 #endif
+
+		gsf_xml_out_end_element (state->output); /* </gnm:Scenario> */
 	}
 
 	gsf_xml_out_end_element (state->output); /* </gnm:Scenarios> */
@@ -1018,7 +995,7 @@ xml_write_objects (GnmOutputXML *state, GSList *ptr)
 
 		tmp = g_strconcat (GNM, type_name, NULL);
 		gsf_xml_out_start_element (state->output, tmp);
-		gsf_xml_out_add_cstr (state->output, "ObjectBound", range_name (&so->anchor.cell_bound));
+		gsf_xml_out_add_cstr (state->output, "ObjectBound", range_as_string (&so->anchor.cell_bound));
 		snprintf (buffer, sizeof (buffer), "%.3g %.3g %.3g %.3g",
 			  so->anchor.offset [0], so->anchor.offset [1],
 			  so->anchor.offset [2], so->anchor.offset [3]);
@@ -1029,7 +1006,7 @@ xml_write_objects (GnmOutputXML *state, GSList *ptr)
 		gsf_xml_out_add_cstr (state->output, "ObjectAnchorType", buffer);
 
 		gsf_xml_out_add_int (state->output, "Direction",
-			so->anchor.direction);
+			so->anchor.base.direction);
 
 		(*klass->write_xml_sax) (so, state->output);
 
@@ -1066,6 +1043,9 @@ xml_write_sheet (GnmOutputXML *state, Sheet const *sheet)
 	if (sheet->text_is_rtl)
 		gsf_xml_out_add_bool (state->output,
 			"RTL_Layout", sheet->text_is_rtl);
+	if (sheet->is_protected)
+		gsf_xml_out_add_bool (state->output,
+			"Protected", sheet->is_protected);
 	gsf_xml_out_add_enum (state->output,
 		"Visibility", GNM_SHEET_VISIBILITY_TYPE, sheet->visibility);
 
@@ -1141,10 +1121,10 @@ xml_io_conventions (void)
 	GnmExprConventions *res = gnm_expr_conventions_new ();
 
 	res->decimal_sep_dot = TRUE;
-	res->ref_parser = gnm_1_0_rangeref_parse;
+	res->ref_parser = rangeref_parse;
+	res->range_ref_handler = gnm_1_0_rangeref_as_string;
 	res->range_sep_colon = TRUE;
 	res->sheet_sep_exclamation = TRUE;
-	res->dots_in_names = TRUE;
 	res->output_sheet_name_sep = "!";
 	res->output_argument_sep  = ",";
 	res->output_array_col_sep = ",";
@@ -1161,12 +1141,14 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 {
 	GnmOutputXML state;
 	char *old_num_locale, *old_monetary_locale;
-	char const *extension = gsf_extension_pointer (gsf_output_name (output));
+	char const *extension = NULL;
 	GsfOutput *gzout = NULL;
 	gboolean compress;
 
 	/* If the suffix is .xml disable compression */
-	if (extension && g_ascii_strcasecmp (extension, "xml") == 0)
+	if (NULL != gsf_output_name (output))
+		extension = gsf_extension_pointer (gsf_output_name (output));
+	if (NULL != extension && g_ascii_strcasecmp (extension, "xml") == 0)
 		compress = FALSE;
 	else 
 		compress = (gnm_app_prefs->xml_compression_level > 0);
@@ -1177,7 +1159,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 	}
 
 	state.wb_view	= wb_view;
-	state.wb	= wb_view_workbook (wb_view);
+	state.wb	= wb_view_get_workbook (wb_view);
 	state.sheet	= NULL;
 	state.output	= gsf_xml_out_new (output);
 	state.exprconv	= xml_io_conventions ();
@@ -1199,7 +1181,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 
 	xml_write_version	    (&state);
 	xml_write_attributes	    (&state);
-	xml_write_summary	    (&state);
+	xml_write_meta_data	    (&state);
 	xml_write_conventions	    (&state);
 	xml_write_sheet_names	    (&state);
 	xml_write_named_expressions (&state, state.wb->names);
@@ -1271,7 +1253,7 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 	gsf_xml_out_add_int (state.output, "Rows", cr->rows);
 	gsf_xml_out_add_int (state.output, "BaseCol", cr->base.col);
 	gsf_xml_out_add_int (state.output, "BaseRow", cr->base.row);
-	if (cr->not_as_content)
+	if (cr->not_as_contents)
 		gsf_xml_out_add_bool (state.output, "NotAsContent", TRUE);
 
 	if (cr->styles != NULL) {
@@ -1286,20 +1268,20 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 		for (ptr = cr->merged ; ptr != NULL ; ptr = ptr->next) {
 			gsf_xml_out_start_element (state.output, GNM "Merge");
 			gsf_xml_out_add_cstr_unchecked (state.output, NULL,
-				range_name (ptr->data));
+				range_as_string (ptr->data));
 			gsf_xml_out_end_element (state.output); /* </Merge> */
 		}
 	}
 
 	pp.wb = NULL; /* NOTE SNEAKY : ensure that sheet names have explicit workbooks */
 	pp.sheet = cr->origin_sheet;
-	if (cr->content != NULL) {
+	if (cr->contents != NULL) {
 		gsf_xml_out_start_element (state.output, GNM "Cells");
-		for (ptr = cr->content; ptr != NULL ; ptr = ptr->next) {
+		for (ptr = cr->contents; ptr != NULL ; ptr = ptr->next) {
 			cc = ptr->data;
 			pp.eval.col = cr->base.col + cc->col_offset,
 			pp.eval.row = cr->base.row + cc->row_offset;
-			xml_write_cell_and_position (&state, cc->expr, cc->val, &pp);
+			xml_write_cell_and_position (&state, cc->texpr, cc->val, &pp);
 		}
 		gsf_xml_out_end_element (state.output); /* </Cells> */
 	}
